MODULE DiaryModule                                                   ! Compile after modules in USE
   ! Latest revisions by MCN on December 6, 2011
   USE GlobalModule                                                  ! Compile this module first
   USE UtilityModule                                                 ! Compile this module second
   USE DistributionModule                                            ! Compile this module third 
   USE ProfileModule                                                 ! Compile this module fourth 
   IMPLICIT NONE                                                     ! Must declare all variables
  
   TYPE :: OCCUPTYPE                                                 ! Occupation used to assign METS
      INTEGER  (KIND=BYT) :: Row                                     ! Occupation number
      CHARACTER(LEN=LEN1) :: OccDesc                                 ! Occupation description
   END TYPE OCCUPTYPE                                                ! Used to declare Occups
 
   TYPE :: DIARYSUMMARY                                              ! Stores diary questionnaire data
      INTEGER (KIND=IK)  :: Pool        = 0                          ! Pool # for diary 
 	  INTEGER (KIND=IK)  :: Cluster     = 0                          ! Cluster # for diary       
      INTEGER (KIND=IK)  :: Index       = 0                          ! # of diary within the pool 
      INTEGER (KIND=BYT) :: Age         = 0                          ! Age of respondent (127 max)
      INTEGER (KIND=BYT) :: MaxTemp     = 0                          ! Max temperature (F) on diary day
      INTEGER (KIND=BYT) :: AvgTemp     = 0                          ! Avg temperature (F) on diary day 
      INTEGER (KIND=BYT) :: Gender      = 0                          ! Gender (1=male, 2=female)
      INTEGER (KIND=BYT) :: Race        = 0                          ! Race (list is in PROFILE)
      INTEGER (KIND=BYT) :: Employed    = 0                          ! Employment status (1=YES, 2=NO)\
      CHARACTER(LEN=LEN1):: Occupation  = ""                         ! Occupation name
      INTEGER (KIND=BYT) :: WeekDay     = 0                          ! Day of week (1=Sunday, 7=Saturday)
      INTEGER (KIND=IK)  :: FirstEvent  = 0                          ! First record # in DData 
      INTEGER (KIND=IK)  :: LastEvent   = 0                          ! Last record # in DData 
      INTEGER (KIND=IK)  :: Selected    = 0                          ! Diary selected for use flag
      REAL    (KIND=RK)  :: Probability = 0.0                        ! Probability of selecting diary
      REAL    (KIND=RK)  :: Statistic   = 0.0                        ! Diary stat for longit assembly
      CHARACTER (LEN=9)  :: DiaryID     = ""                         ! CHADID for each diary
      INTEGER (KIND=IK)  :: DCommTime    = 0                         ! vehicle time commuting hours      
   END TYPE DIARYSUMMARY                                             ! Used to declare QData

   TYPE :: DIARYEVENTS                                               ! Stores diary event data
      INTEGER (KIND=BYT) :: Hour     = 0                             ! Hour of day event start (0-23)
      INTEGER (KIND=BYT) :: Duration = 0                             ! Duration in minutes (1-60)
      INTEGER (KIND=BYT) :: Activity = 0                             ! Activity code (may be negative)
      INTEGER (KIND=BYT) :: Location = 0                             ! Location code (0-115)
      INTEGER (KIND=BYT) :: Location2 = 0                            ! Location code (0-115)       
      INTEGER (KIND=BYT) :: HomeWork = 0                             ! Place (1=H, 2=W, 3=O, 4=R, 5=NH, 6=NW, 7=RW)
      INTEGER (KIND=IK)  :: ActCode = 0                              ! CHAD Activity code
   END TYPE DIARYEVENTS                                              ! Used to declare DData

   INTEGER (KIND=IK), PARAMETER :: MAXOCCUPS=16                      ! Max # of occupation categories
   INTEGER(KIND=IK), PARAMETER	:: NumCluster=3						 ! Max # of cluster   
   INTEGER (KIND=IK)            :: NumOccups=0                       ! # of occupation categories used 
   TYPE (OCCUPTYPE)             :: Occups(MAXOCCUPS)                 ! Vector of type occupation
   INTEGER (KIND=BYT)           :: CHADAct(10300:18940,3) = 0        ! Mapping for CHAD activities
   INTEGER (KIND=BYT)           :: CHADLoc(30000:36500)   = -99      ! Mapping for CHAD locations
   INTEGER (KIND=BYT)           :: CHADLocC(30000:36500)  = -99      ! Mapping for CHAD locations for clustering   
   CHARACTER (LEN=1)            :: APEXLoc(30000:36500)   =  ""      ! Mapping for H/W/O/R/NH/NR locations
   INTEGER (KIND=IK),   ALLOCATABLE :: NumDiaries(:)                 ! # diaries per profile and pool     
   INTEGER (KIND=IK),   ALLOCATABLE :: PCount(:)                     ! Number of diaries per pool
   INTEGER (KIND=IK),   ALLOCATABLE :: PDCount(:,:,:,:)              ! Number of diaries per pool
   INTEGER (KIND=IK),   ALLOCATABLE :: PDCount1(:,:,:,:)			 ! xh1
   INTEGER (KIND=IK),   ALLOCATABLE :: AGEGRPMIN(:),AGEGRPMAX(:)     ! Min, Max Age groups for clustering
   INTEGER (KIND=IK),   ALLOCATABLE :: DiSelIDM(:,:)				 ! Diary sequence# for each pool and cluster
   INTEGER (KIND=IK),   ALLOCATABLE :: GroupMETS(:)                  ! METS Dists for occ groups
   TYPE (DIARYSUMMARY), ALLOCATABLE :: QData(:,:)                    ! Diary questionnaire data
   TYPE (DIARYSUMMARY), ALLOCATABLE :: QCData(:,:)                   ! Diary questionnaire data for clustering
   TYPE (DIARYSUMMARY), ALLOCATABLE :: QCDData(:,:,:,:,:)            ! xh1  ! Diary questionnaire data for clustering1
   TYPE (DIARYSUMMARY), ALLOCATABLE :: QCDData1(:,:,:,:)             ! Diary questionnaire data for clustering1
   TYPE (DIARYSUMMARY), ALLOCATABLE :: QCDCData(:,:,:,:,:,:)         ! xh1 Diary questionnaire data for clustering1
   TYPE (DIARYSUMMARY), ALLOCATABLE :: QSData(:,:)                   ! Diary questionnaire data for clustering1   
   TYPE (DIARYEVENTS),  ALLOCATABLE :: DData(:)                      ! Diary event data 
   INTEGER (KIND=IK)                :: ActCount(-128:127) = 0        ! Count for each activity code
   INTEGER (KIND=IK)                :: ActTime(-128:127)  = 0        ! Minutes for each activity code
   INTEGER (KIND=IK)                :: NumX                          ! Num picks for longit diaries
   REAL    (KIND=RK),   ALLOCATABLE :: DiaryRank(:),XPicks(:)        ! Real array for longit. assembly
   REAL    (KIND=LONG), ALLOCATABLE :: Rn(:)                         ! Long real array for randoms
   REAL    (KIND=LONG), ALLOCATABLE :: ClusterTransM (:,:,:,:)		 ! Cluster Transition Probability from clustering   	
   REAL    (KIND=LONG), ALLOCATABLE :: ClusterTransOut (:,:,:,:,:,:,:) ! xh1Cluster Transition Probability from clustering   	    
   REAL    (KIND=LONG), ALLOCATABLE :: ClusterPRM (:,:)				 ! Cluster Frequency distribution		
   REAL    (KIND=LONG), ALLOCATABLE :: ClusterPRMOUT (:,:,:,:,:)	 ! Cluster Frequency distribution		
   INTEGER (KIND=IK),   ALLOCATABLE :: NUM2DIARY(:,:)                ! Cluster something or other
   INTEGER (KIND=IK),   ALLOCATABLE :: AgeDays(:,:)                  ! # of days for each age Male 
   INTEGER (KIND=IK),   ALLOCATABLE :: ClusterNoOut(:,:)             ! Cluster distribution
   INTEGER (KIND=IK),   ALLOCATABLE :: ClusterNoOutW(:,:,:,:,:)      ! Cluster distribution
   INTEGER (KIND=IK),   ALLOCATABLE :: ClusterIDOut(:,:,:)           ! Clustering CHAD IDs
   INTEGER (KIND=IK),   ALLOCATABLE :: ClusterIDOutW(:,:,:,:,:)      ! Clustering CHAD IDs
   INTEGER (KIND=IK),   ALLOCATABLE :: Picked(:)                     ! Int. array for longit. assembly
   TYPE    (CDIST)                  :: MetsDist(-128:127)            ! Mets distribution definitions
CONTAINS                                                             ! Routines internal to this module


SUBROUTINE ReadDiaries                                               ! Called from main APEX module
   INTEGER (KIND=IK), PARAMETER :: MAXDIARIES=2**15, MAXEVENTS=2**22 ! Maximum space for diary data
   CHARACTER(LEN=LEN1) :: Occup                                      ! Buffer for reading occupation
   CHARACTER(LEN=LEN2) :: DName                                      ! Buffer for distribution name
   CHARACTER(LEN=9)    :: DID="", QID, QID2, QID3                    ! Diary and questionnaire IDs
   CHARACTER(LEN=5)    :: ActCode, LocCode                           ! CHAD activity & location codes
   CHARACTER(LEN=3)    :: CWkDay                                     ! Weekday ('MON'-'SUN')
   CHARACTER(LEN=1)    :: CEmp, CSex, CRace                          ! Local character variables
   REAL    (KIND=RK)   :: CAge, CMaxT, CAvgT, Stat=0                 ! CHAD age, temps, and stat
   INTEGER (KIND=IK)   :: CComTime                                   ! CHAD commuting time   
   REAL    (KIND=RK)   :: Val1, Val2, NumExtras                      ! Local real variables   
   INTEGER (KIND=IK)   :: K, JJ, KK, L, M, MM, NN, B, C              ! Local integer variables
   INTEGER (KIND=BYT)  :: MM1,MM2, MM3Age,JJ2,K2                     ! Local integer variables
   INTEGER (KIND=IK)   :: MAge,AgeMMM,MM33,MM3                       ! Local integer variables
   INTEGER (KIND=IK)   :: MaxAge1 = 100                              ! Local integer variables
   INTEGER (KIND=BYT)  :: AgeWindow1                                 ! Local integer variables
   INTEGER (KIND=IK)   :: Age, AgeCat, Pool, PoolVec(1), RecCt       ! Local integer variables
   INTEGER (KIND=IK)   :: AgeSum, AgeMiss, QCMiss, I, A              ! More local integer variables 
   INTEGER (KIND=IK)   :: STime, LocInt, ActInt, Discard=0           ! Even more integer variables
   INTEGER (KIND=IK)   :: LastEvent=0, Dur=0, FirstEvent, DiaryNum=0 ! Still more integer variables
   INTEGER (KIND=IK)   :: QGender, QRace, QEmployed, QWeekDay        ! CHAD questionnaire variables
   CHARACTER(LEN=LEN1) :: QOccupation                                ! Occupation variable
   INTEGER (KIND=IK), TARGET  :: QMaxTemp(1),QAvgTemp(1),QDayOfWk(1) ! Targets for the pointers below
   INTEGER (KIND=IK), POINTER :: MT(:), AT(:), WK(:)                 ! Pointers to temper. and weekend
   TYPE (DIARYSUMMARY)  :: T                                         ! Local diary questionnaire data
   LOGICAL exists
   ! Global constants: ChQuU, ChEvU, IK, LEN1, LogU, LV_FMT, LV_MSG, RK, ST_FMT, ST_MSG
   ! Global vars used: DebugLevel, FName, NumPools
   ! Global vars set : Ios, ProcLevel, ProcName
   ! Module vars used: CHADAct, CHADLoc, Occups, DiaryRank, XPicks, Picked, NumX 
   ! Module vars set : ActCount, ActTime, DData, NumDiaries, PCount, QData, QCData
   ! Intrinsic procs : ALLOCATE, ANY, CLOSE, COUNT, INDEX, MOD, PRINT, READ, REAL, WRITE
   ! APEX procedures : CheckAllocation, Distrib, Fatal, OpenInput, ReadMicroMap, ReadMETS
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "ReadDiaries"                               ! Procedure name for messages
   IF (DebugLevel>0) WRITE(LogU,ST_FMT) ST_MSG, ProcName(ProcLevel)  ! Message for debugging
   ALLOCATE (QData(MAXDIARIES,NumPools),STAT=AllocErr)               ! Make space for summary data  
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc       
   IF (CLUSTDIARYA==YES.OR.CLUSTDIARYB==YES) THEN                    ! Allocating cluster arrays  
      ALLOCATE (QCDCData(350,3,NumPools,3,3,0:MaxAge1),STAT=AllocErr) ! Make space for clustering data
      CALL CheckAllocation(AllocErr)                                 ! Check the success of the alloc  
      ALLOCATE (ClusterTransOut(NumPools,3,NumPools,3,3,3,0:MaxAge1+1),STAT=AllocErr) ! Make space for Cluster array
      CALL CheckAllocation(AllocErr)                                 ! Check the success of the alloc
      ALLOCATE (ClusterNoOutW(3,NumPools,3,3,0:MaxAge1+1),STAT=AllocErr) ! Make Space for clustering arrays
      CALL CheckAllocation(AllocErr)                                 ! Check the success of the alloc      
      ALLOCATE (ClusterIDoutW(3,NumPools,MAXDIARIES,3,3),STAT=AllocErr) ! Make Space for clustering arrays
      CALL CheckAllocation(AllocErr)                                 ! Check the success of the alloc   
      ALLOCATE (ClusterPRMOUT(NumPools,3,3,3,0:MaxAge1+1),STAT=AllocErr) ! Make space for frequency dis
      CALL CheckAllocation(AllocErr)                                 ! Check the success of the alloc   
      ALLOCATE (NUM2DIARY(3,NumPools),STAT=AllocErr)                 ! Make space for NUM2DIARY array
      CALL CheckAllocation(AllocErr)                                 ! Check the success of the alloc            
      ALLOCATE (QSData(NumCluster,NumPools),STAT=AllocErr)           ! Make space for clustering data
      CALL CheckAllocation(AllocErr)                                 ! Check the success of the alloc     
      ALLOCATE (ClusterPRM(NumPools,3),STAT=AllocErr)                ! Make space for frequency dis.
      CALL CheckAllocation(AllocErr)                                 ! Check the success of the alloc   
      ALLOCATE (ClusterTransM(NumPools,3,NumPools,3),STAT=AllocErr)  ! Make space for cluster transition probability
      CALL CheckAllocation(AllocErr)                                 ! Check the success of the alloc
   ENDIF                                                             ! End allocating clustering arrays   
   IF (CLUSTDIARYB==YES) THEN                                        ! Allocating cluster arrays  
      ALLOCATE (QCData(2000,NumPools),STAT=AllocErr)                 ! Make space for clustering data    
      CALL CheckAllocation(AllocErr)                                 ! Check the success of the alloc       
      ALLOCATE (QCDData(2000,NumPools,3,3,0:MaxAge1),STAT=AllocErr)  ! Make space for clustering data
      CALL CheckAllocation(AllocErr)                                 ! Check the success of the alloc    
      ALLOCATE (QCDData1(4000,NumPools,3,3),STAT=AllocErr)           ! Make space for clustering data
      CALL CheckAllocation(AllocErr)                                 ! Check the success of the alloc      
      ALLOCATE (ClusterNoOut(3,NumPools), ClusterIDout(3,NumPools,MAXDIARIES),STAT=AllocErr) ! Make Space for clustering arrays
      CALL CheckAllocation(AllocErr)                                 ! Check the success of the alloc   
   ENDIF                                                             ! End allocating clustering arrays      
   ALLOCATE (DData(MAXEVENTS),STAT=AllocErr)                         ! Make space for diary events 
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc  
   ALLOCATE (PCount(NumPools),STAT=AllocErr)                         ! Make space for pool counts
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc    
   ALLOCATE (NumDiaries(NumPools),STAT=AllocErr)                     ! Make space for diary counts 
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (AgeDays(4*NumPools,100),STAT=AllocErr)                  ! Make space for age days
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc      
   CALL ReadMicroMap                                                 ! Read CHAD-APEX location mapping
   IF (CLUSTDIARYB==YES.OR.CLUSTDIARYA==YES) CALL ReadClusterMicroMap ! Read CHAD-APEX location mapping	   
   CALL ReadMETS                                                     ! Read METS distributions
   PCount = 0                                                        ! Initialize pool counts to zero
   CALL OpenInput(ChQuU,h=NO)                                        ! Open CHAD questionnaire file
   CALL OpenInput(ChEvU,h=NO)                                        ! Open CHAD diary event file
   IF (LongitDiary==YES) CALL OpenInput(StatU,h=NO)                  ! Open CHAD diary stat file
   IF (UseCHADOccFile==YES) CALL OpenInput(DEmpU,h=NO)               ! Open CHAD empl group file
   DO C=1,100                                                        ! Finding max # of diaries in age & pool
      DO B = 1,4*NumPools                                            ! Entering NumPools loop
        Agedays(B,C) = 0                                             ! Initializing AgeDays array
      END DO                                                         ! Ending NumPools loop
   END DO                                                            ! Ending age loop
   DO                                                                ! Loop until EXIT occurs
      IF (CommuteFlag==YES) THEN                                     ! If commuting, requires an extra column
         READ(ChQuU,*,IOSTAT=Ios(ChQuU)) QID, CWkDay, CSex, CRace,     &
            CEmp, CAge, CMaxT, CAvgT, Occup, QCMiss, RecCt, CComTime ! Read one record from quest file    
      ELSE                                                           ! If not, doesn't need it
         READ(ChQuU,*,IOSTAT=Ios(ChQuU)) QID, CWkDay, CSex, CRace,     &
            CEmp, CAge, CMaxT, CAvgT, Occup, QCMiss, RecCt           ! Read one record from quest file         
      ENDIF                                                          ! End reading in logic
      IF (LongitDiary==YES)                                         &
         READ(StatU,*,IOSTAT=Ios(StatU)) QID2, Stat                  ! Read one rec from diary stat fl
      IF (UseCHADOccFile==YES)                                      &
         READ(DEmpU,*,IOSTAT=Ios(StatU)) QID3, Occup                 ! Read one rec from diary emp fl
      IF (Ios(ChQuU)/=0) EXIT                                        ! Exit on end-of-file
      IF (LongitDiary==YES) THEN                                     ! For longitudinal diaries       
         IF (QID /= QID2) CALL Fatal(10)                             ! Make sure ID in files match   
      ENDIF                                                          ! End longitudinal logic
      IF (UseCHADOccFile==YES) THEN                                  ! For longitudinal diaries       
         IF (QID /= QID3) CALL Fatal(12)                             ! Make sure ID in files match   
      ENDIF                                                          ! End longitudinal logic
      Discard = Discard+1                                            ! Assume record will be discarded
      IF (CMaxT<=-99.) CYCLE                                         ! Discard if maximum temp missing
      IF (CAvgT<=-99.) CYCLE                                         ! Discard if average temp missing
      IF (CWkDay=="X") CYCLE                                         ! Discard if weekday is missing
      IF (QCMiss>=180) CYCLE                                         ! Discard if >=3 hours missing
      IF (QID=="VAL28040A".OR.QID=="VAL28179A".OR.                  &
          QID=="VAL28260A".OR.QID=="VAL38027A") CYCLE                ! These diaries are not 24 hours
      Discard    = Discard-1                                         ! Undo the discard status
      FirstEvent = LastEvent+1                                       ! Add events to end of list
      IF (CAge<0.OR.CAge>=199) CAge = -1.                            ! This means the age is missing
      Age = CAge                                                     ! Age is integer and CAge is real
      IF (Age>=0.AND.Age<=18.AND.CEmp=="X") CEmp="N"                 ! Unknowns under 19 not employed
      AgeCat = 3                                                     ! Default age group is 40+
      IF (Age>25.AND.Age<40)  AgeCat = 2                             ! Middle age group for METS
      IF (Age>=0.AND.Age<=25) AgeCat = 1                             ! Young age group for METS
      DO I=1,NumOccups-1                                             ! Loop over occupations
         IF (Occup==Occups(I)%OccDesc) EXIT                          ! Exit when occupation matches
      ENDDO                                                          ! Try next occupation
      CHADAct(10900,:) = Occups(I)%Row                               ! 10900 is code for work     
      STime            = 0                                           ! Start diary at midnight
      DO WHILE ((STime+Dur < 2360).OR.(DID/=QID))                    ! Loop over diary event records                 
         READ(ChEvU,*,IOSTAT=Ios(ChEvU)) DID, STime, Dur,           & 
            ActCode, LocCode                                         ! Read one diary event record
         IF (Ios(ChEvU)/=0) CALL Fatal(1)                            ! Stop if event file unreadable    
         IF (DID/=QID) CYCLE                                         ! Drop if IDs do not match
         IF (LocCode=="U") LocCode = "36400"                         ! Numeric value for location 'U'
         IF (LocCode=="X") LocCode = "36500"                         ! Numeric value for location 'X'
         READ(LocCode,*,IOSTAT=Ios(ChEvU)) LocInt                    ! Convert character to integer
         IF (Ios(ChEvU)/=0) CALL Fatal(2)                            ! Location code is not integer
         IF (ActCode=="U")     ActCode = "18930"                     ! Numeric value for activity 'U'
         IF (ActCode=="X")     ActCode = "18940"                     ! Numeric value for activity 'X'
         IF (ActCode=="17123") ActCode = "17213"                     ! Correction for CHAD miscoding  
         READ(ActCode,*,IOSTAT=Ios(ChEvU)) ActInt                    ! Convert character to integer
         IF (Ios(ChEvU)/=0) CALL Fatal(3)                            ! Activity code is not integer
         LastEvent = LastEvent+1                                     ! Increment the event index
         IF (LastEvent>MAXEVENTS)  CALL Fatal(9)                     ! If LastEvent > array bounds 
         SELECT CASE (APEXLoc(LocInt))                               ! Assign Home-Work based on Loc
            CASE ("H")                                               ! For these location codes...
               DData(LastEvent)%HomeWork=1                           ! Place is 'at home'
            CASE ("U")                                               ! For these location codes...
               DData(LastEvent)%HomeWork=-1                          ! Place is 'unknown'
            CASE ("W")                                               ! For these location codes...
               DData(LastEvent)%HomeWork=2                           ! Place is 'work'
            CASE ("O")                                               ! For these location codes...
               DData(LastEvent)%HomeWork=3                           ! Place is 'other'               
            CASE ("R")                                               ! For these location codes...
               DData(LastEvent)%HomeWork=4                           ! Place is 'roadway'               
            CASE ("N")                                               ! For these location codes...
               DData(LastEvent)%HomeWork=5                           ! Place is 'near home'     
            CASE ("M")                                               ! For these location codes...
               DData(LastEvent)%HomeWork=6                           ! Place is 'near work'
            CASE ("K")                                               ! For these location codes...
               DData(LastEvent)%HomeWork=7                           ! Place is 'near work'            
            CASE ("L")                                               ! For these location codes...
               DData(LastEvent)%HomeWork=99                          ! Place is last NH/NW                                                  
         END SELECT                                                  ! End of location code block
         IF (CustomWork==YES)  THEN                                  ! Custom work activities defined
            DO I=1, NumWork                                          ! Loop over custom activities
               IF (ActInt==WorkActs(I)) DData(LastEvent)%HomeWork=2  ! These activities are 'at work'
            ENDDO                                                    ! End loop over activities
         ELSE                                                        ! Otherwise use defaults   
            IF (ActInt<11000)  DData(LastEvent)%HomeWork=2           ! These activities are 'at work'
         ENDIF                                                       ! End of custom work logic
         IF (ActInt<11000.AND.ActInt/=10300) ActInt=10900            ! Special code for all work 
         DData(LastEvent)%Hour     = STime/100                       ! Get hour from military time
         DData(LastEvent)%Duration = Dur                             ! Duration is 1-60 minutes         
         DData(LastEvent)%Activity = CHADAct(ActInt,AgeCat)          ! Activity from mapping array
         DData(LastEvent)%Location = CHADLoc(LocInt)                 ! Location from mapping vector
         DData(LastEvent)%Location2 = CHADLocC(LocInt)               ! Location from mapping vector for clustering         
         DData(LastEvent)%ActCode  = ActInt                          ! Recording CHAD activity code
      ENDDO                                                          ! Continue with next record
      IF (LastEvent-FirstEvent+1/=RecCt) THEN                        ! Test for consistency in counts
         WRITE(LogU,*) 'Mismatch ',QID,RecCt,LastEvent-FirstEvent+1  ! Write message to Log file
         DO I=FirstEvent, LastEvent                                  ! Loop over events in diary
            WRITE(LogU,*) DData(I)                                   ! Write the events to Log file
         ENDDO                                                       ! Continue with next event
      ENDIF                                                          ! End of consistency check
      QOccupation = Occup                                            ! Assigning occupation group
      IF (UseOccGroups==YES) THEN                                    ! Set to missing if using CHAD groups
         IF (.NOT. ANY(GroupNames(:)==QOccupation)) QOccupation = "X" ! Convert to missing if not present
      ENDIF                                                          ! End emp group logic
      QGender     = INDEX("MFX",CSex)                                ! Convert gender to number 
      QEmployed   = INDEX("YNX",CEmp)                                ! Convert employment to number
      QRace       = INDEX("WBAHOX",CRace)                            ! Convert race to number
      QWeekDay    = (INDEX("SUNMONTUEWEDTHUFRISAT",CWkDay)+2)/3      ! Convert weekday to number
      QMaxTemp(1) = CMaxT                                            ! Put in target for distrib call
      QAvgTemp(1) = CAvgT                                            ! Put avg temp in pointer target
      QDayOfWk(1) = QWeekDay                                         ! Put weekday in pointer target
      MT => QMaxTemp(1:1)                                            ! Pointer to maximum temperature
      AT => QAvgTemp(1:1)                                            ! Pointer to average temperature
      WK => QDayOfWk(1:1)                                            ! Pointer to day of week
      DName = "DIARYPOOLS"                                           ! Buffer for distribution name
      I = FuncNum(DName,YES)                                         ! Stop if distrib is not defined 
      CALL EvalFunc(Name=DName,I1=MT,I2=AT,I3=WK,                     &
         IRes=PoolVec(1:1))                                          ! Get pools for this diary
      Pool = PoolVec(1)                                              ! Convert from vector to scalar
      IF (Pool<1.OR.Pool>NumPools) CALL Fatal(4)                     ! Pool # out of range
      PCount(Pool) = PCount(Pool)+1                                  ! Increment count for this pool
      IF (PCount(Pool)> MAXDIARIES) CALL Fatal(8)                    ! If PCount>array bounds,error  
      QData(PCount(Pool),Pool)%Pool       = Pool                     ! Load diary into QData array
      QData(PCount(Pool),Pool)%Index      = PCount(Pool)             ! Ditto, index # within pool
      QData(PCount(Pool),Pool)%Age        = Age                      ! Ditto, age for this diary
      QData(PCount(Pool),Pool)%DCommTime  = CComTime                 ! Ditto, commuting time       
      QData(PCount(Pool),Pool)%MaxTemp    = QMaxTemp(1)              ! Ditto, max temperature this day
      QData(PCount(Pool),Pool)%AvgTemp    = QAvgTemp(1)              ! Ditto, avg temperature this day
      QData(PCount(Pool),Pool)%Gender     = QGender                  ! Ditto, gender for this diary
      QData(PCount(Pool),Pool)%Race       = QRace                    ! Ditto, race for this diary
      QData(PCount(Pool),Pool)%Employed   = QEmployed                ! Ditto, employment status 
      QData(PCount(Pool),Pool)%Occupation = QOccupation              ! Ditto, occupation status
      QData(PCount(Pool),Pool)%Weekday    = QWeekDay                 ! Ditto, weekday (1-7)
      QData(PCount(Pool),Pool)%FirstEvent = FirstEvent               ! Ditto, DData # for first event 
      QData(PCount(Pool),Pool)%LastEvent  = LastEvent                ! Ditto, DData # for last event
      QData(PCount(Pool),Pool)%DiaryID    = QID                      ! Ditto, CHADID for diary
      QData(PCount(Pool),Pool)%Statistic  = Stat                     ! Ditto, longitudinal diary stat
      DiaryNum = DiaryNum+1                                          ! Total # of diaries accepted
      IF (CAge>=1.AND.CSex=="M".AND.CEmp=="Y") AgeDays(Pool,CAge)            &  
            = AgeDays(Pool,CAge) + 1                                 ! Populating Male Employed ages
      IF (CAge>=1.AND.CSex=="F".AND.CEmp=="Y") AgeDays(Pool+NumPools,CAge)   &
            = AgeDays(Pool+NumPools,CAge) + 1                        ! Populating Female Employed ages
      IF (CAge>=1.AND.CSex=="M".AND.CEmp=="N") AgeDays(Pool+2*NumPools,CAge) &
            = AgeDays(Pool+2*NumPools,CAge) + 1                      ! Populating Male Unemployed ages
      IF (CAge>=1.AND.CSex=="F".AND.CEmp=="N") AgeDays(Pool+3*NumPools,CAge) &
            = AgeDays(Pool+3*NumPools,CAge) + 1                      ! Populating Female Unemployed ages
      IF (MOD(DiaryNum,1000)==0) PRINT*,"Reading activity diary #"  &  
         ,DiaryNum, QData(PCount(Pool),Pool)%LastEvent               ! Report status each 1000 diaries
   ENDDO                                                             ! End of diary reading loop
   CLOSE(ChEvU)                                                      ! Close diary event file
   CLOSE(ChQuU)                                                      ! Close diary questionnaire file
   DO C=1,4*NumPools                                                 ! Finding Minimum value that can be used for clustering
      DO B=1,100                                                     ! Cycling through ages
         IF (AgeDays(C,B) > MINDIARIES) MINDIARIES = AgeDays(C,B)    ! MinDiaries = the max of the age pool counts
      END DO                                                         ! Ending age loop
   END DO                                                            ! Ending pool loop
   MINDIARIES = MAX(200,MINDIARIES)                                  ! Maximum is either MINDIARIES or 200
   IF (LongitDiary==YES)  THEN                                       ! If longitudinal assembly
      DO K=1, NumPools                                               ! Loop over all diary pools
         M=PCount(K)                                                 ! Init M to num picks
         DO NN = 1,FLOOR(log(real(PCount(K)))/0.69314718+1e-8)       ! Start Shell sort of probs
            M=FLOOR(M/2.)                                            ! Set M 
            DO KK=1,PCount(K)-M                                      ! Start outer loop 
               JJ=KK                                                 ! Init JJ 
               DO                                                    ! Start inner loop
                  L=JJ+M                                             ! Set L
                  Val1 = QData(L,K)%Statistic                        ! First statistic to examine
                  Val2 = QData(JJ,K)%Statistic                       ! Second statistic to examine
                  IF (Val1<Val2) THEN                                ! Compare stat values
                     T           =  QData(JJ,K)                      ! Swap diary data
                     QData(JJ,K) =  QData(L,K)                       ! Swap diary data
                     QData(L,K)  =  T                                ! Swap diary data
                     JJ = JJ-M                                       ! Reset JJ
                     IF (JJ>=1) CYCLE                                ! Continue inner loop  
                  ENDIF                                              ! End value comparison  
                  EXIT                                               ! Exit inner loop
               ENDDO                                                 ! End inner loop
            ENDDO                                                    ! End outer loop 
         ENDDO                                                       ! End Shell sort   
      ENDDO                                                          ! End loop over pools                            
      NumExtras = FLOOR(NumDays/30.)                                 ! Number of extra picks needed
      NumX      = NumDays + NumExtras                                ! Total number of picks
      ALLOCATE (XPicks(NumX),STAT=AllocErr)                          ! Allocate the array of picks     
      CALL CheckAllocation(AllocErr)                                 ! Check the success of the alloc
      ALLOCATE (DiaryRank(NumDays),STAT=AllocErr)                    ! Allocate the array of prcntls     
      CALL CheckAllocation(AllocErr)                                 ! Check the success of the alloc
      ALLOCATE (Picked(NumX),STAT=AllocErr)                          ! Allocate the array of flags              
      CALL CheckAllocation(AllocErr)                                 ! Check the success of the alloc
      ALLOCATE (Rn(0:NumX),STAT=AllocErr)                            ! Allocate the array of flags              
      CALL CheckAllocation(AllocErr)                                 ! Check the success of the alloc
   ENDIF                                                             ! End logitudinal logic   
   IF (ClustDiaryA==YES)  THEN                                       ! Beginning of new clustering code
	  INQUIRE(FILE='chadclust.dat',EXIST=exists)                     ! Looking for existing cluster file
      IF(exists .AND. RerunClus == NO) THEN                          ! If it exists, then use it
	     WRITE(*,*) 'LOADING CLUSTERED CHAD RECORDS ...'             ! Writing notice
	     OPEN(9992,file='chadclust.dat',form='unformatted',action='read') ! Opening existing file
         READ(9992) QCDCData,ClusterTransOut,ClusterNoOutW,ClusterPRMOut  ! Reading existing file
      ELSE                                                           ! If no existing file, or if rerun = YES
         ALLOCATE (QCData(2000,NumPools),STAT=AllocErr)              ! Make space for clustering data    
         CALL CheckAllocation(AllocErr)                              ! Check the success of the alloc       
         ALLOCATE (QCDData(2000,NumPools,3,3,0:MaxAge1),STAT=AllocErr)  ! Make space for clustering data
         CALL CheckAllocation(AllocErr)                              ! Check the success of the alloc    
         ALLOCATE (QCDData1(4000,NumPools,3,3),STAT=AllocErr)        ! Make space for clustering data
         CALL CheckAllocation(AllocErr)                              ! Check the success of the alloc      
         ALLOCATE (ClusterNoOut(3,NumPools), ClusterIDout(3,NumPools,MAXDIARIES),STAT=AllocErr) ! Make Space for clustering arrays
         CALL CheckAllocation(AllocErr)                              ! Check the success of the alloc   
         ALLOCATE (PDCount(NumPools,3,3,0:MaxAge1+1),STAT=AllocErr)  ! Make space for pool counts
         CALL CheckAllocation(AllocErr)                              ! Check the success of the alloc   
         ALLOCATE (PDCount1(NumPools,3,3,1),STAT=AllocErr)           ! Make space for pool counts
         CALL CheckAllocation(AllocErr)                              ! Check the success of the alloc   
         PDCount = 0												 ! Initialize pool counts to zero
         PDCount1 = 0												 ! Initialize pool counts to zero 
	     WRITE(*,*) 'CLUSTERING CHAD RECORDS BY POOL/GENDER/EMPLOYED ...' ! Writing notice
	     DO K=1, NumPools                                            ! Loop over all diary pools
            DO MM=1, PCount(K)                                       ! Loop over all diaries
		       MM1=QData(MM,K)%Gender                                ! Extract gender
		       MM2=QData(MM,K)%Employed                              ! Extract employment status
		       MM3Age=QData(MM,K)%Age                                ! Extract age
	           IF(MM3Age==-1) CYCLE                                  ! If no age, cycle
		       IF(MM1==3) CYCLE                                      ! If no gender, cycle
		       IF(MM2==3) CYCLE                                      ! If no employment status, cycle
		       IF(MM2==1 .and. MM3Age<16) MM2=2                      ! All Age < 16 are employed
		       PDCount1(K,MM1,MM2,1) = PDCount1(K,MM1,MM2,1)+1       ! Incrementing pool counter
		       QCDData1(PDCount1(K,MM1,MM2,1),K,MM1,MM2) = QData(MM,K) ! Incrementing data array
	        END DO                                                   ! Ending diary loop
	     END DO                                                      ! Ending pool group
         DO k=1,NumPools                                             ! Loop over diary pools
            DO mm1=1,2                                               ! For both genders
               DO mm2=1,2                                            ! For employed and unemployed
                  DO MAge=0, MaxAge1                                 ! Loop over ages
	                 AgeWindow1   = INT(MAX(NINT(REAL(MAge)*AgeCutPct/100.0),1),BYT) ! Width of age window in years
                     AgeMMM=PDCount1(k,mm1,mm2,1)                    ! Extracting Age variable
                     IF(MM2==1 .and. MAge<16) CYCLE                  ! (should never occur)
                     DO                                              ! Start counting loop
	                    DO mm33=1,AgeMMM                             ! For all ages
	                       MM3Age=QCDData1(mm33,k,mm1,mm2)%Age       ! Extract age
	                       IF (ABS(MM3Age-MAge) <= 2*AgeWindow1) THEN ! Only keep the records within the window
	                          PDCount(k,mm1,mm2,Mage)=PDCount(k,mm1,mm2,Mage)+1 ! Incrementing count
	                          QCDData(PDCount(k,mm1,mm2,Mage),k,mm1,mm2,Mage) = QCDData1(mm33,k,mm1,mm2)
	                                                                 ! Incrementing data step
                           ENDIF                                     ! End window step
	                    END DO                                       ! End age loop
	                    IF (PDCount(k,mm1,mm2,Mage)<=MINDIARIES+1) THEN ! Setting upper limit on count 
		                   EXIT                                      ! Exit counting loop
	                    ELSE                                         ! If too many, decrease window size
	                       AgeWindow1=AgeWindow1*0.99                ! Decreasing size
	                       PDCount(k,mm1,mm2,Mage)=0                 ! Resetting count
                        ENDIF                                        ! Exit conditional
                     END DO                                          ! Exit age loop
                  END DO                                             ! Exit employment loop
               END DO                                                ! Exit gender loop
            END DO                                                   ! End diary pool loop
         END DO                                                      ! End numpools loop
	     DO MM1=1, 2                                                 ! loop gender
		    DO MM2=1, 2                                              ! loop employ status
			   DO MM3=0,MaxAge1                                      ! loop age group
			      IF(MM2==1 .and. MM3<16) CYCLE                      ! All kids should be employed
		          QCData=QCDData(:,:,MM1,MM2,MM3)                    ! Grabbing clustering data
		          NumDiaries(:) = PDCount(:,MM1,MM2,MM3)             ! Setting number of diaries
		          WRITE(*,*) 'Clustering groups for ','Gender=',MM1, &
		             ' Employ=',MM2,' Age =',MM3,''                  ! Writing notice to screen
		          CALL Clustering2                                   ! Calling clustering subroutine
		          ClusterTransOut(:,:,:,:,MM1,MM2,MM3)=ClusterTransM(:,:,:,:) ! Setting ClusterTrans
		          ClusterNoOutW(:,:,MM1,MM2,MM3)=ClusterNoOut(:,:)   ! Setting ClusterNoOutW
		          ClusterPRMOut(:,:,MM1,MM2,MM3)=ClusterPRM(:,:)     ! Setting ClusterPRMOut
		          DO K=1, NumPools                                   ! Loop over all diary pools
			         DO JJ=1,NumCluster                              ! Loop over all cluster numbers
				        DO MM=1, ClusterNoOut(JJ,K)                  ! Loop over ClusterNoOut
					       QCDCData(MM,JJ,K,MM1,MM2,MM3)=QCData(ClusterIDOut(JJ,K,MM),K) ! Set QCData
				        END DO                                       ! End ClusterNoOut loop
			         END DO                                          ! End loop over clusters
		          END DO                                             ! End loop over diary pools
	           END DO                                                ! End age loop
	        END DO                                                   ! End employ loop
	     END DO                                                      ! End gender group
	     OPEN (9992,file='chadclust.dat',form='unformatted',action='write') ! Opening cluster output file
 	     WRITE(9992) QCDCData,ClusterTransOut,ClusterNoOutW,ClusterPRMOut ! Writing cluster output file
      ENDIF                                                          ! Ending cluster logic
   ENDIF                                                             ! End of new clustering code  
   PRINT*, 'Diaries Discarded=', Discard                             ! Message on discarded diaries
   PRINT*, 'Pool sizes=', PCount                                     ! Screen report on pool counts
   IF (ANY(PCount(1:NumPools)==0)) CALL Fatal(5)                     ! Stop if any pool is empty 
   WRITE(LogU,'(/A33 )') ' Total Diary Counts for each Pool'         ! Header for pool counts report
   WRITE(LogU,'(15I6)') PCount                                       ! Log file report on pool counts
   AgeSum  = 0                                                       ! Initialize summary statistics
   AgeMiss = 0                                                       ! Initialize # with missing age
   DO Pool=1,NumPools                                                ! Loop over pools
      DO I=1,PCount(Pool)                                            ! Loop over diaries within pool
         Age = QData(I,Pool)%Age                                     ! Temporary variable for age 
         IF (Age<0)  AgeMiss = AgeMiss+1                             ! Counter for missing ages
         IF (Age>=0) AgeSum  = AgeSum+Age                            ! Sum the non-missing ages
      ENDDO                                                          ! Continue with next diary
   ENDDO                                                             ! Continue with next pool
   WRITE(LogU,*)                                                     ! Put blank line on log
   WRITE(LogU,*) 'Age mean     ',REAL(Agesum)/(DiaryNum-AgeMiss)     ! Average of all non-missing ages
   WRITE(LogU,*) '# Age     =X ',AgeMiss                             ! # of diaries with missing ages
   WRITE(LogU,*) '# Gender  =M ',COUNT(QData%Gender==1)              ! # of diaries from males
   WRITE(LogU,*) '# Gender  =F ',COUNT(QData%Gender==2)              ! # of diaries from females
   WRITE(LogU,*) '# Gender  =X ',COUNT(QData%Gender==3)              ! # of diaries of unknown gender
   WRITE(LogU,*) '# Employed=Y ',COUNT(QData%Employed==1)            ! # from employed persons
   WRITE(LogU,*) '# Employed=N ',COUNT(QData%Employed==2)            ! # from persons not employed
   WRITE(LogU,*) '# Employed=X ',COUNT(QData%Employed==3)            ! # of unknown employment status
   IF (UseOccGroups==YES) THEN                                       ! If using occupation groups
       DO I=2,NumLevels                                              ! Loop over all empl groups
          WRITE(LogU,*) '# ', GroupNames(I), " = ", COUNT(QData%    &
             Occupation==GroupNames(I))                              ! Write out occup group counts
          IF (COUNT(QData%Occupation==GroupNames(I))==0) CALL       &
             Warning(100+I)                                          ! Warn if none in group
       ENDDO                                                         ! End loop over empl groups
       WRITE(LogU,*) '# MISSING      = ', COUNT(QData%Occupation=="X") ! Write out missing groups
   ENDIF                                                             ! End occupation group logic
   IF (ANY(DData(1:LastEvent)%Activity==0))   CALL Fatal(6)          ! Stop if any missing activities
   IF (ANY(DData(1:LastEvent)%Location==-99)) CALL Fatal(7)          ! Stop if any missing locations
   DO I=1,LastEvent                                                  ! Loop over all diary events 
      A = DData(I)%Activity                                          ! Obtain activity code index
      ActCount(A) = ActCount(A) + 1                                  ! Increment activity counter 
      ActTime(A)  = ActTime(A)  + DData(I)%Duration                  ! Add time to total duration
   ENDDO                                                             ! Continue with next event
   WRITE(LogU,*)                                                     ! Put blank line on log 
   WRITE(LogU,*) "METS Distribution Counts on Activity Database"     ! Write header to log file
   WRITE(LogU,*) "     Distrib#    #events    #minutes"              ! Column headers for table
   DO K=1,166                                                        ! Loop over activity array 
      IF (K>127) THEN                                                ! For acts with negative index 
         I=K-256                                                     ! I is internal APEX number
      ELSE                                                           ! Otherwise positive number
         I=K                                                         ! I is internal APEX number
      ENDIF                                                          ! End mets index logic      
      IF (ActCount(I)>0) WRITE(LogU,*) K,ActCount(I),ActTime(I)      ! Only write nonzero data
   ENDDO                                                             ! Continue with next activity 
   ActCount = 0                                                      ! Reset counter (used by METSEval)
   ActTime  = 0                                                      ! Reset duration array
   IF (DebugLevel>0) WRITE(LogU,LV_FMT) LV_MSG, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END SUBROUTINE ReadDiaries                                           ! Return to main APEX program


SUBROUTINE ReadMicroMap                                              ! Called from ReadDiaries
   CHARACTER(LEN=LEN4) :: InString                                   ! Buffer for input records
   CHARACTER(LEN=5)    :: CHADCode                                   ! CHAD location code
   CHARACTER(LEN=1)    :: Loc                                        ! APEX location code
   CHARACTER(LEN=2)    :: Loc2                                       ! APEX location code
   INTEGER (KIND=IK)   :: CHADInt, LType, Equals, RFlag              ! Local integer variables
   INTEGER (KIND=BYT)  :: MicroNum                                   ! Local 1-byte variable
   ! Global constants: BYT, ChMapU, IK, LEN3, LogU, NO, LV_MS2, SL_FM2, ST_MS2
   ! Global vars used: DebugLevel, NumMicros
   ! Global vars set : Ios, ProcLevel, ProcName
   ! Module vars set : CHADLoc
   ! Intrinsic procs : ADJUSTL, CLOSE, INDEX, READ, WRITE
   ! APEX procedures : Fatal, GetLine, OpenInput
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "ReadMicroMap"                              ! Procedure name for messages
   IF (DebugLevel>0) WRITE(LogU,SL_FM2) ST_MS2, ProcName(ProcLevel)  ! Message for debugging
   CALL OpenInput(ChMapU,h=NO)                                       ! Open CHAD-micro mapping file
   RFlag = 0                                                         ! Initializing local roadway flag
   DO                                                                ! Loop over input file records
      CALL GetLine(ChMapU,InString,LType)                            ! Get one input record 
      IF (Ios(ChMapU)<0) EXIT                                        ! Exit loop at end of file
      IF (INDEX("3UX",InString(1:1))==0) CYCLE                       ! Skip all non-CHAD codes 
      READ(InString,'(A5)',IOSTAT=Ios(4)) CHADCode                   ! Read CHAD code from buffer
      IF (CHADCode=="U") CHADCode="36400"                            ! Numeric code for location 'U'
      IF (CHADCode=="X") CHADCode="36500"                            ! Numeric code for location 'X'
      READ(CHADCode,*,IOSTAT=Ios(5)) CHADInt                         ! Convert location to integer
      Equals = INDEX(InString,"=")                                   ! Locate "=" in input record
      InString = ADJUSTL(InString(Equals+1:))                        ! Portion to right of = sign
      READ(InString,*,IOSTAT=Ios(6)) MicroNum, Loc2                  ! Read values from string
      IF (Loc2=="NH") THEN                                           ! If code is Near Home (NH)
         Loc="N"                                                     ! Set to N
      ELSEIF (Loc2=="NW") THEN                                       ! If code is Near Work (NW)
         Loc="M"                                                     ! Set to M
      ELSEIF (Loc2=="RW") THEN                                       ! If code is Road Work (RW)
         Loc="K"                                                     ! Set to K
      ELSE                                                           ! Otherwise...
         Loc=TRIM(Loc2)                                              ! Trim off excess space
      ENDIF                                                          ! End input logic
      IF (MicroNum<-1.OR.MicroNum>NumMicros) CALL Fatal(1)           ! Stop if micro # out of range
      IF (Loc=="")                           CALL Fatal(2)           ! Stop if loc is missing
      IF (INDEX("R",Loc)>0 .AND. RoadWayFlag==NO) CALL Fatal(4)      ! Stop if road locs, but flag = no
      IF (INDEX("K",Loc)>0 .AND. RoadWayFlag==NO) CALL Fatal(4)      ! Stop if road locs, but flag = no      
      IF (INDEX("R",Loc)>0) RFlag = 1                                ! Setting road flag if R present
      IF (INDEX("K",Loc)>0) RFlag = 1                                ! Setting road flag if R present      
      IF (INDEX("N",Loc)>0) UseNHLocs = YES                          ! Use new home locs
      IF (INDEX("M",Loc)>0) UseNWLocs = YES                          ! Use near work locs
      IF (INDEX("L",Loc)>0) THEN                                     ! Set pars for last loc
         UseNHLocs = YES                                             ! Use near home locs
         UseNWLocs = YES                                             ! Use near work locs
      ENDIF                                                          ! End last loc logic
      IF (INDEX("HWOURNMLK",Loc)==0)          CALL Fatal(3)          ! Stop if loc is not valid
      CHADLoc(CHADInt) = MicroNum                                    ! Assign the CHAD-micro mapping
      APEXLoc(CHADInt) = Loc                                         ! Assign the APEX location
      Loc=""                                                         ! Reset location
   ENDDO                                                             ! Continue with next input record
   IF (UseNHLocs==NO .AND. UseNWLocs==NO .AND. ResampleNear==YES) CALL Fatal(5) ! Warning if Resample=Y, but no NH/NW
   CLOSE(ChMapU)                                                     ! Close CHAD-micro mapping file
   IF (RFlag==0 .AND. RoadWayFlag==YES) CALL Warning(1)              ! Roadway = Yes, but no R's defined
   IF (DebugLevel>0) WRITE(LogU,SL_FM2) LV_MS2, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END SUBROUTINE ReadMicroMap                                          ! Return to ReadDiaries 


SUBROUTINE ReadClusterMicroMap                                       ! Called from ReadDiaries
   CHARACTER(LEN=LEN4) :: InString                                   ! Buffer for input records
   CHARACTER(LEN=5)    :: CHADCode                                   ! CHAD location code
   INTEGER (KIND=IK)   :: CHADInt, LType, Equals                     ! Local integer variables
   INTEGER (KIND=BYT)  :: MicroNum                                   ! Local 1-byte variable
   ! Global constants: BYT, ClMapU, IK, LEN3, LogU, NO, LV_MS2, SL_FM2, ST_MS2
   ! Global vars used: DebugLevel, NumMicros
   ! Global vars set : Ios, ProcLevel, ProcName
   ! Module vars set : CHADLoc
   ! Intrinsic procs : ADJUSTL, CLOSE, INDEX, READ, WRITE
   ! APEX procedures : Fatal, GetLine, OpenInput
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "ReadClusterMicroMap"                       ! Procedure name for messages
   IF (DebugLevel>0) WRITE(LogU,SL_FM2) ST_MS2, ProcName(ProcLevel)  ! Message for debugging
   CALL OpenInput(ClMapU,h=NO)                                       ! Open CHAD-micro mapping file
   DO                                                                ! Loop over input file records
      CALL GetLine(ClMapU,InString,LType)                            ! Get one input record 
      IF (Ios(ClMapU)<0) EXIT                                        ! Exit loop at end of file
      IF (INDEX("3UX",InString(1:1))==0) CYCLE                       ! Skip all non-CHAD codes 
      READ(InString,'(A5)',IOSTAT=Ios(4)) CHADCode                   ! Read CHAD code from buffer
      IF (CHADCode=="U") CHADCode="36400"                            ! Numeric code for location 'U'
      IF (CHADCode=="X") CHADCode="36500"                            ! Numeric code for location 'X'
      READ(CHADCode,*,IOSTAT=Ios(5)) CHADInt                         ! Convert location to integer
      Equals = INDEX(InString,"=")                                   ! Locate "=" in input record
      InString = ADJUSTL(InString(Equals+1:))                        ! Portion to right of = sign
      READ(InString,*,IOSTAT=Ios(6)) MicroNum                        ! Convert this to numeric
      IF (MicroNum<-1.OR.MicroNum>NumMicros) CALL Fatal(1)           ! Stop if micro # out of range
      CHADLocC(CHADInt) = MicroNum                                   ! Assign the CHAD-micro mapping
   ENDDO                                                             ! Continue with next input record
   CLOSE(ClMapU)                                                     ! Close CHAD-micro mapping file
   IF (DebugLevel>0) WRITE(LogU,SL_FM2) LV_MS2, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END SUBROUTINE ReadClusterMicroMap                                   ! Return to ReadDiaries 


SUBROUTINE ReadMETS                                                  ! Called from ReadDiaries
   INTEGER (KIND=BYT)  :: MetsNum                                    ! 1-byte METS distribution #
   INTEGER (KIND=IK)   :: DNum                                       ! METS file row #
   CHARACTER(LEN=5)    :: CHADCode                                   ! CHAD activity code
   CHARACTER(LEN=LEN1) :: Occup                                      ! CHAD occupation category
   CHARACTER(LEN=1)    :: CMetType                                   ! METS distribution type   
   INTEGER (KIND=IK)   :: AgeCat, CHADInt, I, LType, K               ! Local integer variables  
   CHARACTER (LEN=5) :: Char1                                        ! Buffer for distribution part
   CHARACTER (LEN=LEN4) :: String, UpString                          ! Buffers for input records
   ! Global constants: BYT, IK, LogU, LEN1, LV_MS2, MetsU, NO, SL_FM2, ST_MS2
   ! Global vars used: DebugLevel
   ! Global vars set : Ios, ProcLevel, ProcName
   ! Module constants: POINT, UNIFORM, NORMAL, LOGNORMAL, TRIANGLE, EXPONENTIAL
   ! Module vars set : CHADAct, MetsDist, NumOccups, Occups
   ! Intrinsic procs : CLOSE, READ, WRITE 
   ! APEX procedures : Fatal, OpenInput
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "ReadMETS"                                  ! Procedure name for messages
   IF (DebugLevel>0) WRITE(LogU,SL_FM2) ST_MS2, ProcName(ProcLevel)  ! Message for debugging
   CALL OpenInput(MetsU,h=NO)                                        ! Open the METS map input file
   DO                                                                ! Loop over records on input file
      IF (Ios(MetsU)<0) EXIT                                         ! Exit loop on end-of-file
      CALL GetLine(MetsU,String,Ltype)                               ! Get next data line from input
      UpString = Up(String)                                          ! Make uppercase version of line 
      READ(UpString,*)  DNum, Char1                                  ! Read the distribution info
      UpString = UpString(INDEX(UpString,Char1):)                    ! Drop all but distribution info
       IF (DNum >127) THEN                                           ! If Row # 128 or greater  
          MetsNum = DNum - 256                                       ! Shift the MetsNum to 1-byte 
      ELSE                                                           ! Otherwise if Row # 127 or less
          MetsNum = DNum                                             ! No shift needed 
      ENDIF                                                          ! End MetsNum logic    
      MetsDist(MetsNum) = ReadDist(UpString)                         ! Save the METS distribs   
   ENDDO                                                             ! Continue with next record
   CLOSE(MetsU)                                                      ! Close METS input file
   CALL OpenInput(MMapU,h=NO)                                        ! Open METS input file
   READ (MMapU,*,IOSTAT=Ios(MMapU))                                  ! Skip one header line 
   DO                                                                ! Loop over records on input file
      IF (Ios(MMapU)<0) EXIT                                         ! Exit loop on end-of-file
      CALL GetLine(MMapU,String,Ltype)                               ! Get next data line from input
      UpString = Up(String)                                          ! Make uppercase version of line 
      READ(UpString,*)  CHADCode, AgeCat, Occup, DNum                ! Get the CHAD activity info
      IF (Ios(MMapU)<0) EXIT                                         ! Exit loop on end-of-file
      IF (Ios(MMapU)>0) CALL Fatal(1)                                ! Stop if data are unreadable
      IF (DNum < 1)  CALL Fatal(2)                                   ! Stop if using old METS file
      IF (DNum >127) THEN                                            ! If Row # 128 or greater  
          MetsNum = DNum - 256                                       ! Shift MetsNum to 1-byte int
      ELSE                                                           ! Otherwise if Row # 127 or less
          MetsNum = DNum                                             ! No shift needed 
      ENDIF                                                          ! End MetsNum logic   
      IF (CHADCode=="U") CYCLE                                       ! No METS needed for 'U'
      IF (CHADCode=="X") CYCLE                                       ! No METS needed for 'X'
      READ(CHADCode,*,IOSTAT=Ios(3)) CHADInt                         ! Convert activity to integer
      IF (Occup=='ANY') THEN                                         ! If not occupation dependent...
         IF (AgeCat==0)  CHADAct(CHADInt,:) = MetsNum                ! This means all ages
         IF (AgeCat==20) CHADAct(CHADInt,1) = MetsNum                ! Age range 0-25 
         IF (AgeCat==30) CHADAct(CHADInt,2) = MetsNum                ! Age range 26-39
         IF (AgeCat==40) CHADAct(CHADInt,3) = MetsNum                ! Age range 40+
      ELSE                                                           ! Else occupation dependent...
         NumOccups = NumOccups+1                                     ! Add new occupation to list
         IF (NumOccups>MAXOCCUPS) CALL Fatal(3)                      ! If array bounds exceeded   
         Occups(NumOccups)%OccDesc = Occup                           ! Description of new occupation
         Occups(NumOccups)%Row     = MetsNum                         ! Number for new occupation
      ENDIF                                                          ! End of age-occupation block
   ENDDO                                                             ! Continue with next record
   WRITE(LogU,*) '   Num OCCUP=', NumOccups                          ! Summarize the occupations found
   DO I=1, NumOccups                                                 ! Loop over occupations
      WRITE(LogU,*) '   ',Occups(I)%OccDesc,I                        ! Write # and description
   ENDDO                                                             ! Continue with next occupation
   IF (UseOccGroups==YES) THEN                                       ! If matching by group...
      ALLOCATE (GroupMETS(NumOccups+1),STAT=AllocErr)                ! Allocate the array of dists              
      CALL CheckAllocation(AllocErr)                                 ! Check the success of the alloc
      GroupMETS(:) = 0                                               ! Initializing array
      DO I=1,NumOccups                                               ! Loop over all occupations
         IF (Occups(I)%OccDesc=="X") GroupMETS(1)=Occups(I)%Row      ! Find dist for missing
      ENDDO                                                          ! End occupation loop
      DO I=1,NumOccups                                               ! Looping over all occupations
         DO K=1,NumLevels                                            ! Looping over all groups
            IF (GroupNames(K)==Occups(I)%OccDesc) GroupMETS(K) =    &
               Occups(I)%Row                                         ! Assign group dist number
         ENDDO                                                       ! End group loop
      ENDDO                                                          ! End occupation loop
      IF (ANY(GroupMETS(1:NumLevels)==0)) CALL Fatal(4)              ! If not there, call error        
   ENDIF                                                             ! End group occ matching logic       
   ! IF (ANY(GroupMETS(1:NumLevels)==0)) CALL Fatal(4)                 ! If not there, call error   
   CHADAct(18930,:) = -1                                             ! Mapping for CHAD activity 'U'
   CHADAct(18940,:) = -1                                             ! Mapping for CHAD activity 'X'
   CLOSE(MMapU)                                                      ! Close METS map file
   IF (DebugLevel>0) WRITE(LogU,SL_FM2) LV_MS2, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END SUBROUTINE ReadMETS                                              ! Return to ReadDiaries


SUBROUTINE DiaryProbabilities(P)                                     ! Called from main APEX program
   INTEGER (KIND=IK), INTENT(IN) :: P                                ! Profile number (input)
   INTEGER (KIND=IK)  :: J, K, L, NonZero                            ! Local integer variables
   INTEGER (KIND=IK)  :: MaxAge1 = 100                               ! Set local integer variable
   REAL (KIND=LONG),ALLOCATABLE :: AccuFrac2(:,:,:)					 ! Accumulative fraction of records in each cluster
   REAL (KIND=LONG),ALLOCATABLE :: R22(:,:)						     ! Local integer variables
   INTEGER (KIND=IK), ALLOCATABLE :: DiSelID2(:,:)                   ! Local clustering array
   INTEGER (KIND=IK), PARAMETER :: MAXDIARIES=2**15                  ! Local clustering variable
   REAL    (KIND=RK)  :: P1, P2, P3, P4, P5, Total                   ! Local real variables
   INTEGER (KIND=BYT) :: Gender, Age, Employed, AgeWindow, AgeWindow2 ! Local 1-byte integer variables
   REAL    (KIND=RK)  :: TargetTime, CommOff                         ! Local real variables
   ! Global constants: BYT, EMPTEXT, IK, LogU, LV_MS2, RK, SEXTEXT, SL_FM2, ST_MS2, YES
   ! Global vars used: AgeCutPct, Age2Probab, DebugLevel, FirstProfile, MissAge, MissEmpl, MissGender, 
   !                   LogProfiles, NumPools, Person 
   ! Global vars set : ProcLevel, ProcName
   ! Module vars used: DData, PCount
   ! Module vars set : NumDiaries, QData
   ! Intrinsic procs : ABS, ANY, INT, MAX, NINT, REAL, SUM, WRITE
   ! APEX procedures : DiaryRanks, Fatal
   ALLOCATE (R22(NumCluster,NumPools),DiSelID2(NumCluster,NumPools)) ! Allocate clustering array
   ALLOCATE (AccuFrac2(NumCluster,NumPools,MAXDIARIES))              ! Allocate clustering array 
   ALLOCATE (DiSelIDM(3,NumPools),STAT=AllocErr)                     ! Make space for select diary sequence id
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "DiaryProbabilities"                        ! Procedure name for messages
   IF (DebugLevel>1) WRITE(LogU,SL_FM2) ST_MS2, ProcName(ProcLevel)  ! Message for debugging
   Gender      = INT(Person(P)%Gender,BYT)                           ! Local 1-byte version of gender
   Age         = INT(Person(P)%Age,BYT)                              ! Local 1-byte version of age
   Employed    = INT(Person(P)%Employed,BYT)                         ! Local 1-byte version of employ.
   AgeWindow   = INT(MAX(NINT(REAL(Age)*AgeCutPct/100.0),1),BYT)     ! Width of age window in years
   TargetTime  = 2*Person(P)%CommTime                                ! Local version of personal comm time
   IF (ClustDiaryA==NO) THEN									 	 ! Bypass the agewindow dynamic adjustment for clustering   
      QData(:,:)%Probability = 0.0                                   ! Initialize probability values
      DO K=1, NumPools                                               ! Loop over all diary pools
         NonZero = 0                                                 ! Initialize count to zero
	     IF (ClustDiaryB==NO) THEN									 ! Bypass the agewindow dynamic adjustment for clustering      
            DO J=1, PCount(K)                                        ! Loop over diaries in pool
               P1 = 0.0                                              ! Initialize gender probability
               P2 = 0.0                                              ! Initialize employ probability
               P3 = 0.0                                              ! Initialize age probability
               P4 = 1.0                                              ! Initialize commuting probability            
               P5 = 1.0                                              ! Initialize occupation probability
               IF (QData(J,K)%Gender==Gender)       P1 = 1.0         ! If gender matches, full weight
               IF (QData(J,K)%Gender==3)            P1 = MissGender  ! If gender missing, partial wt
               IF (QData(J,K)%Employed==Employed)   P2 = 1.0         ! If employ matches, full weight
               IF (Age<16)                          P2 = 1.0         ! Ignore employ stat for AGE<17
               IF (QData(J,K)%Employed==3)          P2 = MissEmpl    ! If employ missing, partial wt
               IF (ABS(QData(J,K)%Age-Age)<=AgeWindow) THEN          ! If age within window...
                  P3=1.0                                             ! Then give full weight 
               ELSE                                                  ! Or if age not within window...
               IF (ABS(QData(J,K)%Age-Age)<=2*AgeWindow) P3=Age2Probab ! If within shoulder, secondary wt
               ENDIF                                                 ! End of age testing
               IF (QData(J,K)%Age==-1)              P3 = MissAge     ! If age is missing, set low wt                  
               IF (Employed==1 .AND. CommuteFlag==YES) THEN          ! If the person is employed
                  IF (k<4) THEN                                      ! If weekday
                     CommOff = ABS(QData(J,K)%DCommTime-TargetTime)  ! Relate target and diary commuting time
                     IF (CommOff<=ComWindow1) THEN                   ! If commut time within window1
                        P4=1.0                                       ! Then give full weight 
                     ELSE IF (CommOff<=ComWindow2)  THEN             ! Or is window...        
                        P4=ComProbab1                                ! If within shoulder, secondary wt
                     ELSE                                            ! Or is window...
                        P4=ComProbab2                                ! If not within shoulder, terciary wt
                     ENDIF                                           ! End of weight assignments
                  ENDIF                                              ! End of weekday loop
               ENDIF                                                 ! End of Employed loop        
               IF (UseOccGroups==YES) THEN                           ! If we are selecting by occ group
                  IF (QData(J,K)%Occupation/=Person(P)%GroupName) P5=0.0 ! Doesn't match, P = 0
                  IF (TRIM(QData(J,K)%Occupation)=="X") P5=MissOcc   ! If missing, partial weight
                  IF (Person(P)%GroupName=="UNEMPLOYED") P5=1.0      ! If unemployed, any diary will do
               ENDIF                                                 ! End selection by occ group
               QData(J,K)%Probability = P1*P2*P3*P4*P5               ! Set total diary probability
               IF (QData(J,K)%Probability>0.0) THEN                  ! If probability is 0
                  NonZero=NonZero+1                                  ! Count # probabilities > 0
               ENDIF                                                 ! End probability logic
            ENDDO                                                    ! Continue with next diary   
	     ELSE IF (ClustDiaryB==YES) THEN							 ! CLUSTDIARY is used	
	        AgeWindow2=AgeWindow									 ! AgeWindow2 is adjustable if ClustDiary =YES
	        DO														 ! Loop till # of chad record is <MINDIARIES in a pool
               DO J=1, PCount(K)                                     ! Loop over diaries in pool
	              P1 = 0.0                                           ! Initialize gender probability
                  P2 = 0.0                                           ! Initialize employ probability
                  P3 = 0.0                                           ! Initialize age probability
                  P4 = 1.0                                           ! Initialize commuting probability
                  P5 = 1.0                                           ! Initialize occupation probability
                  IF (QData(J,K)%Gender==Gender)       P1 = 1.0      ! If gender matches, full weight
                  IF (QData(J,K)%Gender==3)            P1 = MissGender ! If gender missing, partial wt
                  IF (QData(J,K)%Employed==Employed)   P2 = 1.0      ! If employ matches, full weight
                  IF (Age<16)                          P2 = 1.0      ! Ignore employ stat for AGE<17
                  IF (QData(J,K)%Employed==3)          P2 = MissEmpl ! If employ missing, partial wt
                  IF (ABS(QData(J,K)%Age-Age)<=AgeWindow2) THEN      ! If age within window...
                     P3=1.0                                          ! Then give full weight 
                  ELSE                                               ! Or if age not within window...
                     IF (ABS(QData(J,K)%Age-Age)<=2*AgeWindow2) P3=Age2Probab ! If within shoulder, secondary wt
                  ENDIF                                              ! End of age testing
                  IF (QData(J,K)%Age==-1)              P3 = MissAge  ! If age is missing, set low wt
                  IF (Employed==1 .AND. CommuteFlag==YES) THEN       ! If the person is employed
                     IF (k<4) THEN                                   ! If weekday
                        CommOff = ABS(QData(J,K)%DCommTime-TargetTime) ! Relate target and diary commuting time
                        IF (CommOff<=ComWindow1) THEN                ! If commut time within 10%
                            P4=1.0                                   ! Then give full weight 
                        ELSE IF (CommOff<=ComWindow2)  THEN          ! Or is window...        
                            P4=ComProbab1                            ! If within shoulder, secondary wt
                        ELSE                                         ! Or is window...
                            P4=ComProbab2                            ! If not within shoulder, terciary wt
                        ENDIF                                        ! End of weight assignments
                     ENDIF                                           ! End of weekday loop
                  ENDIF                                              ! End of Employed loop                  
                  IF (UseOccGroups==YES) THEN                        ! If we are selecting by occ group
                     IF (QData(J,K)%Occupation/=Person(P)%GroupName) P5=0.0 ! Doesn't match, P = 0
                     IF (TRIM(QData(J,K)%Occupation)=="X") P5=MissOcc ! If missing, partial weight
                     IF (Person(P)%GroupName=="UNEMPLOYED") P5=1.0   ! If unemployed, any diary will do
                  ENDIF                                              ! End selection by occ group
                  QData(J,K)%Probability = P1*P2*P3*P4*P5            ! Set total diary probability
                  IF (QData(J,K)%Probability>0.0) THEN               ! If probability is 0
                     NonZero=NonZero+1                               ! Count # probabilities > 0
			         QCData(NonZero,K)=QData(J,K)					 ! Qdata for clustering
                  ENDIF                                              ! End probability logic
               ENDDO                                                 ! Continue with next diary  
	           IF (NonZero<=MINDIARIES) THEN						 ! If matched CHAD # <=MINDIARIES
		          EXIT												 ! Accept NonZero #
	           ELSE IF (NonZero>MINDIARIES) THEN					 ! If matched chad # > MINDIARIES
		          AgeWindow2=AgeWindow2*0.99						 ! Adjust AgeWindow
		          NonZero=0											 ! Re-initialize the chad# in each pool to zero
	           END IF												 ! End of checking CHAD#
	        END DO													 ! End of dyanmic loop for adjusting NonZero
         END IF														 ! End of dyanmic agewindow choice
         NumDiaries(K)=NonZero                                       ! Set diary count array
         Total = SUM(QData(1:PCount(K),K)%Probability)               ! Total the probabilities
         IF (Total>0.0) THEN                                         ! If there is any probability...
            QData(1,K)%Probability=QData(1,K)%Probability/Total      ! Convert to unit probability
            DO J=2,PCount(K)                                         ! Loop over diaries in pool
               QData(J,K)%Probability=QData(J,K)%Probability/Total+    & 
               QData(J-1,K)%Probability                              ! Convert to cumulative probab.
            ENDDO                                                    ! Continue with next diary
         ENDIF                                                       ! End of setting probability
      ENDDO                                                          ! Continue with next pool
      IF (DebugLevel>0.AND.NumPools<=10.AND.LogProfiles==YES) THEN   ! If user wants diary counts...
         IF (P==FirstProfile) WRITE(LogU,*)                            & 
            "Count of Suitable Diaries by Pool"                      ! Header for Profile summary 
         IF (P==FirstProfile) WRITE(LogU,'(A26,10I6)')                 &
            " Profile Age Gender Employ",(J,J=1,NumPools)            ! Header for Profile summary
         WRITE(LogU,'(I8,I4,1X,A6,1X,A5,2X,10I6)')                     &
            P,Age,SEXTEXT(Gender),EMPTEXT(Employed),NumDiaries(:)    ! Put status message on log
      ENDIF                                                          ! End diary count logic
      IF (ANY(NumDiaries(:)==0)) CALL Fatal(1)                       ! Stop if a diary pool is empty
      IF (DebugLevel>1) WRITE(LogU,SL_FM2) LV_MS2, ProcName(ProcLevel) ! Message for debugging
      ProcLevel = ProcLevel-1                                        ! Decrement procedure depth
      IF (ClustDiaryB==YES) THEN									 ! If ClustDiary = Yes
	     CALL clustering											 ! Call Clustering Subroutine
      END IF														 ! End of Clustering B
   ELSE IF (ClustDiaryA==YES) THEN                                   ! If ClusterDiaryA
      NUM2DIARY=0                                                    ! Resetting diary number
      AccuFrac2=0                                                    ! Resetting fraction
      IF(AGE<16) THEN                                                ! If age < 16
	  	 IF(Employed== YES) THEN                                     ! If they are employed
		    Employed=NO                                              ! Set to not employed
		    WRITE(*,*) 'warning employed status wrong for underage'  ! Write warning to log
		 ENDIF                                                       ! End employed conditional
      ENDIF                                                          ! End age conditional
      DO K=1, NumPools 										         ! Loop over diaries in pool
         DO J=1, NumCluster                                          ! Loop over all clusters
            DO L=1, ClusterNoOutW(J,K,Gender,Employed,Age)           ! Loop over ClusterNoOutW
               P1 = 1.0                                              ! Initialize gender probability
               P2 = 1.0                                              ! Initialize employ probability
               P3 = 0.0                                              ! Initialize age probability
               P4 = 1.0                                              ! Initialize commuting probability
               P5 = 1.0                                              ! Initialize occupation probability
               IF (ABS(QCDCData(L,J,K,Gender,Employed,Age)%Age-Age) &
                  <=AgeWindow) THEN                                  ! If age within window...
                  P3=1.0                                             ! Then give full weight 
               ELSE                                                  ! Or if age not within window...
                  IF (ABS(QCDCData(L,J,K,Gender,Employed,Age)%Age   &
                     -Age)<=2*AgeWindow) P3=Age2Probab               ! If within shoulder, secondary wt
               ENDIF                                                 ! End of age testing
               IF (Employed==1 .AND. CommuteFlag==YES) THEN          ! If the person is employed
                  IF (k<4) THEN                                      ! If weekday
                     CommOff = ABS(QCDCData(L,J,K,Gender,Employed,  &
                        Age)%DCommTime-TargetTime)                   ! Relate target and diary commuting time              
                     IF (CommOff<=ComWindow1) THEN                   ! If commut time within frst window
                        P4=1.0                                       ! Then give full weight 
                     ELSE IF (CommOff<=ComWindow2) THEN              ! Or is window 2...        
                        P4=ComProbab1                                ! If within shoulder, secondary wt
                     ELSE                                            ! Or is outside both windows
                        P4=ComProbab2                                ! If not within shoulder, terciary wt
                     ENDIF                                           ! End of weight assignments
                  ENDIF                                              ! End of weekday loop
               ENDIF                                                 ! End of Employed loop               
               IF (UseOccGroups==YES) THEN                           ! If we are selecting by occ group
                  IF (QCDCData(L,J,K,Gender,Employed,Age)%          &
                     Occupation/=Person(P)%GroupName) P5=0.0         ! Doesn't match, P = 0
                  IF (TRIM(QCDCData(L,J,K,Gender,Employed,Age)%     &
                     Occupation)=="X") P5=MissOcc                    ! If missing, partial weight
                  IF (Person(P)%GroupName=="UNEMPLOYED") P5=1.0      ! If unemployed, any diary will do
               ENDIF                                                 ! End selection by occ group
               QCDCData(L,J,K,Gender,Employed,Age)%Probability =    &
                  P1*P2*P3*P4*P5                                     ! Set total diary probability              
               IF (QCDCData(L,J,K,Gender,Employed,Age)%Probability  &
                  >0.0) THEN                                         ! If probability is 0
                  NUM2DIARY(J,K) = NUM2DIARY(J,K)+1                  ! Count # probabilities > 0
               ENDIF                                                 ! End probability logic
            ENDDO                                                    ! End clusterNoOut logic
	     ENDDO                                                       ! End cluster loop
      ENDDO                                                          ! End diary pool loop
      DO K=1, NumPools                                               ! Loop over pool numbers
         NumDiaries(K) = sum(NUM2DIARY(:,K))                         ! Sum over diaries
      END DO                                                         ! End pool number loop
      DO K=1, NumPools 											     ! Loop over diaries in pool
	     DO J=1, NumCluster                                          ! Loop over all clusters
            Total = SUM(QCDCData(1:ClusterNoOutW(J,K,Gender,        &
               Employed,Age),J,K,Gender,Employed,Age)%Probability)   ! Total the probabilities
               IF (Total>0.0) THEN                                   ! If there is any probability...
                  QCDCData(1,J,K,Gender,Employed,Age)%Probability=  &
                     QCDCData(1,J,K,Gender,Employed,Age)%           &
                     Probability/Total                               ! Convert to unit probability
                  DO L=2,ClusterNoOutW(J,K,Gender,Employed,Age)      ! Loop over diaries in pool
                     QCDCData(L,J,K,Gender,Employed,Age)%           &
                        Probability=QCDCData(L,J,K,Gender,Employed, &
                        Age)%Probability/Total+QCDCData(L-1,J,K,    &
                        Gender,Employed,Age)%Probability             ! Convert to cumulative probab.
                  ENDDO                                              ! Continue with next diary
               ENDIF                                                 ! End probability logic
	     END DO                                                      ! End cluster loop
      END DO                                                         ! End diary pool loop
      CALL RANDOM_SEED(Put=ISdAct2(1:KSeed))						 ! Set the random seed			
      CALL RANDOM_NUMBER(R22)										 ! Find 0-1 random number to select a chad record per cluster per pool
      CALL RANDOM_SEED(Get=ISdAct2(1:KSeed))						 ! Record the random seed
      DO K=1, NumPools 												 ! Loop over diaries in pool
         DO J=1, NumCluster                                          ! Loop over all clusters
            DO L=1, ClusterNoOutW(J,K,Gender,Employed,Age)           ! Loop over ClusterNoOutW
	           IF(R22(J,K) <QCDCData(L,J,K,Gender,Employed,Age)%Probability) THEN ! 
	              QSData(J,K)=QCDCData(L,J,K,Gender,Employed,Age)    ! Probability logic
	              EXIT	                                             ! Exit do loop
         	   END IF                                                ! Exit Probability logic
            END DO                                                   ! End ClusterNoOutW loop
         END DO                                                      ! End cluster loop
      END DO                                                         ! End diary pool loop
      ClusterTransM(:,:,:,:)=ClusterTransOut(:,:,:,:,Gender,Employed,Age) ! Setting ClusterTrans
      ClusterPRM(:,:)=ClusterPRMOut(:,:,Gender,Employed,Age)         ! Setting Cluster PRM
      IF (DebugLevel>0.AND.NumPools<=10.AND.LogProfiles==YES) THEN   ! If user wants diary counts...
         IF (P==1) WRITE(LogU,*) "Count of Suitable Diaries by Pool" ! Header for Profile summary 
         IF (P==1) WRITE(LogU,'(A26,10I6)')                            &
         " Profile Age Gender Employ",(J,J=1,NumPools)               ! Header for Profile summary
         WRITE(LogU,'(I8,I4,1X,A6,1X,A5,2X,10I6)')                     &
         P,Age,SEXTEXT(Gender),EMPTEXT(Employed),NumDiaries(:)       ! Put status message on log
      ENDIF                                                          ! End diary count logic
      IF (ANY(NumDiaries(:)==0)) CALL Fatal(1)                       ! Stop if a diary pool is empty
      IF (DebugLevel>1) WRITE(LogU,SL_FM2) LV_MS2, ProcName(ProcLevel) ! Message for debugging
      ProcLevel = ProcLevel-1	                                     ! Lowering ProcLevel
   END IF                                                            ! End ClusterdiaryA loop
   DEALLOCATE (DiSelID2, AccuFrac2, R22, DiSelIDM)      			 ! Deallocate the arrays declared in clustering
END SUBROUTINE DiaryProbabilities                                    ! Return to main APEX program


SUBROUTINE CompositeDiary(P)                                         ! Called from main APEX program
   INTEGER (KIND=IK), INTENT(IN) :: P                                ! Profile number (input)
   INTEGER (KIND=IK) :: I, J, K, Start, L, LOffset, Hour, HOffset    ! Local integer variables
   INTEGER (KIND=IK) :: Pool,Pool2,Cluster1,Cluster2                 ! Local variable for diary pool #   
   INTEGER (KIND=IK) :: NextBreak                                    ! Local integer variables 
   INTEGER (KIND=IK) :: NextEvent,TimeNew,NewEvents                  ! Variables for diary processing
   INTEGER (KIND=IK) :: LastHour, LastTS, TimeStep                   ! Variables for diary processing
   REAL    (KIND=RK) :: R(NumDays)                                   ! Automatic array for randoms
   REAL    (KIND=RK) :: AccuFrac1,AccuFrac2                          ! Temporary accumulative fraction   
   TYPE (EVENTSEQUENCE),TARGET,ALLOCATABLE :: TempSeq(:)             ! Event sequence for current profile
   ! Global constants: HOURSDAY, IK, LV_MS2, NO, RK, SL_FM2, ST_MS2
   ! Global vars used: DebugLevel, KSeed, NumDays, NumProfiles, Person, SList, TemData
   ! Global vars set : EventSeq, EventsPerson, EventStart, EventStop, HourSeq, ISdAct, NumEvents, 
   !                   ProcLevel, ProcName
   ! Module vars used: ActCount, ActTime, DData, PCount, QData
   ! Intrinsic procs : RANDOM_SEED, RANDOM_NUMBER, WRITE
   ! APEX procedures : MetsEval
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "CompositeDiary"                            ! Procedure name for messages
   IF (DebugLevel>1) WRITE(LogU,SL_FM2) ST_MS2, ProcName(ProcLevel)  ! Message for debugging
   Start                 = 0                                         ! Initialize first event offset
   EventSeq(:)%Day       = 0                                         ! Initialize day for each event 
   EventSeq(:)%Hour      = 0                                         ! Initialize hour for each event
   EventSeq(:)%Duration  = 0                                         ! Initialize duration per event
   EventSeq(:)%Activity  = 0                                         ! Initialize activity per event
   EventSeq(:)%ActCode   = 0                                         ! Initialize CHAD activity code
   EventSeq(:)%Location  = 0                                         ! Initialize location per event
   EventSeq(:)%HomeWork  = 0                                         ! Initialize home-work per event
   EventSeq(:)%Mets      = 0                                         ! Initialize METS value per event
   EventSeq(:)%UMET      = 0.                                        ! Initialize UMET value per event
   HourSeq(:)%FirstEvent = 0                                         ! Initialize first event per hour
   HourSeq(:)%LastEvent  = 0                                         ! Initialize last event per hour
   HourSeq(:)%Mets       = 0.                                        ! Initialize METS value per hour
   EventsPerson          = NO                                        ! Save events for this person
   EventStart            = 1                                         ! First event to record
   EventStop             = 0                                         ! Last event to record
   QData(:,:)%Selected   = 0                                         ! Initialize "selected" flags
   IF(ClustDiaryA==NO .AND. ClustDiaryB==NO) THEN					 ! Clustering Mechanism is bypased     
      CALL RANDOM_SEED(Put=ISdAct(1:KSeed))                          ! Set ACT random number seed
      IF (LongitDiary==YES) THEN                                     ! If longitudinal method is used
         CALL DiaryRanks                                             ! Determine ranks for each day
         R = DiaryRank                                               ! Set R vec to diary rank vec   
      ELSE                                                           ! Diary picks are random
         CALL RANDOM_NUMBER(R)                                       ! Get uniform randoms for each day
      ENDIF                                                          ! End of diary selection method 
      CALL RANDOM_SEED(Get=ISdAct(1:KSeed))                          ! Record ACT random seed setting
      DO J=1,NumDays                                                 ! Loop over days in simulation
         Pool = TemData(J,SList(Person(P)%HomeSec)%NumZ)%Pool        ! Pool for that person and day
         DO K=1,PCount(Pool)-1                                       ! Loop over diaries in pool
            IF (R(J)<QData(K,Pool)%Probability) EXIT                 ! Exit if R < cumulative probab
         ENDDO                                                       ! Try next diary in this pool
         QData(K,Pool)%Selected = 1                                  ! This diary is selected  
         PersonDay(J)%DiaryID=QData(K,Pool)%DiaryID                  ! Save the DiaryID for day    
         PersonDay(J)%DiaryAge =QData(K,Pool)%Age                    ! Save the diary age for day
         PersonDay(J)%DiaryEmp =QData(K,Pool)%Employed               ! Save the diary emp stat for day
         PersonDay(J)%Stat     =QData(K,Pool)%Statistic              ! Save the key statistic for day
         PersonDay(J)%DiaryOcc =QData(K,Pool)%Occupation             ! Save the diary occupation for day
         LOffset = QData(K,Pool)%FirstEvent-1                        ! Offset for start of diary
         HOffset = HOURSDAY*(J-1)+1                                  ! Offset for starting hour
         DO L=LOffset+1,QData(K,Pool)%LastEvent                      ! Loop over events in diary
            I    = Start+L-LOffset                                   ! I is event # on composite
            Hour = HOffset+DData(L)%Hour                             ! Hour is hour # on composite
            EventSeq(I)%Day      = J                                 ! Set day for each event
            EventSeq(I)%Hour     = Hour                              ! Set hour for each event 
            EventSeq(I)%Duration = DData(L)%Duration                 ! Set duration for each event
            EventSeq(I)%Activity = DData(L)%Activity                 ! Set activity for each event
            EventSeq(I)%Location = DData(L)%Location                 ! Set location for each event
            EventSeq(I)%HomeWork = DData(L)%HomeWork                 ! Set home-work for each event
            EventSeq(I)%ActCode  = DData(L)%ActCode                  ! Set CHAD activity code
            IF (HourSeq(Hour)%FirstEvent==0) HourSeq(Hour)%FirstEvent=I ! Set hour for first event
            HourSeq(Hour)%LastEvent=I                                ! Set hour for last event
         ENDDO                                                       ! Continue with next event 
         Start=Start+QData(K,Pool)%LastEvent-LOffSet                 ! Update the event offset
      ENDDO                                                          ! Continue with next day
      Person(P)%NumDiaries=SUM(QData(:,:)%Selected)                  ! Total number of diaries used
   ELSE ! IF (ClustDiary==YES .OR. ClustDiary1==YES) THEN			 ! Clustering Mechanism is activated
	  CALL RANDOM_SEED(Put=ISdAct(1:KSeed))                          ! Set ACT random number seed
      CALL RANDOM_NUMBER(R)                                          ! Get uniform randoms for each day
      CALL RANDOM_SEED(Get=ISdAct(1:KSeed))                          ! Record ACT random seed setting
      DO J=1,NumDays                                                 ! Loop over days in simulation
         Pool = TemData(J,SList(Person(P)%HomeSec)%NumZ)%Pool        ! Pool for that person and day
	     IF(J==1) THEN												 ! For the first day 
		    AccuFrac1=0.0                                            ! Setting AccuFrac1
		    DO K=1,3												 ! Loop through the clusters
			   AccuFrac1=AccuFrac1+ClusterPRM(Pool,K)				 ! Convert it to Accumulative fraction
			   IF(R(J)<AccuFrac1) THEN								 ! pick a cluster for the first day
			      Cluster1=K										 ! from frequency distribution
			      EXIT                                               ! exit once cluster is picked
			   END IF												 ! end of cluster pick end if block
		    END DO                                                   ! end of cluster pick for the first day
	  	 ELSE														 ! for all other days except the first day
	        AccuFrac2=0.0                                            ! Setting AccuFrac2
		    DO K=1,3												 ! loop through clusters		
			   AccuFrac2=AccuFrac2+ClusterTransM(Pool2,Cluster2,Pool,K) ! Covert to accumulative frac
			   IF(R(J)<AccuFrac2) THEN								 ! pick a cluster based on the transition probability
			      Cluster1=K										 ! pick a cluster for the current day
			      EXIT												 ! exit after cluster is picked
			   END IF												 ! end of cluster picking end if block
            END DO													 ! end of clustering loop	
 	     END IF														 ! end of cluster picking for the day 
	     LOffset = QSData(Cluster1,Pool)%FirstEvent-1                ! Offset for start of diary
         HOffset = HOURSDAY*(J-1)+1                                  ! Offset for starting hour  
         PersonDay(J)%DiaryID=QSData(Cluster1,Pool)%DiaryID          ! Save the DiaryID for day    
         PersonDay(J)%DiaryAge =QSData(Cluster1,Pool)%Age            ! Save the diary age for day
         PersonDay(J)%DiaryEmp =QSData(Cluster1,Pool)%Employed       ! Save the diary emp stat for day
         PersonDay(J)%Stat     =QSData(Cluster1,Pool)%Statistic      ! Save the key statistic for day         
         PersonDay(J)%DiaryOcc =QSData(Cluster1,Pool)%Occupation     ! Save the diary occupation for day
         DO L=LOffset+1,QSData(Cluster1,Pool)%LastEvent              ! Loop over events in diary
            I    = Start+L-LOffset                                   ! I is event # on composite
            Hour = HOffset+DData(L)%Hour                             ! Hour is hour # on composite
            EventSeq(I)%Day      = J                                 ! Set day for each event
            EventSeq(I)%Hour     = Hour                              ! Set hour for each event 
            EventSeq(I)%Duration = DData(L)%Duration                 ! Set duration for each event
            EventSeq(I)%Activity = DData(L)%Activity                 ! Set activity for each event
            EventSeq(I)%Location = DData(L)%Location                 ! Set location for each event
            EventSeq(I)%HomeWork = DData(L)%HomeWork                 ! Set home-work for each event
            EventSeq(I)%ActCode  = DData(L)%ActCode                  ! Set CHAD activity code            
            IF (HourSeq(Hour)%FirstEvent==0) HourSeq(Hour)%FirstEvent=I ! Set hour for first event
            HourSeq(Hour)%LastEvent=I                                ! Set hour for last event
         ENDDO                                                       ! Continue with next event 
         Start=Start+QSData(Cluster1,Pool)%LastEvent-LOffSet         ! Update the event offset
         Pool2=Pool													 ! remember previous pool#
         Cluster2=Cluster1											 ! remember previous cluster#
      ENDDO															 ! end of days looping
   END IF															 ! end of clustering diary selection
   NumEvents = Start                                                 ! # events for this composite
   DO I=1,NumEvents                                                  ! Loop over events in composite
      EventSeq(1)%Location = EventSeq(I)%Location                    ! Reset first location 
      IF (EventSeq(1)%Location/=-1) EXIT                             ! Exit when it is not missing
   ENDDO                                                             ! Continue with next event
   DO I=1,NumEvents                                                  ! Loop over events in composite
      EventSeq(1)%Activity = EventSeq(I)%Activity                    ! Reset first activity  
      IF (EventSeq(1)%Activity/=-1) EXIT                             ! Exit when it is not missing
   ENDDO                                                             ! Continue with next event
   DO I=1,NumEvents                                                  ! Loop over events in composite  
      EventSeq(1)%HomeWork = EventSeq(I)%HomeWork                    ! Reset first home-work value
      IF (EventSeq(1)%HomeWork/=-1) EXIT                             ! Exit when it is not missing 
   ENDDO                                                             ! Continue with next event
   DO I=2,NumEvents                                                  ! Loop over remaining events
      IF (EventSeq(I)%Location==-1)                                 &
         EventSeq(I)%Location = EventSeq(I-1)%Location               ! Set missing locs to previous
      IF (EventSeq(I)%Activity==-1)                                 & 
         EventSeq(I)%Activity = EventSeq(I-1)%Activity               ! Set missing acts to previous
      IF (EventSeq(I)%HomeWork==-1)                                 &
         EventSeq(I)%HomeWork = EventSeq(I-1)%HomeWork               ! Set missing HWs to previous
   ENDDO                                                             ! Continue with next event
   IF (ANY(EventSeq(1:NumEvents)%Location==-1)) CALL Fatal(2)        ! Stop if any location is missing
   IF (UseOccGroups==YES) THEN                                       ! Only if matching by groups
      DO I=1,NumEvents                                               ! Looping over all events
         IF (EventSeq(I)%ActCode==10900) EventSeq(I)%Activity=      &
            GroupMETS(Person(P)%FactorGroup)                         ! If worktime, set to profile occ
         IF (EventSeq(I)%ActCode==18930 .OR. EventSeq(I)%ActCode==  &
            18940) THEN                                              ! If code is missing
            DO K=1,24                                                ! Loop over last events
               IF (I-K < 1) EXIT                                     ! Exit if beginning of sequence
               IF (EventSeq(I-K)%ActCode/=18930 .AND.               &
                  EventSeq(I-K)%ActCode/=18940) EXIT                 ! Exit if non-missing found
            ENDDO                                                    ! End finding missing loop
            IF (EventSeq(I-K)%ActCode==10900) EventSeq(I)%Activity= &
               GroupMETS(Person(P)%FactorGroup)                      ! Assign missing dist
         ENDIF                                                       ! End missing code logic
      ENDDO                                                          ! End event loop
   ENDIF                                                             ! End OccGroup logic
   CALL METSEval                                                     ! Evaluate METS for all activities
   CALL FEVEEval(MIN(Person(P)%Age,100),Person(P)%Gender)            ! Evaluate FEV E's for all activities
   IF (P==NumProfiles.AND.DebugLevel>0) THEN                         ! When last profile is complete
      WRITE(LogU,*)                                                  ! Put blank line on log
      WRITE(LogU,*)"METS Categories - Cumulative over Simulation"    ! Write activity summary to log
      WRITE(LogU,*)"     Distrib#    #events    #minutes"            ! Column headers for table
      DO I=-128,127                                                  ! Loop over activity categories
         IF (ActCount(I)>0) WRITE(LogU,*) I,ActCount(I),ActTime(I)   ! Only write nonzero data
      ENDDO                                                          ! Continue with next activity 
      WRITE(LogU,*) "Finished METS Categories"
   ENDIF                                                             ! End of activity summary logic
   IF (TimeStepsPerHour > 1) THEN                                    ! Reprocess diary for timesteps
      ALLOCATE (TempSeq(NumDays*EVENTSDAY),STAT=AllocErr)            ! Space for temporary events 
      CALL CheckAllocation(AllocErr)                                 ! Check the success of the alloc
      NextBreak    = 60/TimeStepsPerHour                             ! Intialize event break location
      TimeNew = 0                                                    ! Running time on new diary   
      NextEvent = 0                                                  ! Init time of next event
      NewEvents = 0                                                  ! Init num events in new diary
      DO I=1, NumEvents                                              ! Loop over events, calc # ts 
         NextEvent = NextEvent + EventSeq(I)%Duration                ! Calculate time of next event       
         IF (NextBreak <= NextEvent) THEN                            ! If need a new event break
            DO WHILE (NextBreak <= NextEvent)                        ! Add new event breaks
               NewEvents          = NewEvents+1                      ! Update # of events
               TempSeq(NewEvents) = EventSeq(I)                      ! Copy old event values
               TempSeq(NewEvents)%Duration = NextBreak-TimeNew       ! Duration for this event
                   TimeNew = TimeNew + TempSeq(NewEvents)%Duration   ! Update time on new events   
               NextBreak=NextBreak + 60/TimeStepsPerHour             ! Next break in events
            ENDDO                                                    ! End adding new breaks
            IF (TimeNew<NextEvent) THEN                              ! Piece left before next event
               NewEvents          = NewEvents+1                      ! Update # of events
               TempSeq(NewEvents) = EventSeq(I)                      ! Copy old event values
               TempSeq(NewEvents)%Duration = NextEvent-TimeNew       ! Duration of final piece
            ENDIF                                                    ! End leftover logic    
         ELSE                                                        ! Else if don't need break(s)
            NewEvents          = NewEvents+1                         ! Update # of events
            TempSeq(NewEvents) = EventSeq(I)                         ! Copy old event values
         ENDIF                                                       ! End break logic
         TimeNew = NextEvent                                         ! Update running time (new diary)   
      ENDDO                                                          ! End loop over events
      EventSeq=TempSeq                                               ! Copy the new events to EventSeq
      NumEvents=NewEvents                                            ! Update # events
      TSSeq(:)%FirstEvent = 0                                        ! Initialize first event of ts
      TSSeq(:)%LastEvent  = 0                                        ! Initialize last event of ts
      Hour     = 0                                                   ! Init hour of simulation  
      LastHour = 0                                                   ! Init last hour 
      DO I=1,NumEvents                                               ! Loop events, update HourSeq   
         Hour = EventSeq(I)%Hour                                     ! Hour of simulation
         IF (Hour .NE. LastHour) THEN                                ! If this is new hour
            HourSeq(Hour)%FirstEvent=I                               ! Set first event for hour
            IF (LastHour>0) HourSeq(LastHour)%LastEvent=I-1          ! Set last event for prev hour
         ENDIF                                                       ! End new hour logic
        LastHour = Hour                                              ! Update last hour              
      ENDDO                                                          ! End loop over events
      HourSeq(Hour)%LastEvent=NumEvents                              ! Last event for last hour                         
   ENDIF                                                             ! End timestep logic
   TimeNew  = 0                                                      ! Init running minutes on diary   
   LastTS   = 0                                                      ! Init last TS 
   TimeStep = 1                                                      ! Initialize timestep 
   DO I=1,NumEvents                                                  ! Loop events, update TSSeq   
      EventSeq(I)%TS=TimeStep                                        ! Timestep for this event       
      Hour = EventSeq(I)%Hour                                        ! Hour of simulation
      TimeNew = TimeNew+EventSeq(I)%Duration                         ! Running minutes on diary 
      IF (TimeStep .NE. LastTS) THEN                                 ! If this is new ts
         TSSeq(TimeStep)%Hour = Hour                                 ! Set the hour for this ts
         TSSeq(TimeStep)%FirstEvent = I                              ! Set first event for ts
         IF (LastTS>0) TSSeq(LastTS)%LastEvent=I-1                   ! Set last event for prev ts
      ENDIF                                                          ! End new ts logic  
      LastTS=Timestep                                                ! Update last timestep 
      IF (TimeNew>=TSMinutes.AND. MOD(TimeNew,TSMinutes)==0) THEN    ! When hitting a new TS 
         TimeStep=TimeStep+1                                         ! Update timestep   
      ENDIF                                                          ! End timestep logic
   ENDDO                                                             ! End loop over events
   TSSeq(TimeStep-1)%LastEvent=NumEvents                             ! Last event for last ts   
   DO I=1,NumHours                                                   ! Loop over all hours
      IF (HourSeq(I)%LastEvent==0) CALL Fatal(1)                     ! If the last event = 0, stop
   ENDDO                                                             ! End debugging code
   IF (TimeStepsPerHour > 1) DEALLOCATE(TempSeq)                     ! Deallocate internal arrays
   IF (DebugLevel>1) WRITE(LogU,SL_FM2) LV_MS2, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END SUBROUTINE CompositeDiary                                        ! Return to main APEX program

SUBROUTINE METSEval                                                  ! Called from CompositeDiary
   INTEGER (KIND=IK) :: MetsNum, I                                   ! Input variable declaration
   REAL    (KIND=RK)          ::  METS(1)                            ! Local real variables
   ! Global constants: IK, LV_MS3, RK, SL_FM3, ST_MS3
   ! Global vars used: DebugLevel, KSeed, NumEvents
   ! Global vars set : EventSeq, IsdMETS, ProcLevel, ProcName
   ! Module constants: POINT, UNIFORM, NORMAL, LOGNORMAL, TRIANGLE, EXPONENTIAL
   ! Module vars used: MetsDist
   ! Module vars set : ActCount, ActTime
   ! Intrinsic procs : EXP, LOG, SQRT, RANDOM_NUMBER, RANDOM_SEED, WRITE
   ! APEX procedures : Fatal, RnNor
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "METSEval"                                  ! Procedure name for messages
   IF (DebugLevel>1) WRITE(LogU,SL_FM3) ST_MS3, ProcName(ProcLevel)  ! Message for debugging
   CALL RANDOM_SEED(Put=ISdMETS(1:KSeed))                            ! Set METS random number seed
   DO I=1,NumEvents                                                  ! Loop over events on diary
      MetsNum = EventSeq(I)%Activity                                 ! Get METS distribution number  
      ActCount(MetsNum)=ActCount(MetsNum)+1                          ! Add diary to activity counts
      ActTime(MetsNum) =ActTime(MetsNum)+EventSeq(I)%Duration        ! Add to accumuated duration  
      CALL SampleDist(MetsDist(MetsNum),METS)                        ! Sample from appropriate dist
      EventSeq(I)%METS = METS(1)                                     ! Copy value to global variable
      EventSeq(I)%UMET = METS(1)                                     ! Copy value to global variable
   ENDDO                                                             ! Continue with next event
   IF (DebugLevel>1) WRITE(LogU,SL_FM3) LV_MS3, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END SUBROUTINE METSEval                                              ! Return to CompositeDiary

SUBROUTINE DiaryRanks                                                ! Called from CompositeDiary  
   INTEGER(KIND=IK) :: K, J, JJ, L, M, N, SN                         ! Local integer variables
   REAL (KIND=RK)   :: D, S, V, T, Tmp, Next, Last, Z, Width         ! Local real variables
   REAL (KIND=LONG) :: P, Q                                          ! Local double precision vars
   ! Global vars used: AutoC, DiaryD, NumDays
   ! Global vars set : ProcLevel, ProcName
   ! Module vars used: NumX
   ! Module vars set : DiaryRank, XPicks, Picked, Rn
   ! Intrinsic procs : ABS, CEILING, EXP, FLOOR, LOG, MAX, MIN, REAL, SIGN, SQRT, RANDOM_NUMBER 
   ! APEX procedures : BetaInv
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "DiaryRanks"                                ! Procedure name for messages
   D      = MIN(MAX((NumDays*DiaryD-1.)/(NumDays-1.),0.),0.999)      ! Adjusted target diversity D
   CALL RANDOM_NUMBER(Rn)                                            ! Fill uniform random array   
   T  = RN(0)*SQRT(D) + (1-SQRT(D))/2                                ! Pick T from dist  
   P  = 2.*T/(1.-D)                                                  ! First parm for indv T dist
   Q  = 2.*(1.-T)/(1.-D)                                             ! Second parm for indv T dist
   DO J = 1, NumX                                                    ! Loop over # picks
      XPicks(J) = BetaInv(Rn(J),P,Q)                                 ! Pick from indiv. beta
   ENDDO                                                             ! End loop over picks  
   M = NumX                                                          ! Init M to num picks
   DO K = 1,FLOOR(LOG(REAL(NumX))/LOG(2.)+1.E-8)                     ! Start Shell sort
      M = M/2                                                        ! Set M 
      DO J=1,NumX-M                                                  ! Start outer loop 
         DO JJ=J, 1, -M                                              ! Start inner loop
            L = JJ+M                                                 ! Set L
            IF (XPicks(L) >= XPicks(JJ)) EXIT                        ! Compare XPicks values
            Tmp           =  XPicks(JJ)                              ! Swap 
            XPicks(JJ)    =  XPicks(L)                               ! Swap 
            XPicks(L)     =  Tmp                                     ! Swap
         ENDDO                                                       ! End inner loop
      ENDDO                                                          ! End outer loop 
   ENDDO                                                             ! End shell sort  
   Picked(:) = 0                                                     ! Initialize flag vector
   CALL RANDOM_NUMBER(Rn)                                            ! Fill uniform random array
   Width = MIN(2.-2.*(ABS(AutoC)),1.)                                ! Range of A   
   V = AutoC + Width*(BetaInv(Rn(0),2.625D0,2.625D0)-0.5)            ! Personal target for A   
   S = 2./(1.-V*V)                                                   ! Sum of Beta parameters  
   N = CEILING(Rn(1)*REAL(NumX))                                     ! Pick for first day
   Picked(N) = 1                                                     ! Mark pick as used
   DiaryRank(1) = XPicks(N)                                          ! Record first pick
   Last = (REAL(N)-0.5)/NumX                                         ! Set fractional rank
   DO J = 2, NumDays                                                 ! Loop over remaining days
      P    = S/2. - S*V/2. + S*V*Last                                ! First shape parameter
      Q    = S/2. + S*V/2. - S*V*Last                                ! Second shape parameter
      Next = BetaInv(Rn(J),P,Q)                                      ! Pick next score
      Z    = Next*REAL(NumX)                                         ! Set pick index
      N    = CEILING(Z)                                              ! Make index an integer
      sn   = SIGN(1,CEILING(REAL(N)-Z-0.5))                          ! Choose initial sign
      DO K = 1, 2*NumX                                               ! Loop enough times
         IF (N>=1 .AND. N<=NumX) THEN                                ! If N is a valid value
            IF (Picked(N)==0) EXIT                                   ! Exit loop if N picked
         ENDIF                                                       ! End if valid N   
         N  = N+K*sn                                                 ! Try next closest N1
         sn = -sn                                                    ! Change sign
      ENDDO                                                          ! End index search loop
      Picked(N)    = 1                                               ! Mark score as picked
      DiaryRank(J) = XPicks(N)                                       ! Current rank
      Last         = (REAL(N)-0.5)/NumX                              ! Set fractional rank
   ENDDO                                                             ! End loop over days
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END SUBROUTINE DiaryRanks                                            ! Return to calling routine


SUBROUTINE CLUSTERING												 ! Call from DiaryProbabilities
   INTEGER(KIND=IK) :: K, K2,J,J2, JJ,JJ2, J4,J5,L, M, N, SN		 ! Local integer variables
   INTEGER(KIND=IK) :: I,II,III,indic,INDX,Mh						 ! Local integer variables
   INTEGER(KIND=IK) :: FirstEvent, LastEvent,JSelect1,Jselect2		 ! Local integer variables
   INTEGER(KIND=IK) :: exitsign, exitsign2,ncluster, ncluster2		 ! Local integer variables
   INTEGER(KIND=IK) :: NumMicros2 =5								 ! # of the micros							
   INTEGER(KIND=IK) :: AR1(2)=0,zeros(3)							 ! Local integer variables
   INTEGER(KIND=IK)	:: NumCluster=3									 ! # of the clusters
   INTEGER(KIND=IK) :: IDLTNUM, IDLT2NUM                             ! Declare local variables   
   REAL (KIND=LONG),ALLOCATABLE :: AccuFrac(:,:,:)					 ! Accumulative fraction of records in each cluster
   REAL (KIND=LONG),ALLOCATABLE :: R2(:,:)							 ! Local integer variables
   INTEGER (KIND=IK), ALLOCATABLE :: CLUSTERSN(:,:)                  ! Allocate cluster array
   INTEGER (KIND=IK), ALLOCATABLE :: MicroMinutes(:,:,:)             ! Allocate minutes in micro array
   INTEGER (KIND=IK), ALLOCATABLE :: ClusterID(:,:,:),ClusterID2(:,:,:) ! Allocate ClusterID arrays
   INTEGER (KIND=IK), ALLOCATABLE :: ClusterNo(:,:),ClusterNo2(:,:)  ! Allocate Cluster Number Arrays
   INTEGER (KIND=IK), ALLOCATABLE :: DiSelID (:,:),CLUSTERTRAN(:,:,:,:) ! Allocating local Arrays
   REAL (KIND=RK), ALLOCATABLE :: DISTANCE(:,:,:),DISTANCE2(:,:)     ! Allocate Distance Arrays
   REAL (KIND=LONG), ALLOCATABLE :: CLUSTERPR(:,:),CLUSTERTRAN2(:,:,:,:) ! Allocating local Arrays
   REAL (KIND=RK) :: minDistance                                     ! Declaring local variables
   REAL (KIND=RK) :: Dist1,Dist2,Dist3                               ! Declaring local variables
   REAL (KIND=RK) :: totalcls,totalprob,Totalclufrac                 ! Declaring local variables
   CHARACTER (LEN=26):: ALPHATC='ABCDEFGHIJKLMNOPQRSTUVWXYZ'         ! Declaring local variables
   CHARACTER (LEN=2) :: ALPHATC2(52)                                 ! Declaring local variables
   CHARACTER (LEN=10):: NUMTC='0123456789'                           ! Declare local variables   
   CHARACTER (LEN=1) :: IDLT                                         ! Declaring local variables
   ALPHATC2(1) = 'A'                                                 ! Populating alphabet array
   ALPHATC2(2) = 'B'                                                 ! Populating alphabet array
   ALPHATC2(3) = 'C'                                                 ! Populating alphabet array
   ALPHATC2(4) = 'D'                                                 ! Populating alphabet array
   ALPHATC2(5) = 'E'                                                 ! Populating alphabet array
   ALPHATC2(6) = 'F'                                                 ! Populating alphabet array
   ALPHATC2(7) = 'G'                                                 ! Populating alphabet array
   ALPHATC2(8) = 'H'                                                 ! Populating alphabet array
   ALPHATC2(9) = 'I'                                                 ! Populating alphabet array
   ALPHATC2(10) = 'J'                                                ! Populating alphabet array
   ALPHATC2(11) = 'K'                                                ! Populating alphabet array
   ALPHATC2(12) = 'L'                                                ! Populating alphabet array
   ALPHATC2(13) = 'M'                                                ! Populating alphabet array
   ALPHATC2(14) = 'N'                                                ! Populating alphabet array
   ALPHATC2(15) = 'O'                                                ! Populating alphabet array
   ALPHATC2(16) = 'P'                                                ! Populating alphabet array
   ALPHATC2(17) = 'Q'                                                ! Populating alphabet array
   ALPHATC2(18) = 'R'                                                ! Populating alphabet array
   ALPHATC2(19) = 'S'                                                ! Populating alphabet array
   ALPHATC2(20) = 'T'	                                             ! Populating alphabet array
   ALPHATC2(21) = 'U'                                                ! Populating alphabet array
   ALPHATC2(22) = 'V'                                                ! Populating alphabet array
   ALPHATC2(23) = 'W'                                                ! Populating alphabet array
   ALPHATC2(24) = 'X'                                                ! Populating alphabet array
   ALPHATC2(25) = 'Y'                                                ! Populating alphabet array
   ALPHATC2(26) = 'Z'                                                ! Populating alphabet array
   ALPHATC2(27) = 'AA'                                               ! Setting alphabet array
   ALPHATC2(28) = 'BB'                                               ! Setting alphabet array
   ALPHATC2(29) = 'CC'                                               ! Setting alphabet array
   ALPHATC2(30) = 'DD'                                               ! Setting alphabet array
   ALPHATC2(31) = 'EE'                                               ! Setting alphabet array
   ALPHATC2(32) = 'FF'                                               ! Setting alphabet array
   ALPHATC2(33) = 'GG'                                               ! Setting alphabet array
   ALPHATC2(34) = 'HH'                                               ! Setting alphabet array
   ALPHATC2(35) = 'II'                                               ! Setting alphabet array
   ALPHATC2(36) = 'JJ'                                               ! Setting alphabet array
   ALPHATC2(37) = 'KK'                                               ! Setting alphabet array
   ALPHATC2(38) = 'LL'                                               ! Setting alphabet array												
   ALPHATC2(39) = 'MM'                                               ! Setting alphabet array
   ALPHATC2(40) = 'NN'                                               ! Setting alphabet array
   ALPHATC2(41) = 'OO'                                               ! Setting alphabet array
   ALPHATC2(42) = 'PP'                                               ! Setting alphabet array
   ALPHATC2(43) = 'QQ'                                               ! Setting alphabet array
   ALPHATC2(44) = 'RR'                                               ! Setting alphabet array
   ALPHATC2(45) = 'SS'                                               ! Setting alphabet array
   ALPHATC2(46) = 'TT'                                               ! Setting alphabet array
   ALPHATC2(47) = 'UU'                                               ! Setting alphabet array
   ALPHATC2(48) = 'VV'                                               ! Setting alphabet array
   ALPHATC2(49) = 'WW'                                               ! Setting alphabet array
   ALPHATC2(50) = 'XX'                                               ! Setting alphabet array
   ALPHATC2(51) = 'YY'                                               ! Setting alphabet array
   ALPHATC2(52) = 'ZZ'                                               ! Setting alphabet array	   
   NumMicros2=maxval(DData(:)%Location2)							 ! Find the # of micros defined for clustering 
   ALLOCATE (MicroMinutes(MaxVal(NumDiaries(:)),NumPools,NumMicros2))! Allocate the space for the arrays
   ALLOCATE (ClusterNo(MaxVal(NumDiaries(:)),NumPools))              ! Allocating arrays
   ALLOCATE (ClusterNo2(NumCluster,NumPools))                        ! Allocating arrays
   ALLOCATE (ClusterSN(MaxVal(NumDiaries(:)),NumPools))              ! Allocating arrays
   ALLOCATE (ClusterID(MaxVal(NumDiaries(:)),NumPools,MaxVal(NumDiaries(:)))) ! Allocating arrays
   ALLOCATE (ClusterID2(NumCluster,NumPools,MaxVal(NumDiaries(:))))  ! Allocating arrays
   ALLOCATE (DISTANCE(MaxVal(NumDiaries(:)),NumPools,MaxVal(NumDiaries(:)))) ! Allocating arrays
   ALLOCATE (DISTANCE2(MaxVal(NumDiaries(:)),MaxVal(NumDiaries(:)))) ! Allocating arrays
   ALLOCATE (CLUSTERPR(NumPools,NumCluster),                        &                       
   		     CLUSTERTRAN(NumPools,NumCluster,NumPools,NumCluster))   ! Allocating arrays
   ALLOCATE (CLUSTERTRAN2(NumPools,NumCluster,NumPools,NumCluster))  ! Allocating arrays
   ALLOCATE (R2(NumCluster,NumPools),DiSelID(NumCluster,NumPools))   ! Allocating arrays
   ALLOCATE (AccuFrac(NumCluster,NumPools,MaxVal(NumDiaries(:))))	 ! Allocate the space for the arrays
   MicroMinutes=0													 ! Initialize the array for minutes in micros defined for clustering
   AccuFrac=0.0														 ! Initialize the accumulative fraction array for clustering
! (1) Calculate the minutes in each micros for each matched chad record
   DO K=1, NumPools												     ! Loop through pool#
	  DO J=1, NumDiaries(K)										     ! Loop through NonZero chad records in each pool
		 FirstEvent = QCData(J,K)%FirstEvent						 ! Find the first event in the chad record
		 LastEvent = QCData(J,K)%LastEvent						     ! Find the last event in the chad record 
		 DO I=FirstEvent,LastEvent								     ! Loop through all the events in the chad record
			IF(DData(I)%Location2==-1)                              &                        
			   DData(I)%Location2=DData(I-1)%Location2			     ! Use the previous chad record if the location locode is unknown
   		    IF(DData(I)%Location2 > 0) THEN                          ! Only if loc is in ClustMap		   
			    MicroMinutes(J,K,DData(I)%Location2) =              &           
			    MicroMinutes(J,K,DData(I)%Location2)+ DData(I)%Duration  ! Sum minutes in each cluster micro 
			ENDIF                                                    ! End conditional			
		 END DO													     ! End of Event loop
!		 IF(sum(MicroMinutes(J,K,:))/=1440) THEN					 ! Check if total minutes must be equal to 1440
!			WRITE(*,*) 'ErrorTotal Minutes /= 1440 minutes'          ! Error message
!			STOP													 ! Program will stop
!		 END IF													     ! End of total minutes check
	  END DO														 ! End of NonZero chad record loop
   END DO															 ! End of pool loop (end of step 1)
! (2) Calculate the distances among chad records using the right upper (above diagonal) part of array only
   DISTANCE = -1													 ! Initialize the distance to -1
   ClusterNo = 1													 ! Initiallize the # of cluster to 1
   ClusterID = -99													 ! Initialize the cluster id to -99
   ClusterSN = -99                                                   ! Initialize the cluster SN to -99
   DO K=1,NumPools													 ! Begining of pool loop
	  DO J=1, NumDiaries(K)										     ! Loop through chad records in each pool
		 ClusterID(J,K,1)=J										     ! Assign sequence # of chad records to the first element of the array
		 DO JJ=J+1,NumDiaries(K)									 ! Only calculate the distances on the upper part of the array (above the diagonal)
			DISTANCE(J,K,JJ)=DISTANCE(J,K,JJ)+1				         ! Initialize it to zero(-1+1=0)
			DO M=1, NumMicros2								         ! Loop through micros
		 	   DISTANCE(J,K,JJ)=DISTANCE(J,K,JJ)+                   &                
			   (MicroMinutes(J,K,M)-MicroMinutes(JJ,K,M))**2./2.     ! Calculate the distance
			END DO											         ! End of micros loop
		 END DO													     ! End of destination chad
	  END DO														 ! End of home chad loop
   END DO															 ! End of pool loop	(end of step 2
! (3) Grouping the chad records in each pool into three clusters
   DO K=1,NumPools													 ! Loop through pools
	  ncluster=NumDiaries(K)										 ! Beginning # of clusters = # of chad in a pool
	  DO WHILE (ncluster>3)										     ! Loop till # of clusters=3
		 Distance2=Distance(:,K,:)								     ! Assign the 3-dimen distance to 2-dimen array
		 AR1=MINLOC(Distance2,Mask = Distance2 >-1.0)			     ! Find the location of minimum (>-1) value in the array
		 JSelect1 = AR1(1)										     ! The array location  subscript 1 = # of home chad record
		 JSelect2 = AR1(2)										     ! The array location  subscript 2 = # of destination chad record
		 DO J4=1, ClusterNo(JSelect2,K)							     ! Transfer all CHAD IDs in ClusterJSelect2 to Cluster JSelect1)
		    J5=ClusterNo(JSelect1,K)+J4							     ! Index for Cluster JSelect1
		    ClusterID(JSelect1,K,J5)=ClusterID(JSelect2,K,J4)        ! Transfer ID from Cluster JSelect2 to JSelect1
		    ClusterID(JSelect2,K,J4) = -99						     ! Assign -99 to ID in eliminated Cluster JSelect2			
		 END DO													     ! End of transfer loop
		 ncluster=ncluster-1										 ! Eliminate one cluster for the loop
		 DO JJ=1, NumDiaries(K)									     ! Loop through the chad records in the pool
		    IF(clusterNo(JJ,K)==0) CYCLE						     ! Skip if cluster JJ is empty
		    IF(JJ==JSelect1) CYCLE								     ! Skip as the JSelect1 to JSelect1 distance is not needed
		    IF(JJ==JSelect2) CYCLE								     ! Skip as Cluster JSelect2 is just eliminated
		    IF(JJ>JSelect1) THEN							         ! Horizonal element in distance array for Jselect1
		  	   Dist1 = (ClusterNo(JJ,K)+ClusterNo(JSelect1,K))      &    
		 	   *Distance(JSelect1,K,JJ)						         ! Part 1 of new distance
		       IF(JJ>JSelect2) THEN						             ! Horizonal element in the distance array for Jselect2
				  Dist2 = (ClusterNo(JJ,K)+                         &                      
				  ClusterNo(JSelect2,K))*Distance(JSelect2,K,JJ)     ! Part 2 of new distance
			   ELSE										             ! Vertical element in the distance array for Jselect2
				  Dist2 = (ClusterNo(JJ,K)+                         &                    
				  ClusterNo(JSelect2,K))*Distance(JJ,K,JSelect2)     ! Part 2 of new distance
			   END IF                                                ! End if
			   Dist3 =  ClusterNo(JJ,K)*Distance(JSelect1,K,JSelect2)! Part 3 of new distance
 			   Distance(JSelect1,K,JJ) = (Dist1+Dist2-Dist3)        & 
   			   /(ClusterNo(JJ,K)+ClusterNo(JSelect1,K)+             &  
			   ClusterNo(JSelect2,K))							     ! New distance
			ELSE IF(JJ<JSelect1) THEN						         ! Vertical element in the distance array for Jselect1
			   Dist1 = (ClusterNo(JJ,K)+ClusterNo(JSelect1,K))&    
			   *Distance(JJ,K,JSelect1)                              ! Part 1 of new distance
			   IF(JJ>JSelect2) THEN						             ! Horizonal element in the distance array for Jselect2
			 	  Dist2 = (ClusterNo(JJ,K)+                         &                        
				  ClusterNo(JSelect2,K))*                           &                         
				  Distance(JSelect2,K,JJ)					         ! Part 2 of new distance
			   ELSE									                 ! Vertical element in the distance array for JSelect2
				  Dist2 = (ClusterNo(JJ,K)+                         &                        
				  ClusterNo(JSelect2,K))*                           &                         
				  Distance(JJ,K,JSelect2)					         ! Part2 of 2 of new distance
			   END IF                                                ! end if
			   Dist3 =  ClusterNo(JJ,K)*Distance(JSelect1,K,JSelect2)! Part3 of new distance
			   Distance(JJ,K,JSelect1) = (Dist1+Dist2-Dist3)/       &      
			   (ClusterNo(JJ,K)+	ClusterNo(JSelect1,K)           &         
			   +ClusterNo(JSelect2,K))			                     ! New Distance
			END IF                                                   ! End of the array index adjustment for Jselect1
	     END DO													     ! End of new distance calculation for new cluster Jselect1
		 ClusterNo(JSelect1,K) = ClusterNo(JSelect1,K)+             &         
			              ClusterNo(JSelect2,K)			             ! Combine the # of chad records in cluster Jselect1 and Jselect2 into Jselect1
		 ClusterNo(JSelect2,K) = 0								     ! Make the # of chad records zero in cluster Jselect2
		 Distance(JSelect2,K,:)=-1.0								 ! Make the distance to -1 of JSelect2 to other clusters 
		 Distance(:,K,JSelect2)=-1.0								 ! Make the distance to -1 of JSelect2 to other clusters 
      END DO														 ! End of clustering for chad records in pool K
	  IF (ncluster/=3) THEN										     ! Make sure that the chad records are grouped into 3 clusters
		 WRITE(*,*) 'Error in clustering'                            ! Printing error
		 WRITE(*,*) 'Please increase the AgeCutPct in Parameter File' ! The problem may be due to two small age window specified in the parameter file
		 STOP                                                        ! If error, then end
	  END IF                                                         ! End error checking
	  12222 FORMAT(3I5,500I6)                                        ! Formatting
	  ncluster2=0													 ! Initialize the cluster #
	  totalcls=0													 ! Initialize the variable
	  DO J = 1, NumDiaries(K)										 ! Loop through chad records or initial clusters in pool K
		 IF(ClusterNo(J,K)/=0) THEN								     ! Identify one of 3 clusters that have been grouped
		    ncluster2=   ncluster2+1							     ! increment of clusters 1-3	
			ClusterPR(K,ncluster2)=real(ClusterNo(J,K))			     ! Transfer the # of chad records in each cluster into a new array
			ClusterNo2(ncluster2,K)=ClusterNo(J,K)				     ! Transfer the # of chad records in each cluster into a new array
			ClusterID2(ncluster2,K,:)=ClusterID(J,K,:)			     ! Transfer the chad id (sequence # in QCData) to an new array
			totalcls=totalcls+real(ClusterNo(J,K))                   ! Total # of chad records in the pool
			DO J4=1,ClusterNo(J,K)								     ! loop through chad records in each cluster
			   AccuFrac(ncluster2,K,J4) =                           &                       
   			   QCData(ClusterID(J,K,J4),K)%Probability			     ! Transfer the probability for each chad record in the (con't)
			   QCData(ClusterID(J,K,J4),K)%Cluster=ncluster2         ! cluster to a new array
			END DO												     ! End of the loop
			Totalclufrac=sum(AccuFrac(ncluster2,K,1:ClusterNo2(ncluster2,K))) ! sum the probability of chad records in ncluster2
			AccuFrac(ncluster2,K,1)=AccuFrac(ncluster2,K,1)/Totalclufrac ! calculate the accumulative fraction for chad record 1
			DO J4=2,ClusterNo2(ncluster2,K)						     ! loop through chad records in the cluster
			   AccuFrac(ncluster2,K,J4)=AccuFrac(ncluster2,K,J4-1)+ & 
			           AccuFrac(ncluster2,K,J4)/Totalclufrac		 ! calculate the accumulative fraction for each chad record in the cluster
			END DO												     ! End of accumulative fraction loop
	     END IF													     ! End of cluster identification
      END DO														 ! End ofloop for searching clusters
	  ClusterPR(K,:)=ClusterPR(K,:)/totalcls						 ! Calculate the distribution frequency of chad records in three clusters
   END DO															 ! End of clustering (end of step 3)
! (4) Randomly select a chad record for each pool/cluster combination
   CALL RANDOM_SEED(Put=ISdAct2(1:KSeed))							 ! Set the random seed			
   CALL RANDOM_NUMBER(R2)											 ! Find 0-1 random number to select a chad record per cluster per pool
   CALL RANDOM_SEED(Get=ISdAct2(1:KSeed))							 ! Record the random seed
   DO K=1, NumPools													 ! Loop through pool #
	  DO J=1, NumCluster											 ! Loop through cluster #
		 DO J4=1, ClusterNo2(J,K)									 ! Loop through chad records in cluster
			IF (R2(J,K)<AccuFrac(J,K,J4)) THEN						 ! Identify the chad record whose accumu. frac> R
			   DiSelID(J,K)=ClusterID2(J,K,J4)						 ! Identify a chad record from each cluster
			   EXIT													 ! exit
			END IF													 ! End of selection for one cluster
		 END DO														 ! End of chad record loop
	  END DO														 ! End of cluster loop
   END DO															 ! End of pool loop (end of step 4)
! (5) Calculate the transition counts from (poolA,clusterA,poolB,clusterB)
   ClusterTran=0													 ! Initialize the transition probability array
   DO K=1, NumPools													 ! Loop through pools
  	  DO J=1, NumCluster											 ! Loop through clusters
		 DO JJ=1, ClusterNo2(J,K)									 ! Loop through chad records in each cluster			
			DO K2=1, NumPools										 ! Loop through pools again for cross transition probability
			   DO J2=1, NumCluster									 ! Loop through clusters again
			      DO JJ2=1,ClusterNo2(J2,K2)						 ! Loop through chad records in each cluster again
                     IF (LEN_TRIM(QCData(ClusterID2(J,K,JJ),K)%DiaryID)==8) THEN ! If length of CHADID is 8 characters                   
			            IF(QCData(ClusterID2(J,K,JJ),K)%DiaryID(1:7)    & 
				        ==QCData(ClusterID2(J2,K2,JJ2),K2)%DiaryID(1:7)) THEN ! Identify chad records with the same 1:8 characters in ID in home cluster to destination cluster
			                IDLT=QCData(ClusterID2(J,K,JJ),K)%DiaryID(8:8) ! Identify the 9th character in ID of the record in home cluster
				            INDX=index(Alphatc,IDLT)			     ! Identify the sequence# in aphabetic sequence
				            IF(QCData(ClusterID2(J2,K2,JJ2),K2)%DiaryID(8:8)& 
				      	    ==ALPHATC2(INDX+1)) THEN				 ! if the ninth character in destination cluster is the letter in alphabet subsequent to the 9th character in home cluster
				                ClusterTran(K,J,K2,J2) = ClusterTran(K,J,K2,J2)+1 ! Record the matched records in home to desti clusters
				            END IF									 ! end of the ninth character check
			            END IF										 ! end of the 1:8 characters ID check
                     ELSE IF (index(NUMTC,QCData(ClusterID2(J,K,JJ),K)%DiaryID(9:9))) THEN ! If last value in CHADID is a number
			            IF(QCData(ClusterID2(J,K,JJ),K)%DiaryID(1:6) & 
				        ==QCData(ClusterID2(J2,K2,JJ2),K2)%DiaryID(1:6)) THEN ! Identify chad records with the same 1:8 characters in ID in home cluster to destination cluster
			                READ(QCData(ClusterID2(J,K,JJ),K)%DiaryID(7:9),'(i10)') IDLTNUM ! Identify the number in ID of the record in home cluster               
			                READ(QCData(ClusterID2(J2,K2,JJ2),K2)%DiaryID(7:9),'(i10)') IDLT2NUM ! Identify the number in ID of the record in new cluster  
				            IF(IDLTNUM+1 == IDLT2NUM) THEN			 ! if the last three digits in destination cluster is the number subsequent to the number in home cluster
				                ClusterTran(K,J,K2,J2) = ClusterTran(K,J,K2,J2)+1 ! Record the matched records in home to desti clusters
				            END IF									 ! end of the ninth character check
			            END IF										 ! end of the 1:8 characters ID check
                    ELSE IF (index(ALPHATC, QCData(ClusterID2(J,K,JJ),K)%DiaryID(8:8)).OR.QCData(ClusterID2(J,K,JJ),K)%DiaryID(9:9)=="Z") THEN ! If 8th char is a letter
			            IF (QCData(ClusterID2(J,K,JJ),K)%DiaryID(1:7)& 
				        ==QCData(ClusterID2(J2,K2,JJ2),K2)%DiaryID(1:7)) THEN ! Identify chad records with the same 1:8 characters in ID in home cluster to destination cluster
			                IDLT=QCData(ClusterID2(J,K,JJ),K)%DiaryID(9:9) ! Identify the 9th character in ID of the record in home cluster
				            INDX=index(ALPHATC,IDLT)				 ! Identify the sequence# in aphabetic sequence
				            IF (index(ALPHATC, QCData(ClusterID2(J,K,JJ),K)%DiaryID(8:8))) INDX = INDX+26
				            IF(QCData(ClusterID2(J2,K2,JJ2),K2)%DiaryID(8:9)& 
				      	    ==ALPHATC2(INDX+1)) THEN				 ! if the ninth character in destination cluster is the letter in alphabet subsequent to the 9th character in home cluster
				                ClusterTran(K,J,K2,J2) = ClusterTran(K,J,K2,J2)+1 ! Record the matched records in home to desti clusters
				            END IF									 ! end of the ninth character check
			            END IF	 
			         ELSE                                            ! Normal way to do it
			            IF (QCData(ClusterID2(J,K,JJ),K)%DiaryID(1:8)& 
				        ==QCData(ClusterID2(J2,K2,JJ2),K2)%DiaryID(1:8)) THEN ! Identify chad records with the same 1:8 characters in ID in home cluster to destination cluster
			                IDLT=QCData(ClusterID2(J,K,JJ),K)%DiaryID(9:9) ! Identify the 9th character in ID of the record in home cluster
				            INDX=index(Alphatc,IDLT)				 ! Identify the sequence# in aphabetic sequence
				            IF(QCData(ClusterID2(J2,K2,JJ2),K2)%DiaryID(9:9)& 
				      	    ==ALPHATC2(INDX+1)) THEN				 ! if the ninth character in destination cluster is the letter in alphabet subsequent to the 9th character in home cluster
				                ClusterTran(K,J,K2,J2) = ClusterTran(K,J,K2,J2)+1 ! Record the matched records in home to desti clusters				                
				            END IF									 ! end of the ninth character check
			            END IF										 ! end of the 1:8 characters ID check
			         END IF                                          ! end of else if statements
			      END DO											 ! end of chad record inner loop
		       END DO												 ! end of cluster inner loop
	        END DO													 ! end of pool inner loop
         END DO														 ! end of chad record outer loop
	  END DO														 ! end of cluster outer loop
   END DO															 ! end of pool inner loop (end fo step 5)   
! (6) Calculate the transition probabilities from pooli,clusteri to poolj,clusterj
   ClusterTran2=0.0													 ! Calculate the transition probability 2
   DO K=1,NumPools													 ! Loop through pools
	  DO J=1,NumCluster												 ! Loop through clusters
		 DO K2=1,numpools											 ! Loop through inner pools
			Mh=0                                                     ! Setting local value
			zeros=0                                                  ! Setting local value
			DO J2=1,NumCluster										 ! Loop through inner clusters
			   Mh=Mh+ClusterTran(K,J,K2,J2)							 ! Sum the transition counts from home to destination clusters
			   IF(ClusterTran(K,J,K2,J2)==0) zeros(J2)=1			 ! Identify the cluster with zero transition counts
			END DO													 ! End of the inner cluster loop
		    IF(Mh<=3) THEN											 ! If the total transition counts less than 3
			   ClusterTran2(K,J,K2,:)=ClusterPR(K2,:)				 ! Use the frequence distr. as transition probability
	        ELSE													 ! If the transition counts>3
			   ClusterTran2(K,J,K2,:)=real(ClusterTran(K,J,K2,:))/real(Mh) ! Calculate the transition probability
			   IF(sum(zeros)==1) THEN								 ! if there is one zero count in ClusterTran
			      IF(zeros(1)==1) THEN								 ! if zero is in cluster 1
				     ClusterTran2(K,J,K2,1)=0.5/real(Mh)			 ! replace the zero with 0.5/Mh
				     ClusterTran2(K,J,K2,2)=ClusterTran2(K,J,K2,2)-0.5/real(Mh)/2 ! reduce by 0.5/2Mh
   	     		     ClusterTran2(K,J,K2,3)=ClusterTran2(K,J,K2,3)-0.5/real(Mh)/2 ! reduce by 0.5/2Mh
			      ELSE IF(zeros(2)==1) THEN							 ! if zero is in cluster 2	
				     ClusterTran2(K,J,K2,2)=0.5/real(Mh)			 ! replace the zero with 0.5/Mh
				     ClusterTran2(K,J,K2,1)=ClusterTran2(K,J,K2,1)-0.5/real(Mh)/2 ! reduce by 0.5/2Mh
				     ClusterTran2(K,J,K2,3)=ClusterTran2(K,J,K2,3)-0.5/real(Mh)/2 ! reduce by 0.5/2Mh
			      ELSE IF(zeros(3)==1) THEN							 ! if zero is in cluster 3
				     ClusterTran2(K,J,K2,3)=0.5/real(Mh)			 ! replace the zero with 0.5/Mh
				     ClusterTran2(K,J,K2,1)=ClusterTran2(K,J,K2,1)-0.5/real(Mh)/2 ! reduce by 0.5/2Mh
				     ClusterTran2(K,J,K2,2)=ClusterTran2(K,J,K2,2)-0.5/real(Mh)/2 ! reduce by 0.5/2Mh
			      END IF                                             ! End if
			   ELSE IF(sum(zeros)==2) THEN							 ! if there are two zero counts in ClusterTran
			      IF(zeros(1)==1 .and. zeros(2)==1) THEN             ! replace the zeros with non-zero probability
				     ClusterTran2(K,J,K2,1)=0.5/real(Mh)             ! replace the zero with 0.5/Mh
				     ClusterTran2(K,J,K2,2)=0.5/real(Mh)             ! replace the zero with 0.5/Mh
				     ClusterTran2(K,J,K2,3)=ClusterTran2(K,J,K2,3)-2.0*0.5/real(Mh) ! reduce by 0.5/2Mh
			      ELSE IF(zeros(1)==1 .and. zeros(3)==1) THEN	     ! if zero is in cluster 1 and 3
				     ClusterTran2(K,J,K2,1)=0.5/real(Mh)             ! replace the zero with 0.5/Mh
				     ClusterTran2(K,J,K2,3)=0.5/real(Mh)             ! replace the zero with 0.5/Mh
				     ClusterTran2(K,J,K2,2)=ClusterTran2(K,J,K2,2)-2.0*0.5/real(Mh) ! reduce by 0.5/2Mh
			      ELSE IF(zeros(3)==1 .and. zeros(2)==1) THEN	     ! if zero is in cluster 2 and 3
				     ClusterTran2(K,J,K2,3)=0.5/real(Mh)             ! replace the zero with 0.5/Mh
				     ClusterTran2(K,J,K2,2)=0.5/real(Mh)             ! replace the zero with 0.5/Mh
				     ClusterTran2(K,J,K2,1)=ClusterTran2(K,J,K2,1)-2.0*0.5/real(Mh) ! reduce by 0.5/2Mh
			      END IF                                             ! end if
			   END IF												 ! end of zero replacements
		    END IF													 ! end of transition probability for one destination pool
	     END DO														 ! end of inner pool loop
      END DO														 ! end of outer cluster loop
   END DO															 ! end of outer pool loop (end of step 6)
! Step (7) Transfer the clustering results to global array
   ClusterTransM=ClusterTran2										 ! Assign the cluster transition probability to a global variable
   ClusterPRM=ClusterPR												 ! Assign the cluster frequency distribution to a global variable
   DiSelIDM=DiSelID													 ! Assign the pool# x cluster# chad records to a global variable
   DO K=1, NumPools													 ! Loop through pool #
  	  DO J=1, NumCluster                                             ! Go through all clusters
		 QSData(J,K) = QCData(DiSelID(J,K),K)                        ! Transfer data to global array
      END DO                                                         ! End cluster loop
   END DO                                                            ! End of pool loop (end of step 7)
   DEALLOCATE (MicroMinutes)										 ! Deallocate the arrays declared in clustering
   DEALLOCATE (ClusterNo)											 ! Deallocate the arrays declared in clustering
   DEALLOCATE (ClusterNo2)											 ! Deallocate the arrays declared in clustering
   DEALLOCATE (ClusterSN)											 ! Deallocate the arrays declared in clustering
   DEALLOCATE (ClusterID)											 ! Deallocate the arrays declared in clustering
   DEALLOCATE (ClusterID2)											 ! Deallocate the arrays declared in clustering
   DEALLOCATE (DISTANCE)											 ! Deallocate the arrays declared in clustering
   DEALLOCATE (DISTANCE2)											 ! Deallocate the arrays declared in clustering
   DEALLOCATE (CLUSTERPR,CLUSTERTRAN)								 ! Deallocate the arrays declared in clustering
   DEALLOCATE (CLUSTERTRAN2)										 ! Deallocate the arrays declared in clustering
   DEALLOCATE (R2,DiSelID)											 ! Deallocate the arrays declared in clustering
   DEALLOCATE (AccuFrac)											 ! Deallocate the arrays declared in clustering												 
END SUBROUTINE CLUSTERING											 ! End of subroutine Clustering


SUBROUTINE CLUSTERING2 										         ! Call from DiaryProbabilities
   INTEGER(KIND=IK) :: K, K2,J,J2, JJ,JJ2, J4,J5,L, M, N, SN		 ! Local integer variables
   INTEGER(KIND=IK) :: I,II,III,indic,INDX,Mh						 ! Local integer variables
   INTEGER(KIND=IK) :: FirstEvent, LastEvent,JSelect1,Jselect2		 ! Local integer variables
   INTEGER(KIND=IK) :: exitsign, exitsign2,ncluster, ncluster2		 ! Local integer variables
   INTEGER(KIND=IK) :: NumMicros2 =5								 ! # of the micros							
   INTEGER(KIND=IK) :: AR1(2)=0,zeros(3)							 ! Local integer variables
   INTEGER(KIND=IK)	:: NumCluster=3									 ! # of the clusters
   INTEGER(KIND=IK) :: IDLTNUM, IDLT2NUM                             ! Declare local variables
   REAL (KIND=LONG),ALLOCATABLE :: AccuFrac(:,:,:)					 ! Accumulative fraction of records in each cluster
   REAL (KIND=LONG),ALLOCATABLE :: R2(:,:)							 ! Local integer variables
   INTEGER (KIND=IK), ALLOCATABLE :: CLUSTERSN(:,:)                  ! Allocate local array
   INTEGER (KIND=IK), ALLOCATABLE :: MicroMinutes(:,:,:)             ! Allocate local array
   INTEGER (KIND=IK), ALLOCATABLE :: ClusterID(:,:,:),ClusterID2(:,:,:) ! Allocate local array
   INTEGER (KIND=IK), ALLOCATABLE :: ClusterNo(:,:),ClusterNo2(:,:)  ! Allocate local array
   INTEGER (KIND=IK), ALLOCATABLE :: DiSelID (:,:),CLUSTERTRAN(:,:,:,:) ! Allocate local array
   REAL (KIND=RK), ALLOCATABLE :: DISTANCE(:,:,:),DISTANCE2(:,:)     ! Allocate local array
   REAL (KIND=LONG), ALLOCATABLE :: CLUSTERPR(:,:),CLUSTERTRAN2(:,:,:,:) ! Allocate local array
   REAL (KIND=RK) :: minDistance                                     ! Declare local variables
   REAL (KIND=RK) :: Dist1,Dist2,Dist3                               ! Declare local variables
   REAL (KIND=RK) :: totalcls,totalprob,Totalclufrac                 ! Declare local variables
   CHARACTER (LEN=26):: ALPHATC='ABCDEFGHIJKLMNOPQRSTUVWXYZ'         ! Declare local variables
   CHARACTER (LEN=10):: NUMTC='0123456789'                           ! Declare local variables
   CHARACTER (LEN=2) :: ALPHATC2(52)                                 ! Declare local variables
   CHARACTER (LEN=1) :: IDLT                                         ! Declare local variables
	ALPHATC2(1) = 'A'                                                ! Setting alphabet array
	ALPHATC2(2) = 'B'                                                ! Setting alphabet array
	ALPHATC2(3) = 'C'                                                ! Setting alphabet array
	ALPHATC2(4) = 'D'                                                ! Setting alphabet array
	ALPHATC2(5) = 'E'                                                ! Setting alphabet array
	ALPHATC2(6) = 'F'                                                ! Setting alphabet array
	ALPHATC2(7) = 'G'                                                ! Setting alphabet array
	ALPHATC2(8) = 'H'                                                ! Setting alphabet array
	ALPHATC2(9) = 'I'                                                ! Setting alphabet array
	ALPHATC2(10) = 'J'                                               ! Setting alphabet array
	ALPHATC2(11) = 'K'                                               ! Setting alphabet array
	ALPHATC2(12) = 'L'                                               ! Setting alphabet array
	ALPHATC2(13) = 'M'                                               ! Setting alphabet array
	ALPHATC2(14) = 'N'                                               ! Setting alphabet array
	ALPHATC2(15) = 'O'                                               ! Setting alphabet array
	ALPHATC2(16) = 'P'                                               ! Setting alphabet array
	ALPHATC2(17) = 'Q'                                               ! Setting alphabet array
	ALPHATC2(18) = 'R'                                               ! Setting alphabet array
	ALPHATC2(19) = 'S'                                               ! Setting alphabet array
	ALPHATC2(20) = 'T'	                                             ! Setting alphabet array
	ALPHATC2(21) = 'U'                                               ! Setting alphabet array
	ALPHATC2(22) = 'V'                                               ! Setting alphabet array
	ALPHATC2(23) = 'W'                                               ! Setting alphabet array
	ALPHATC2(24) = 'X'                                               ! Setting alphabet array
	ALPHATC2(25) = 'Y'                                               ! Setting alphabet array
	ALPHATC2(26) = 'Z'                                               ! Setting alphabet array
	ALPHATC2(27) = 'AA'                                              ! Setting alphabet array
	ALPHATC2(28) = 'BB'                                              ! Setting alphabet array
	ALPHATC2(29) = 'CC'                                              ! Setting alphabet array
	ALPHATC2(30) = 'DD'                                              ! Setting alphabet array
	ALPHATC2(31) = 'EE'                                              ! Setting alphabet array
	ALPHATC2(32) = 'FF'                                              ! Setting alphabet array
	ALPHATC2(33) = 'GG'                                              ! Setting alphabet array
	ALPHATC2(34) = 'HH'                                              ! Setting alphabet array
	ALPHATC2(35) = 'II'                                              ! Setting alphabet array
	ALPHATC2(36) = 'JJ'                                              ! Setting alphabet array
	ALPHATC2(37) = 'KK'                                              ! Setting alphabet array
	ALPHATC2(38) = 'LL'                                              ! Setting alphabet array												
	ALPHATC2(39) = 'MM'                                              ! Setting alphabet array
	ALPHATC2(40) = 'NN'                                              ! Setting alphabet array
	ALPHATC2(41) = 'OO'                                              ! Setting alphabet array
	ALPHATC2(42) = 'PP'                                              ! Setting alphabet array
	ALPHATC2(43) = 'QQ'                                              ! Setting alphabet array
	ALPHATC2(44) = 'RR'                                              ! Setting alphabet array
	ALPHATC2(45) = 'SS'                                              ! Setting alphabet array
	ALPHATC2(46) = 'TT'                                              ! Setting alphabet array
	ALPHATC2(47) = 'UU'                                              ! Setting alphabet array
	ALPHATC2(48) = 'VV'                                              ! Setting alphabet array
	ALPHATC2(49) = 'WW'                                              ! Setting alphabet array
	ALPHATC2(50) = 'XX'                                              ! Setting alphabet array
	ALPHATC2(51) = 'YY'                                              ! Setting alphabet array
	ALPHATC2(52) = 'ZZ'                                              ! Setting alphabet array														
   NumMicros2=maxval(DData(:)%Location2)							 ! Find the # of micros defined for clustering 
   ALLOCATE (MicroMinutes(MaxVal(NumDiaries(:)),NumPools,NumMicros2)) ! Allocate the space for the arrays
   ALLOCATE (ClusterNo(MaxVal(NumDiaries(:)),NumPools))              ! Allocate local arrays
   ALLOCATE (ClusterNo2(NumCluster,NumPools))                        ! Allocate local arrays
   ALLOCATE (ClusterSN(MaxVal(NumDiaries(:)),NumPools))              ! Allocate local arrays
   ALLOCATE (ClusterID(MaxVal(NumDiaries(:)),NumPools,MaxVal(NumDiaries(:)))) ! Allocate local arrays
   ALLOCATE (ClusterID2(NumCluster,NumPools,MaxVal(NumDiaries(:))))  ! Allocate local arrays
   ALLOCATE (DISTANCE(MaxVal(NumDiaries(:)),NumPools,MaxVal(NumDiaries(:)))) ! Allocate local arrays
   ALLOCATE (DISTANCE2(MaxVal(NumDiaries(:)),MaxVal(NumDiaries(:)))) ! Allocate local arrays
   ALLOCATE (CLUSTERPR(NumPools,NumCluster),&                      
 		  CLUSTERTRAN(NumPools,NumCluster,NumPools,NumCluster))      ! Allocate local arrays
   ALLOCATE (CLUSTERTRAN2(NumPools,NumCluster,NumPools,NumCluster))  ! Allocate local arrays
   ALLOCATE (R2(NumCluster,NumPools),DiSelID(NumCluster,NumPools))   ! Allocate local arrays
   ALLOCATE (AccuFrac(NumCluster,NumPools,MaxVal(NumDiaries(:))))	 ! Allocate local arrays
   MicroMinutes=0													 ! Initialize the array for minutes in micros defined for clustering
   AccuFrac=0.0	    												 ! Initialize the accumulative fraction array for clustering
! (1) Calculate the minutes in each micros for each matched chad record
   DO K=1, NumPools													 ! Loop through pool#
  	  DO J=1, NumDiaries(K)											 ! Loop through NonZero chad records in each pool
		 FirstEvent = QCData(J,K)%FirstEvent						 ! Find the first event in the chad record
		 LastEvent = QCData(J,K)%LastEvent							 ! Find the last event in the chad record 	 
		 DO I=FirstEvent,LastEvent									 ! Loop through all the events in the chad record
		    IF(DData(I)%Location2==-1)                              &                         
   		         DData(I)%Location2=DData(I-1)%Location2			 ! Use the previous chad record if the location locode is unknown
   		    IF(DData(I)%Location2 > 0) THEN                          ! Only if loc is in ClustMap
			    MicroMinutes(J,K,DData(I)%Location2) =              & 
			    MicroMinutes(J,K,DData(I)%Location2)+ DData(I)%Duration ! Sum minutes in each cluster micro 
			ENDIF                                                    ! End conditional
		 END DO														 ! End of Event loop
!		 IF(sum(MicroMinutes(J,K,:))/=1440) THEN					 ! Check if total minutes must be equal to 1440
!			WRITE(*,*) 'ErrorTotal Minutes /= 1440 minutes'          ! Error message
!			STOP													 ! Program will stop
!		 END IF														 ! End of total minutes check
	  END DO   														 ! End of NonZero chad record loop
   END DO															 ! End of pool loop (end of step 1)
! (2) Calculate the distances among chad records using the right upper (above diagonal) part of array only
   DISTANCE = -1													 ! Initialize the distance to -1
   ClusterNo = 1													 ! Initiallize the # of cluster to 1
   ClusterID = -99													 ! Initialize the cluster id to -99
   ClusterSN = -99                                                   ! Initialize the cluster SN to -99
   DO K=1,NumPools													 ! Begining of pool loop
  	  DO J=1, NumDiaries(K)											 ! Loop through chad records in each pool
		 ClusterID(J,K,1)=J											 ! Assign sequence # of chad records to the first element of the array
		 DO JJ=J+1,NumDiaries(K)									 ! Only calculate the distances on the upper part of the array (above the diagonal)
			DISTANCE(J,K,JJ)=DISTANCE(J,K,JJ)+1					     ! Initialize it to zero(-1+1=0)
			DO M=1, NumMicros2									     ! Loop through micros
			   DISTANCE(J,K,JJ)=DISTANCE(J,K,JJ)+                   &                
				 (MicroMinutes(J,K,M)-MicroMinutes(JJ,K,M))**2./2.   ! Calculate the distance
			END DO												     ! End of micros loop
		 END DO														 ! End of destination chad
	  END DO														 ! End of home chad loop
   END DO															 ! End of pool loop	(end of step 2)
! (3) Grouping the chad records in each pool into three clusters
   DO K=1,NumPools													 ! Loop through pools
 	  ncluster=NumDiaries(K)										 ! Beginning # of clusters = # of chad in a pool
	  DO WHILE (ncluster>3)											 ! Loop till # of clusters=3
		 Distance2=Distance(:,K,:)									 ! Assign the 3-dimen distance to 2-dimen array
		 AR1=MINLOC(Distance2,Mask = Distance2 >-1.0)				 ! Find the location of minimum (>-1) value in the array
		 JSelect1 = AR1(1)											 ! The array location  subscript 1 = # of home chad record
		 JSelect2 = AR1(2)											 ! The array location  subscript 2 = # of destination chad record
		 DO J4=1, ClusterNo(JSelect2,K)								 ! Transfer all CHAD IDs in ClusterJSelect2 to Cluster JSelect1)
		 	J5=ClusterNo(JSelect1,K)+J4								 ! Index for Cluster JSelect1
			ClusterID(JSelect1,K,J5)=ClusterID(JSelect2,K,J4)        ! Transfer ID from Cluster JSelect2 to JSelect1
			ClusterID(JSelect2,K,J4) = -99							 ! Assign -99 to ID in eliminated Cluster JSelect2			
		 END DO														 ! End of transfer loop
		 ncluster=ncluster-1										 ! Eliminate one cluster for the loop
		 DO JJ=1, NumDiaries(K)										 ! Loop through the chad records in the pool
			IF(clusterNo(JJ,K)==0) CYCLE							 ! Skip if cluster JJ is empty
			IF(JJ==JSelect1) CYCLE									 ! Skip as the JSelect1 to JSelect1 distance is not needed
			IF(JJ==JSelect2) CYCLE									 ! Skip as Cluster JSelect2 is just eliminated
			IF(JJ>JSelect1) THEN								     ! Horizonal element in distance array for Jselect1
			   Dist1 = (ClusterNo(JJ,K)+ClusterNo(JSelect1,K))      &   
			 	    *Distance(JSelect1,K,JJ)						 ! Part 1 of new distance
			   IF(JJ>JSelect2) THEN							         ! Horizonal element in the distance array for Jselect2
				  Dist2 = (ClusterNo(JJ,K)+                         &                       
				        ClusterNo(JSelect2,K))*Distance(JSelect2,K,JJ) ! Part 2 of new distance
			   ELSE											         ! Vertical element in the distance array for Jselect2
				  Dist2 = (ClusterNo(JJ,K)+                         &                      
				        ClusterNo(JSelect2,K))*Distance(JJ,K,JSelect2) ! Part 2 of new distance
			   END IF                                                ! End if statements
			   Dist3 =  ClusterNo(JJ,K)*Distance(JSelect1,K,JSelect2)! Part 3 of new distance
 			   Distance(JSelect1,K,JJ) = (Dist1+Dist2-Dist3)        &      
			 	  /(ClusterNo(JJ,K)+ClusterNo(JSelect1,K)+          &         
				  ClusterNo(JSelect2,K))							 ! New distance
			ELSE IF(JJ<JSelect1) THEN							     ! Vertical element in the distance array for Jselect1
			   Dist1 = (ClusterNo(JJ,K)+ClusterNo(JSelect1,K))      &   
			        *Distance(JJ,K,JSelect1)                         ! Part 1 of new distance
			   IF(JJ>JSelect2) THEN							         ! Horizonal element in the distance array for Jselect2
				  Dist2 = (ClusterNo(JJ,K)+                         &                     
						ClusterNo(JSelect2,K))*                     &                    
						Distance(JSelect2,K,JJ)						 ! Part 2 of new distance
			   ELSE										             ! Vertical element in the distance array for JSelect2
				  Dist2 = (ClusterNo(JJ,K)+                         &                      
						ClusterNo(JSelect2,K))*                     &                  
						Distance(JJ,K,JSelect2)						 ! Part2 of 2 of new distance
			   END IF                                                ! End if statements
			   Dist3 =  ClusterNo(JJ,K)*Distance(JSelect1,K,JSelect2)! Part3 of new distance
			   Distance(JJ,K,JSelect1) = (Dist1+Dist2-Dist3)/       &      
			       (ClusterNo(JJ,K)+	ClusterNo(JSelect1,K)       &     
				   +ClusterNo(JSelect2,K))			                 ! New Distance
			END IF                                                   ! End of the array index adjustment for Jselect1
		 END DO														 ! End of new distance calculation for new cluster Jselect1
		 ClusterNo(JSelect1,K) = ClusterNo(JSelect1,K)+             &         
							    ClusterNo(JSelect2,K)				 ! Combine the # of chad records in cluster Jselect1 and Jselect2 into Jselect1
		 ClusterNo(JSelect2,K) = 0									 ! Make the # of chad records zero in cluster Jselect2
		 Distance(JSelect2,K,:)=-1.0								 ! Make the distance to -1 of JSelect2 to other clusters 
		 Distance(:,K,JSelect2)=-1.0								 ! Make the distance to -1 of JSelect2 to other clusters 
	  END DO														 ! End of clustering for chad records in pool K
	  IF (ncluster/=3) THEN											 ! Make sure that the chad records are grouped into 3 clusters
	     WRITE(*,*) 'Error in clustering'                            ! Print error
		 WRITE(*,*) 'Please increase the AgeCutPct in Parameter File'! The problem may be due to two small age window specified in the parameter file
		 STOP                                                        ! stop
	  END IF                                                         ! End error code
	  12222 FORMAT(3I5,500I6)                                        ! Formatting
	  ncluster2=0													 ! Initialize the cluster #
	  totalcls=0													 ! Initialize the variable
	  DO J = 1, NumDiaries(K)										 ! Loop through chad records or initial clusters in pool K
		 IF(ClusterNo(J,K)/=0) THEN									 ! Identify one of 3 clusters that have been grouped
			ncluster2=   ncluster2+1								 ! increment of clusters 1-3	
			ClusterPR(K,ncluster2)=real(ClusterNo(J,K))				 ! Transfer the # of chad records in each cluster into a new array
			ClusterNo2(ncluster2,K)=ClusterNo(J,K)					 ! Transfer the # of chad records in each cluster into a new array
			ClusterID2(ncluster2,K,:)=ClusterID(J,K,:)				 ! Transfer the chad id (sequence # in QCData) to an new array
			totalcls=totalcls+real(ClusterNo(J,K))                   ! Total # of chad records in the pool
		 END IF														 ! End of cluster identification
	  END DO														 ! End ofloop for searching clusters
	  ClusterPR(K,:)=ClusterPR(K,:)/totalcls						 ! Calculate the distribution frequency of chad records in three clusters
   END DO															 ! End of clustering (end of step 3)
! (5) Calculate the transition counts from (poolA,clusterA,poolB,clusterB)
   ClusterTran=0													 ! Initialize the transition probability array
   DO K=1, NumPools													 ! Loop through pools
  	  DO J=1, NumCluster											 ! Loop through clusters
		 DO JJ=1, ClusterNo2(J,K)									 ! Loop through chad records in each cluster			
			DO K2=1, NumPools										 ! Loop through pools again for cross transition probability
			   DO J2=1, NumCluster									 ! Loop through clusters again
			      DO JJ2=1,ClusterNo2(J2,K2)						 ! Loop through chad records in each cluster again
                     IF (LEN_TRIM(QCData(ClusterID2(J,K,JJ),K)%DiaryID)==8) THEN ! If length of CHADID is 8 characters                   
			            IF(QCData(ClusterID2(J,K,JJ),K)%DiaryID(1:7)& 
				        ==QCData(ClusterID2(J2,K2,JJ2),K2)%DiaryID(1:7)) THEN ! Identify chad records with the same 1:8 characters in ID in home cluster to destination cluster
			                IDLT=QCData(ClusterID2(J,K,JJ),K)%DiaryID(8:8) ! Identify the 9th character in ID of the record in home cluster
				            INDX=index(Alphatc,IDLT)			     ! Identify the sequence# in aphabetic sequence
				            IF(QCData(ClusterID2(J2,K2,JJ2),K2)%DiaryID(8:8)& 
				      	    ==ALPHATC2(INDX+1)) THEN				 ! if the ninth character in destination cluster is the letter in alphabet subsequent to the 9th character in home cluster
				                ClusterTran(K,J,K2,J2) = ClusterTran(K,J,K2,J2)+1 ! Record the matched records in home to desti clusters
				            END IF									 ! end of the ninth character check
			            END IF										 ! end of the 1:8 characters ID check
                     ELSE IF (index(NUMTC,QCData(ClusterID2(J,K,JJ),K)%DiaryID(9:9))) THEN ! If last value in CHADID is a number
			            IF(QCData(ClusterID2(J,K,JJ),K)%DiaryID(1:6) & 
				        ==QCData(ClusterID2(J2,K2,JJ2),K2)%DiaryID(1:6)) THEN ! Identify chad records with the same 1:8 characters in ID in home cluster to destination cluster
			                READ(QCData(ClusterID2(J,K,JJ),K)%DiaryID(7:9),'(i10)') IDLTNUM ! Identify the number in ID of the record in home cluster               
			                READ(QCData(ClusterID2(J2,K2,JJ2),K2)%DiaryID(7:9),'(i10)') IDLT2NUM ! Identify the number in ID of the record in new cluster  
				            IF(IDLTNUM+1 == IDLT2NUM) THEN			 ! if the last three digits in destination cluster is the number subsequent to the number in home cluster
				                ClusterTran(K,J,K2,J2) = ClusterTran(K,J,K2,J2)+1 ! Record the matched records in home to desti clusters
				            END IF									 ! end of the ninth character check
			            END IF										 ! end of the 1:8 characters ID check
                    ELSE IF (index(ALPHATC, QCData(ClusterID2(J,K,JJ),K)%DiaryID(8:8)).OR.QCData(ClusterID2(J,K,JJ),K)%DiaryID(9:9)=="Z") THEN ! If 8th char is a letter
			            IF (QCData(ClusterID2(J,K,JJ),K)%DiaryID(1:7)& 
				        ==QCData(ClusterID2(J2,K2,JJ2),K2)%DiaryID(1:7)) THEN ! Identify chad records with the same 1:8 characters in ID in home cluster to destination cluster
			                IDLT=QCData(ClusterID2(J,K,JJ),K)%DiaryID(9:9) ! Identify the 9th character in ID of the record in home cluster
				            INDX=index(ALPHATC,IDLT)				 ! Identify the sequence# in aphabetic sequence
				            IF (index(ALPHATC, QCData(ClusterID2(J,K,JJ),K)%DiaryID(8:8))) INDX = INDX+26
				            IF(QCData(ClusterID2(J2,K2,JJ2),K2)%DiaryID(8:9)& 
				      	    ==ALPHATC2(INDX+1)) THEN				 ! if the ninth character in destination cluster is the letter in alphabet subsequent to the 9th character in home cluster
				                ClusterTran(K,J,K2,J2) = ClusterTran(K,J,K2,J2)+1 ! Record the matched records in home to desti clusters
				            END IF									 ! end of the ninth character check
			            END IF	 
			         ELSE                                            ! Normal way to do it
			            IF (QCData(ClusterID2(J,K,JJ),K)%DiaryID(1:8)& 
				        ==QCData(ClusterID2(J2,K2,JJ2),K2)%DiaryID(1:8)) THEN ! Identify chad records with the same 1:8 characters in ID in home cluster to destination cluster
			                IDLT=QCData(ClusterID2(J,K,JJ),K)%DiaryID(9:9) ! Identify the 9th character in ID of the record in home cluster
				            INDX=index(Alphatc,IDLT)				 ! Identify the sequence# in aphabetic sequence
				            IF(QCData(ClusterID2(J2,K2,JJ2),K2)%DiaryID(9:9)& 
				      	    ==ALPHATC2(INDX+1)) THEN				 ! if the ninth character in destination cluster is the letter in alphabet subsequent to the 9th character in home cluster
				                ClusterTran(K,J,K2,J2) = ClusterTran(K,J,K2,J2)+1 ! Record the matched records in home to desti clusters				                
				            END IF									 ! end of the ninth character check
			            END IF										 ! end of the 1:8 characters ID check
			         END IF                                          ! end of else if statements
			      END DO											 ! end of chad record inner loop
		       END DO												 ! end of cluster inner loop
	        END DO													 ! end of pool inner loop
         END DO														 ! end of chad record outer loop
	  END DO														 ! end of cluster outer loop
   END DO															 ! end of pool inner loop (end fo step 5)
! (6) Calculate the transition probabilities from pooli,clusteri to poolj,clusterj
   ClusterTran2=0.0													 ! Calculate the transition probability 2
   DO K=1,NumPools													 ! Loop through pools
	  DO J=1,NumCluster												 ! Loop through clusters
		 DO K2=1,numpools											 ! Loop through inner pools
		    Mh=0                                                     ! Initializing local variables
			zeros=0                                                  ! Initializing local variables
			DO J2=1,NumCluster										 ! Loop through inner clusters
			   Mh=Mh+ClusterTran(K,J,K2,J2)							 ! Sum the transition counts from home to destination clusters
			   IF(ClusterTran(K,J,K2,J2)==0) zeros(J2)=1			 ! Identify the cluster with zero transition counts
			END DO													 ! End of the inner cluster loop
		    IF(Mh<=3) THEN											 ! If the total transition counts less than 3
		   	   ClusterTran2(K,J,K2,:)=ClusterPR(K2,:)				 ! Use the frequence distr. as transition probability
	        ELSE													 ! If the transition counts>3
			   ClusterTran2(K,J,K2,:)=real(ClusterTran(K,J,K2,:))/real(Mh) ! Calculate the transition probability
			   IF(sum(zeros)==1) THEN								 ! if there is one zero count in ClusterTran
			      IF(zeros(1)==1) THEN								 ! if zero is in cluster 1
				     ClusterTran2(K,J,K2,1)=0.5/real(Mh)			 ! replace the zero with 0.5/Mh
				     ClusterTran2(K,J,K2,2)=ClusterTran2(K,J,K2,2)-0.5/real(Mh)/2 ! reduce by 0.5/2Mh
				     ClusterTran2(K,J,K2,3)=ClusterTran2(K,J,K2,3)-0.5/real(Mh)/2 ! reduce by 0.5/2Mh
			      ELSE IF(zeros(2)==1) THEN							 ! if zero is in cluster 2	
				     ClusterTran2(K,J,K2,2)=0.5/real(Mh)			 ! replace the zero with 0.5/Mh
				     ClusterTran2(K,J,K2,1)=ClusterTran2(K,J,K2,1)-0.5/real(Mh)/2 ! reduce by 0.5/2Mh
				     ClusterTran2(K,J,K2,3)=ClusterTran2(K,J,K2,3)-0.5/real(Mh)/2 ! reduce by 0.5/2Mh
			      ELSE IF(zeros(3)==1) THEN							 ! if zero is in cluster 3
				     ClusterTran2(K,J,K2,3)=0.5/real(Mh)			 ! replace the zero with 0.5/Mh
				     ClusterTran2(K,J,K2,1)=ClusterTran2(K,J,K2,1)-0.5/real(Mh)/2 ! reduce by 0.5/2Mh
				     ClusterTran2(K,J,K2,2)=ClusterTran2(K,J,K2,2)-0.5/real(Mh)/2 ! reduce by 0.5/2Mh
			      END IF                                             ! end if
			   ELSE IF(sum(zeros)==2) THEN							 ! if there are two zero counts in ClusterTran
			      IF(zeros(1)==1 .and. zeros(2)==1) THEN             ! replace the zeros with non-zero probability
				     ClusterTran2(K,J,K2,1)=0.5/real(Mh)             ! replace the zero with 0.5/Mh
				     ClusterTran2(K,J,K2,2)=0.5/real(Mh)             ! replace the zero with 0.5/Mh
				     ClusterTran2(K,J,K2,3)=ClusterTran2(K,J,K2,3)-2.0*0.5/real(Mh) ! reduce by 0.5/2Mh
			      ELSE IF(zeros(1)==1 .and. zeros(3)==1) THEN	     ! if zero in 1 or 3
				     ClusterTran2(K,J,K2,1)=0.5/real(Mh)             ! replace the zero with 0.5/Mh
				     ClusterTran2(K,J,K2,3)=0.5/real(Mh)             ! replace the zero with 0.5/Mh
				     ClusterTran2(K,J,K2,2)=ClusterTran2(K,J,K2,2)-2.0*0.5/real(Mh) ! reduce by 0.5/2Mh
			      ELSE IF(zeros(3)==1 .and. zeros(2)==1) THEN	     ! if zero in 2 or 3
				     ClusterTran2(K,J,K2,3)=0.5/real(Mh)             ! replace the zero with 0.5/Mh
				     ClusterTran2(K,J,K2,2)=0.5/real(Mh)             ! replace the zero with 0.5/Mh
				     ClusterTran2(K,J,K2,1)=ClusterTran2(K,J,K2,1)-2.0*0.5/real(Mh) ! reduce by 0.5/2Mh
			      END IF                                             ! end if statements
			   END IF												 ! end of zero replacements
		    END IF                                                   ! end of transition probability for one destination pool
	     END DO                                                      ! end of inner pool loop
	  END DO                                                         ! end of outer cluster loop
   END DO															 ! end of outer pool loop (end of step 6)
! Step (7) Transfer the clustering results to global array
   ClusterTransM=ClusterTran2										 ! Assign the cluster transition probability to a global variable
   ClusterPRM=ClusterPR												 ! Assign the cluster frequency distribution to a global variable
   ClusterNoOut=ClusterNo2                                           ! Initialize local variable
   ClusterIDOut=0                                                    ! Initialize local variable
   DO K=1, NumPools													 ! Loop through pools
  	  DO J=1, NumCluster											 ! Loop through clusters
		 DO JJ=1, ClusterNo2(J,K)                                    ! Loop through Cluster2 Numbers
		 ClusterIDOut(J,K,JJ)=ClusterID2(J,K,JJ)                     ! Transfer results to global array
		 END DO                                                      ! End Cluster2 loop
	  END DO                                                         ! End Cluster loop
   END DO                                                            ! End Pool loop (end of step 7)
   DEALLOCATE (MicroMinutes)										 ! Deallocate the arrays declared in clustering
   DEALLOCATE (ClusterNo)											 ! Deallocate the arrays declared in clustering
   DEALLOCATE (ClusterNo2)											 ! Deallocate the arrays declared in clustering
   DEALLOCATE (ClusterSN)											 ! Deallocate the arrays declared in clustering
   DEALLOCATE (ClusterID)											 ! Deallocate the arrays declared in clustering
   DEALLOCATE (ClusterID2)											 ! Deallocate the arrays declared in clustering
   DEALLOCATE (DISTANCE)											 ! Deallocate the arrays declared in clustering
   DEALLOCATE (DISTANCE2)											 ! Deallocate the arrays declared in clustering
   DEALLOCATE (CLUSTERPR,CLUSTERTRAN)								 ! Deallocate the arrays declared in clustering
   DEALLOCATE (CLUSTERTRAN2)										 ! Deallocate the arrays declared in clustering
   DEALLOCATE (R2,DiSelID)											 ! Deallocate the arrays declared in clustering	
   DEALLOCATE (AccuFrac)											 ! Deallocate the arrays declared in clustering												 
   END SUBROUTINE CLUSTERING2										 ! End of subroutine Clustering

END MODULE DiaryModule                                               ! Last source code line in module