MODULE GlobalModule                                         ! Start of GlobalModule source code
   ! Latest revisions by WGG on August 16, 2012
   ! This module contains three sections:
   ! The first has certain useful constants that are used throughout APEX
   ! The second contains the TYPE declarations used here or used in multiple modules
   ! The third contains all the global variables used in APEX
   
   IMPLICIT NONE                                            ! All variables must be declared

   ! APEX Version Info
   CHARACTER(LEN=27) :: Version = "4.5 (dated August 16, 2012)" ! Global version definition

   ! Default name for PARAMS input file (blank name means get name from command line)
   CHARACTER(LEN=1600) :: ParFile = ""                      ! Full path name in double quotes  
   
   ! Useful Constants 
   INTEGER,          PARAMETER :: BYT=1, SHORT=4, LONG=8    ! Lengths for KIND values
   INTEGER,          PARAMETER :: IK=SHORT, RK=LONG         ! Selected KIND values for Int and Real
   INTEGER,          PARAMETER :: IK2=LONG, RK2=SHORT       ! Selected KIND values for Int and Real
   INTEGER(KIND=IK), PARAMETER :: YES=1, NO=2               ! Constants for binary switches in code
   INTEGER(KIND=IK), PARAMETER :: OPN=1, CLSE=2             ! Constants for managing inpt/outpt files
   INTEGER(KIND=IK), PARAMETER :: LEN1       = 12           ! Maximum length for short strings (words)
   INTEGER(KIND=IK), PARAMETER :: LEN2       = 40           ! Maximum length for medium text strings
   INTEGER(KIND=IK), PARAMETER :: LEN3       = 250          ! Maximum length for long text strings 
   INTEGER(KIND=IK), PARAMETER :: LEN4       = 1600         ! Maximum length for input file record
   INTEGER(KIND=IK), PARAMETER :: LEN5       = 12000        ! Maximum length for long input record
   INTEGER(KIND=IK), PARAMETER :: LENTAB     = 6018         ! Maximum length of output lines on Log (500)
   INTEGER(KIND=IK), PARAMETER :: MAXCOUNTY  = 4000         ! Maximum # of counties listed on PARAMS
   INTEGER(KIND=IK), PARAMETER :: MAXTRACT   = 66304        ! Maximum # of tracts listed on PARAMS
   INTEGER(KIND=IK), PARAMETER :: MAXFEV     = 30           ! Maximum # of FEV risk distributions
   INTEGER(KIND=IK), PARAMETER :: MAXDISTRIB = 200          ! Maximum # of user-defined distributions
   INTEGER(KIND=IK), PARAMETER :: MAXPOLLUT  = 20           ! Maximum # of pollutants
   INTEGER(KIND=IK), PARAMETER :: MAXMIC     = 50           ! Maximum # of micros to print to MRes fl
   INTEGER(KIND=IK), PARAMETER :: MAXDIM     = 6            ! Maximum # of input variables per distrib.
   INTEGER(KIND=IK), PARAMETER :: MAXCV      = 50           ! Maximum # of items on CV List
   INTEGER(KIND=IK), PARAMETER :: NUMPSUMVARS= 63           ! # psum var types in psum file
   INTEGER(KIND=IK), PARAMETER :: NUMDAYVARS = 20           ! # var types in daily file
   INTEGER(KIND=IK), PARAMETER :: NUMTSVARS  = 15           ! # var types in ts file
   INTEGER(KIND=IK), PARAMETER :: NUMMRESVARS= 38           ! # variables in micro results file
   INTEGER(KIND=IK), PARAMETER :: NUMEVENTVARS=23           ! # variables in event file
   INTEGER(KIND=IK), PARAMETER :: NUMTABLEVARS=19           ! # variables in tables file
   INTEGER(KIND=IK)            :: EVENTSDAY  = 210          ! Limit on mean events/day for composites
   INTEGER(KIND=IK), PARAMETER :: HOURSDAY   = 24           ! Number of hours per day 
   INTEGER(KIND=IK), PARAMETER :: MINSHOUR   = 60           ! Number of minutes per hour
   CHARACTER(LEN=8), PARAMETER :: FMT1="(A12)   "           ! Format for reading LEN1 strings
   CHARACTER(LEN=8), PARAMETER :: FMT2="(A40)   "           ! Format for reading LEN2 strings
   CHARACTER(LEN=8), PARAMETER :: FMT3="(A250)  "           ! Format for reading LEN3 strings
   CHARACTER(LEN=8), PARAMETER :: FMT4="(A1600) "           ! Format for reading LEN4 strings
   CHARACTER(LEN=8), PARAMETER :: FMT5="(A12000)"           ! Format for reading LEN5 strings
   CHARACTER(LEN=3), PARAMETER :: YESNO(0:2)               &  
                                  =(/"X  ","Yes","No "/)    ! For printing binary switch status 
   CHARACTER(LEN=1), PARAMETER :: YN(0:2)                  &  
                                  =(/"X","Y","N"/)          ! For printing binary switch status 
   INTEGER(KIND=IK), PARAMETER :: ParU   = 10               ! Unit # for job parameters input file
   INTEGER(KIND=IK), PARAMETER :: SectU  = 11               ! Unit # for list of pop. sector locations
   INTEGER(KIND=IK), PARAMETER :: MonU   = 12               ! Unit # for list of air monitor locations
   INTEGER(KIND=IK), PARAMETER :: ZoneU  = 13               ! Unit # for list of temperature locations
   INTEGER(KIND=IK), PARAMETER :: PopU   = 14               ! Unit # for population input files
   INTEGER(KIND=IK), PARAMETER :: CommU  = 15               ! Unit # for commuter input file
   INTEGER(KIND=IK), PARAMETER :: TempU  = 16               ! Unit # for temperature input file
   INTEGER(KIND=IK), PARAMETER :: AQU    = 17               ! Unit # for air quality input file 
   INTEGER(KIND=IK), PARAMETER :: MetsU  = 18               ! Unit # for METS input file
   INTEGER(KIND=IK), PARAMETER :: PhysU  = 19               ! Unit # for physiology input file
   INTEGER(KIND=IK), PARAMETER :: DistU  = 20               ! Unit # for user distributions input file
   INTEGER(KIND=IK), PARAMETER :: ChMapU = 21               ! Unit # for CHAD code mapping input file
   INTEGER(KIND=IK), PARAMETER :: ChQuU  = 22               ! Unit # for CHAD questionnaire input file
   INTEGER(KIND=IK), PARAMETER :: ChEvU  = 23               ! Unit # for CHAD diary events input file
   INTEGER(KIND=IK), PARAMETER :: MicroU = 24               ! Unit # for microenvironments input file
   INTEGER(KIND=IK), PARAMETER :: EmpU   = 25               ! Unit # for employment probability file 
   INTEGER(KIND=IK), PARAMETER :: VentU  = 26               ! Unit # for ventilation parameter file 
   INTEGER(KIND=IK), PARAMETER :: StatU  = 27               ! Unit # for longit. diary stat input file
   INTEGER(KIND=IK), PARAMETER :: PrevU  = 28               ! Unit # for disease prevalence file 
   INTEGER(KIND=IK), PARAMETER :: MMapU  = 29               ! Unit # for METS mapping file
   INTEGER(KIND=IK), PARAMETER :: MAXINU = 29               ! Maximum unit number used for input files
   INTEGER(KIND=IK), PARAMETER :: LogU   = 30               ! Unit # for execution log output file
   INTEGER(KIND=IK), PARAMETER :: PSumU  = 31               ! Unit # for profile summary output file
   INTEGER(KIND=IK), PARAMETER :: MSumU  = 32               ! Unit # for microenv. summary output file
   INTEGER(KIND=IK), PARAMETER :: TabU   = 33               ! Unit # for tables output file
   INTEGER(KIND=IK), PARAMETER :: SiteU  = 34               ! Unit # for site list output file
   INTEGER(KIND=IK), PARAMETER :: EventU = 35               ! Unit # for events output file
   INTEGER(KIND=IK), PARAMETER :: HourU  = 36               ! Unit # for hourly ventilation output file
   INTEGER(KIND=IK), PARAMETER :: MResU  = 37               ! Unit # for micro results output file
   INTEGER(KIND=IK), PARAMETER :: DayU   = 38               ! Unit # for daily output file
   INTEGER(KIND=IK), PARAMETER :: TSU    = 39               ! Unit # for the TS file
   INTEGER(KIND=IK), PARAMETER :: ClMapU = 40				! Unit # for cluster micro mapping file   
   INTEGER(KIND=IK), PARAMETER :: CommTU = 41               ! Unit # for commuting time input file   
   INTEGER(KIND=IK), PARAMETER :: ProU   = 42               ! Unit # for profile factors input file
   INTEGER(KIND=IK), PARAMETER :: DEmpU  = 43               ! Unit # for optional CHAD employment file
   INTEGER(KIND=IK), PARAMETER :: MAXU   = 43               ! Maximum unit number used
   INTEGER(KIND=IK), PARAMETER :: MAXK   = 8                ! Max. # words available for storing seeds
   INTEGER(KIND=IK), PARAMETER :: MAXINPUTS = 109           ! # non-level, non-flname input variables
   INTEGER(KIND=IK), PARAMETER :: MAXTABLES=12              ! Maximum number of table types
   INTEGER (KIND=IK), PARAMETER :: DM1HE=1, DM8HE=2,       &
      DAVGE=3, SAVGE=4, TIMEE=5, DM1HD=6, DM8HD=7, DAVGD=8,&
      SAVGD=9, TIMED=10, DMEHD=11, H_EHD=12                 ! Constants for table numbering
   INTEGER (KIND=IK), PARAMETER :: UNDEFINED=0, TABLE=1,   &
      FIXED=2, POINT=2, UNIFORM=3, NORMAL=4, LOGNORMAL=5,  &
      TRIANGLE=6, EXPONENTIAL=7, OFFON=8, BETA=9,          &   
      GAMMA=10,PARETO=11,WEIBULL=12,LUNIFORM=13,           &
      LGT=14,CAUCHY=15,EVALUE=16, DISCRETE=17, BURR=18      ! Allowed APEX distribution types
   CHARACTER(LEN=5), PARAMETER :: DistText(1:18)           &  
       =(/"TABLE","POINT","UNIFO","NORMA","LOGNO","TRIAN", &
       "EXPON","OFFON","BETA ","GAMMA","PARET","WEIBU",    &
       "LUNIF","LGT  ","CAUCH","EVALU","DISCR","BURR "/)    ! For printing dist types 
   INTEGER(KIND=IK), PARAMETER :: MALE=1, FEMALE=2          ! Internal model values for each gender
   INTEGER(KIND=IK), PARAMETER :: PPM=1, UGM3=2, PPB=3      ! Internal model values for units
   CHARACTER(LEN=13),PARAMETER :: ST_MSG='=== Starting '    ! Log file indicator for start of proc
   CHARACTER(LEN=13),PARAMETER :: LV_MSG='=== Leaving  '    ! Log file indicator for end of proc
   CHARACTER(LEN=10),PARAMETER :: ST_FMT='(/A13,A40)'       ! Format for proc start message
   CHARACTER(LEN=10),PARAMETER :: LV_FMT='(A13,A40/)'       ! Format for proc end message
   CHARACTER(LEN=17),PARAMETER :: ST_MS2='    '// ST_MSG    ! Start indicator for 2nd level proc
   CHARACTER(LEN=17),PARAMETER :: LV_MS2='    '// LV_MSG    ! Leave indicator for 2nd level proc
   CHARACTER(LEN=9), PARAMETER :: SL_FM2='(A17,A40)'        ! Format for 2nd level proc message
   CHARACTER(LEN=21),PARAMETER :: ST_MS3='    '// ST_MS2    ! Start indicator for 2nd level proc
   CHARACTER(LEN=21),PARAMETER :: LV_MS3='    '// LV_MS2    ! Leave indicator for 2nd level proc
   CHARACTER(LEN=9), PARAMETER :: SL_FM3='(A21,A40)'        ! Format for 2nd level proc message
   CHARACTER(LEN=5), PARAMETER :: UNITTEXT(1:3) =          &
      (/" ppm ","ug/m3"," ppb "/)                          ! Text for printing out units
   CHARACTER(LEN=6), PARAMETER :: SEXTEXT(0:2) =           & 
      (/"X     ","Male  ","Female"/)                        ! Text for printing out gender data
   CHARACTER(LEN=1), PARAMETER :: ST(0:2) =                & 
      (/"X","M","F"/)                                       ! Text for printing out gender data
   CHARACTER(LEN=5), PARAMETER :: EMPTEXT(0:2) =           & 
      (/"X    ","Works","NoWrk"/)                           ! Text for printing out employment data
   CHARACTER(LEN=5), PARAMETER :: DISTEXT(0:2) =           & 
      (/"X    ","Ill  ","NoIll"/)                           ! Text for printing out disease data
   CHARACTER(LEN=5), PARAMETER :: GASTEXT(0:2) =           & 
      (/"X    ","Gas  ","NoGas"/)                           ! Text for printing out gas stove data
   CHARACTER(LEN=5), PARAMETER :: PILOTTEXT(0:2) =         & 
      (/"X    ","Pilot","NoPlt"/)                           ! Text for printing out pilot light data
   CHARACTER(LEN=3), PARAMETER :: DAYOFWEEK(1:7) =         &
      (/"SUN","MON","TUE","WED","THU","FRI","SAT"/)         ! Days of week for printing output
   CHARACTER(LEN=6), PARAMETER :: ACTEXT(0:20) =           &
      (/"X     ","Type1 ","Type2 ","Type3 ","Type4 ","Type5 ", & 
        "Type6 ","Type7 ","Type8 ","Type9 ","Type10",     &
        "Type11","Type12","Type13","Type14","Type15",     &
        "Type16","Type17","Type18","Type19","Type20"/)      ! Default text for printing AC data
   REAL(KIND=RK), PARAMETER :: PI =  3.14159265358979D0     ! Value of PI
   
   ! TYPE declarations

   TYPE :: CALENDAR                                         ! Stores calendar data for each day
      INTEGER (KIND=IK) :: Year       = 0                   ! Year for each day of simulation
      INTEGER (KIND=IK) :: Month      = 0                   ! Month for each day of simulation
      INTEGER (KIND=IK) :: Day        = 0                   ! Day of month for each simulation day
      INTEGER (KIND=IK) :: DayOfWeek  = 0                   ! Day of week (1=Sun, 2=Mon, etc)
      INTEGER (KIND=IK) :: FirstHour  = 0                   ! First hour of simulation for given day
      INTEGER (KIND=IK) :: LastHour   = 0                   ! Last hour of simulation for given day
      INTEGER (KIND=IK) :: Weekend    = 0                   ! Weekend status (Sat,Sun=1, others=0)
   END TYPE CALENDAR                                        ! Used to declare Calen 

   TYPE :: TEMPERDAY                                        ! Stores daily temperature data
      INTEGER (KIND=IK) :: MaxTemp                          ! Maximum daily 1-hour temperature (deg F)
      INTEGER (KIND=RK2):: AvgTemp                          ! Mean daily temperature (deg F)
      INTEGER (KIND=IK) :: TempCatM                         ! Daily categories for Max Temperature
      INTEGER (KIND=IK) :: TempCatA                         ! Daily categories for Avg Temperature
      INTEGER (KIND=IK) :: Pool                             ! Diary pool assignment for current day
   END TYPE TEMPERDAY                                       ! Used to declare TemData 

   TYPE :: POLLUTANT                                        ! Stores pollutant-specific data
      CHARACTER(LEN=LEN2) :: Name           = ""            ! Name of pollutant
      CHARACTER(LEN=LEN3) :: AQName         = ""            ! Name of AQ file for poll
      CHARACTER(LEN=LEN3) :: MSumName       = ""            ! Name of msum file for poll
      CHARACTER(LEN=LEN3) :: MResName       = ""            ! Name of mres file for poll
      CHARACTER(LEN=LEN3) :: TabName        = ""            ! Name of tables file for poll
      INTEGER  (KIND=IK)  :: DoDose         = NO            ! Flag for dose calculation
      INTEGER  (KIND=IK)  :: MSumU                          ! Unit # for micro sum file
      INTEGER  (KIND=IK)  :: MResU                          ! Unit # for micro result file
      INTEGER  (KIND=IK)  :: TabU                           ! Unit # for tables file
      INTEGER  (KIND=IK)  :: UseRoadAQ      = NO            ! Flag for using roadway concentrations
      INTEGER  (KIND=IK)  :: RoadLast       = NO            ! Flag for using H/W road concs      
      INTEGER  (KIND=IK)  :: InUnits     = PPM              ! Units for input air quality data
      INTEGER  (KIND=IK)  :: OutUnits    = PPM              ! Units for reporting exposure output
      REAL     (KIND=RK)  :: PPMFact     = 0.0              ! Conversion: number of ug/m3 in 1 ppm
      REAL     (KIND=RK)  :: AlertThresh = 0.0              ! Threshold concentration for warnings
      INTEGER  (KIND=IK)  :: NumSources  = 0                ! Maximum source number used
      INTEGER  (KIND=IK)  :: N_Pctl                         ! Percentiles for use on output tables
      INTEGER  (KIND=IK)  :: N_DM1HExpLev                   ! Levels for daily max 1-hour exposure
      INTEGER  (KIND=IK)  :: N_DM8HExpLev                   ! Levels for daily max 8-hour exposure
      INTEGER  (KIND=IK)  :: N_DAvgExpLev                   ! Levels for daily average exposure
      INTEGER  (KIND=IK)  :: N_SAvgExpLev                   ! Levels for annual average exposure
      INTEGER  (KIND=IK)  :: N_TimeExpLev                   ! Levels for exposure duration table
      INTEGER  (KIND=IK)  :: N_DMEHDoseLev                  ! Levels for daily max end-of-hour dose
      INTEGER  (KIND=IK)  :: N_DM1HDoseLev                  ! Levels for daily max 1-hour dose
      INTEGER  (KIND=IK)  :: N_DM8HDoseLev                  ! Levels for daily max 8-hour dose
      INTEGER  (KIND=IK)  :: N_DAvgDoseLev                  ! Levels for daily average dose
      INTEGER  (KIND=IK)  :: N_H_EHDoseLev                  ! Levels for end-of-hour dose
      INTEGER  (KIND=IK)  :: N_SAvgDoseLev                  ! Levels for annual average dose
      INTEGER  (KIND=IK)  :: N_TimeDoseLev                  ! Levels for dose duration table
      INTEGER  (KIND=IK)  :: N_TSExpLev                     ! Levels for timestep exposure
      INTEGER  (KIND=IK)  :: N_TSDoseLev                    ! Levels for timestep dose
      INTEGER  (KIND=IK)  :: N_DMTSExpLev                   ! Levels for daily max timestep exposure
      INTEGER  (KIND=IK)  :: N_DMTSDoseLev                  ! Levels for daily max timestep dose
      INTEGER  (KIND=IK)  :: N_TSMultiLev                   ! Cutpoints for multi exceedance tables
      INTEGER  (KIND=IK)  :: PM       = NO                  ! Flag for if pollut is a particle
      REAL     (KIND=RK)  :: Size     = 0.0                 ! Aerodynamic diameter (for particles)
      REAL     (KIND=RK)  :: Density  = 0.0                 ! Density (for particles)                     
   END TYPE POLLUTANT                                       ! Used to declare Pol

   TYPE :: PROFILE                                          ! Stores constant personal profile data 
      INTEGER (KIND=IK) :: Index      = 0                   ! Sequential number for profile
      INTEGER (KIND=IK) :: Gender     = 0                   ! Gender (1=male, 2=female)
      INTEGER (KIND=IK) :: PopCat     = 0                   ! Population category (input file #)
      INTEGER (KIND=IK) :: Age        = 0                   ! Age in years
      INTEGER (KIND=IK) :: HomeSec    = 0                   ! Number of home sector
      INTEGER (KIND=IK) :: WorkSec    = 0                   ! Work sector (=HomeSec if not worker)
      REAL    (KIND=RK) :: CommTime   = 0                   ! Commuting Time
      REAL    (KIND=RK) :: CommDist   = 0                   ! Commuting Distance
      INTEGER (KIND=IK) :: HomeD      = 0                   ! Number of home air district 
      INTEGER (KIND=IK) :: WorkD      = 0                   ! Work district (=HomeD if not worker)    
      INTEGER (KIND=IK) :: RoadD      = 0                   ! Roadway district
      INTEGER (KIND=IK) :: RoadWD     = 0                   ! Work Roadway district
      INTEGER (KIND=IK) :: HomeST     = 0                   ! Number of home state (subregion) 
      INTEGER (KIND=IK) :: WorkST     = 0                   ! Number of work state (subregion)
      INTEGER (KIND=IK) :: Zone       = 0                   ! Temperature zone for home sector
      INTEGER (KIND=IK) :: Employed   = 0                   ! Employment status (1=YES, 2=NO)
      INTEGER (KIND=IK) :: FactorGroup= 0                   ! Factor Group number 
      CHARACTER (LEN=LEN1) :: GroupName = ""                ! Name of profile factor group
      INTEGER (KIND=IK) :: GasStove   = 0                   ! Has gas stove (1=YES, 2=NO)
      INTEGER (KIND=IK) :: GasPilot   = 0                   ! Has gas stove pilot light (1=YES, 2=NO)
      INTEGER (KIND=IK) :: AC_Home    = 0                   ! Has home air conditioning (1=YES, 2=NO)
      INTEGER (KIND=IK) :: AC_Car     = 0                   ! Has car air conditioning (1=YES, 2=NO)
      INTEGER (KIND=IK) :: PCond1     = 0                   ! Value of profile conditional 1
      INTEGER (KIND=IK) :: PCond2     = 0                   ! Value of profile conditional 2
      INTEGER (KIND=IK) :: PCond3     = 0                   ! Value of profile conditional 3
      INTEGER (KIND=IK) :: PCond4     = 0                   ! Value of profile conditional 4
      INTEGER (KIND=IK) :: PCond5     = 0                   ! Value of profile conditional 5
      INTEGER (KIND=IK) :: RCond1     = 0                   ! Value of regional conditional 1
      INTEGER (KIND=IK) :: RCond2     = 0                   ! Value of regional conditional 2
      INTEGER (KIND=IK) :: RCond3     = 0                   ! Value of regional conditional 3
      INTEGER (KIND=IK) :: RCond4     = 0                   ! Value of regional conditional 4
      INTEGER (KIND=IK) :: RCond5     = 0                   ! Value of regional conditional 5
      REAL    (KIND=RK) :: PAI        = 0.0                 ! Median physical activity index
      INTEGER (KIND=IK) :: SEED       = 0                   ! Main initial random seed for person      
      INTEGER (KIND=IK) :: NumDiaries = 0                   ! Number of different activity diaries     
    END TYPE PROFILE                                        ! Used to declare Person 

   TYPE :: PHYSIOLOGY                                       ! Stores personal physiological data
      REAL    (KIND=RK) :: Blood    = 0.0                   ! Blood volume (ml) 
      REAL    (KIND=RK) :: BM       = 0.0                   ! Body mass (kg)
      REAL    (KIND=RK) :: BSA      = 0.0                   ! Body surface area (m^2)
      REAL    (KIND=RK) :: Diff     = 0.0                   ! Lung diffusivity (ml/min/torr) 
      REAL    (KIND=RK) :: ECF      = 0.0                   ! Energy conversion factor (liters-O2/kcal)
      REAL    (KIND=RK) :: Endgn1   = 0.0                   ! Endogenous CO production rate #1 (ml/min)
      REAL    (KIND=RK) :: Endgn2   = 0.0                   ! Endogenous CO production rate #2 (ml/min)
      REAL    (KIND=RK) :: Height   = 0.0                   ! Body height (inches) 
      REAL    (KIND=RK) :: Hmglb    = 0.0                   ! Hemoglobin density (g/ml-blood)
      REAL    (KIND=RK) :: MetsMax  = 0.0                   ! Maximum METS value (dimensionless)
      REAL    (KIND=RK) :: MOXD     = 0.0                   ! Maximum O2 debt (ml/kg body mass)
      REAL    (KIND=RK) :: RecTime  = 0.0                   ! Time to recover maximum O2 debt
      REAL    (KIND=RK) :: RMR      = 0.0                   ! Resting metabolic rate (kcal/min)
      REAL    (KIND=RK) :: Weight   = 0.0                   ! Body weight (pounds) 
      INTEGER (KIND=IK) :: Start    = 0                     ! Starting day for menstrual cycle 
      REAL    (KIND=RK) :: VEinter  = 0.0                   ! Intercept for ln(VE/VO2) relationship
      REAL    (KIND=RK) :: VEslope  = 0.0                   ! Slope for ln(VE/VO2) relationship
      REAL    (KIND=RK) :: VEresid  = 0.0                   ! Residual for ln(VE/VO2) relationship
      REAL    (KIND=RK) :: VO2Max   = 0.0                   ! Vo2max (L-O2/min)
      INTEGER (KIND=IK) :: Ill      = 0                     ! Disease status (1=YES, 2=NO)
      REAL    (KIND=RK) :: BMI      = 0                     ! Body mass index (lbs/in2)
      REAL    (KIND=RK) :: Beta1    = 0                     ! FEV1 beta1 parameter
      REAL    (KIND=RK) :: Beta2    = 0                     ! FEV1 beta2 parameter
      REAL    (KIND=RK) :: Beta3    = 0                     ! FEV1 beta3 parameter
      REAL    (KIND=RK) :: Beta4    = 0                     ! FEV1 beta4 parameter
      REAL    (KIND=RK) :: Beta5    = 0                     ! FEV1 beta5 parameter
      REAL    (KIND=RK) :: Beta6    = 0                     ! FEV1 beta6 parameter
      REAL    (KIND=RK) :: Beta7    = 0                     ! FEV1 beta7 parameter
      REAL    (KIND=RK) :: FEVU     = 0                     ! FEV1 FEVU parameter  
      REAL    (KIND=RK) :: FEVSLP   = 0                     ! FEV1 FEVSLP parameter
      REAL    (KIND=RK) :: FEVINT   = 0                     ! FEV1 FEVINT parameter                             
   END TYPE PHYSIOLOGY                                      ! Used to declare Phys 

   TYPE :: PROFILEDAY                                       ! Stores daily-varying personal data  
      INTEGER (KIND=IK) :: DiaryNum   = 0                   ! Selected diary # for each day
      INTEGER (KIND=IK) :: FirstEvent = 0                   ! First event # on EventSeq for given day
      INTEGER (KIND=IK) :: LastEvent  = 0                   ! Last event # on EventSeq for given day
      INTEGER (KIND=IK) :: WindowRes  = 0                   ! Residential window position category
      INTEGER (KIND=IK) :: WindowCar  = 0                   ! Vehicle window position category 
      INTEGER (KIND=IK) :: SpeedCat   = 0                   ! Vehicle speed category 
      INTEGER (KIND=IK) :: DCond1     = 0                   ! Conditional1 category 
      INTEGER (KIND=IK) :: DCond2     = 0                   ! Conditional2 category 
      INTEGER (KIND=IK) :: DCond3     = 0                   ! Conditional3 category 
      REAL    (KIND=RK) :: EndGn      = 0.0                 ! Endogenous CO production rate
      REAL    (KIND=RK) :: PAI        = 0.0                 ! Daily physical activity index
      REAL    (KIND=RK) :: Stat       = 0.0                 ! Diary statistic (key var) for day
      CHARACTER (LEN=9) :: DiaryID    = ""                  ! CHADID for each day
      INTEGER (KIND=IK) :: DiaryAge   = 0                   ! Age on selected CHAD diary for day
      INTEGER (KIND=IK) :: DiaryEmp   = 0                   ! Empl status for selected CHAD diary
      CHARACTER (LEN=LEN1) :: DiaryOcc= ""                  ! CHAD job name for each day
   END TYPE PROFILEDAY                                      ! Used to declare PersonDay 

   TYPE :: EVENTSEQUENCE                                    ! Stores event-based time profile data
      INTEGER (KIND=IK) :: Day        = 0                   ! Day of simulation (1-NumDays)
      INTEGER (KIND=IK) :: Hour       = 0                   ! Hour of simulation (1-NumHours)
      INTEGER (KIND=IK) :: TS         = 0                   ! Timestep of simulation (1-NumTS)
      REAL    (KIND=RK) :: Deficit    = 0.0                 ! Oxygen debt, percent of maximal
      INTEGER (KIND=IK) :: Duration   = 0                   ! Duration of event in minutes (1-60) 
      INTEGER (KIND=IK) :: Activity   = 0                   ! Activity code for event (may be negative) 
      INTEGER (KIND=IK) :: ActCode    = 0                   ! CHAD Activity code for event
      INTEGER (KIND=IK) :: Location   = 0                   ! Location code for event (0-115)
      INTEGER (KIND=IK) :: HomeWork   = 0                   ! Place code (1=H, 2=W, 3=O, 4=R, 5=NH, 6=NW, 7=RW)                     
      REAL    (KIND=RK) :: METS       = 0.0                 ! METS value for event
      REAL    (KIND=RK) :: VA         = 0.0                 ! Alveolar ventilation rate (ml/min) 
      REAL    (KIND=RK) :: VE         = 0.0                 ! Ventilation (volume expired) rate (l/min)
      REAL    (KIND=RK) :: EVR        = 0.0                 ! Ventilation (volume expired) rate (l/min)
      REAL    (KIND=RK) :: Exposure   = 0.0                 ! Average exposure over the event
      REAL    (KIND=RK) :: Amb        = 0.0                 ! Ambient conc for that event
      REAL    (KIND=RK) :: AmbHome    = 0.0                 ! Ambient conc in home district for event
      REAL    (KIND=RK) :: Dose       = 0.0                 ! Dose for the event
      REAL    (KIND=RK) :: FDose      = 0.0                 ! Final dose at end of event      
      REAL    (KIND=RK) :: DepDose    = 0.0                 ! Deposited mass dose for event (PM)
      REAL    (KIND=RK) :: IntakeDose = 0.0                 ! Intake dose rate for event (PM)    
      REAL    (KIND=RK) :: UMET       = 0.0                 ! Unmodified METS values for event
      REAL    (KIND=RK) :: FinalFEV   = 0.0                 ! Final FEV value for each event
      REAL    (KIND=RK) :: InitialFEV = 0.0                 ! Initial FEV for each event
      REAL    (KIND=RK) :: FinalX     = 0.0                 ! Final X used in FEV calc for each event
      REAL    (KIND=RK) :: FEVE1      = 0.0                 ! E1 term used in FEV calc for each event      
      REAL    (KIND=RK) :: FEVE2      = 0.0                 ! E2 term used in FEV calc for each event            
   END TYPE EVENTSEQUENCE                                   ! Used to declare EventSeq 

   TYPE :: HOURSEQUENCE                                     ! Stores hourly time sequence data
      INTEGER (KIND=IK) :: FirstEvent = 0                   ! # of first event for that hour 
      INTEGER (KIND=IK) :: LastEvent  = 0                   ! # of last event for that hour
      REAL    (KIND=RK) :: METS       = 0.0                 ! Mean METS value for that hour
      REAL    (KIND=RK) :: Exposure   = 0.0                 ! Average exposure for that hour
      REAL    (KIND=RK) :: Dose       = 0.0                 ! Average dose for that hour
      REAL    (KIND=RK) :: EF         = 0.0                 ! Average exp factor for that hour
      REAL    (KIND=RK) :: Amb        = 0.0                 ! Average ambient conc for that hour
      REAL    (KIND=RK) :: VE         = 0.0                 ! Average ventilation rate for that hour
      REAL    (KIND=RK) :: VA         = 0.0                 ! Average alv vent rate for that hour  
      REAL    (KIND=RK) :: EE         = 0.0                 ! Average energy expediture for that hour  
      REAL    (KIND=RK) :: EVR        = 0.0                 ! Average EVR (Ve/BSA) for that hour  
      REAL    (KIND=RK) :: FDose      = 0.0                 ! End-of-hour dose for that hour
      REAL    (KIND=RK) :: Run8Exp    = 0.0                 ! Running 8-hour average exposure
      REAL    (KIND=RK) :: Run8EVR    = 0.0                 ! Running 8-hour average EVR
      REAL    (KIND=RK) :: Run8Dose   = 0.0                 ! Running 8-hour average dose
      REAL    (KIND=RK) :: PAI        = 0.0                 ! Average hourly physical activity index            
      REAL    (KIND=RK) :: DepDose    = 0.0                 ! Deposited mass dose for hour (PM)
      REAL    (KIND=RK) :: IntakeDose = 0.0                 ! Intake dose rate for hour (PM)      
      REAL    (KIND=RK) :: Run8AmbHome = 0.0                ! Running 8-hour ambient concentration
      REAL    (KIND=RK) :: AmbHome    = 0.0                 ! Ambient home concentration      
      REAL    (KIND=RK) :: HomeEF     = 0.0                 ! Hourly Exposure Factor Exposure/Home Conc  
      REAL    (KIND=RK) :: DFEV1max   = 0.0                 ! Hourly maximum of the event level dFEV1
      REAL    (KIND=RK) :: FEVE1      = 0                   ! FEVE1 parameter
      REAL    (KIND=RK) :: FEVE2      = 0                   ! FEVE2 parameter                 
   END TYPE HOURSEQUENCE                                    ! Used to declare HourSeq 

   TYPE :: TIMESTEPSEQUENCE                                 ! Stores TS time sequence data
      INTEGER (KIND=IK) :: FirstEvent = 0                   ! # of first event for that TS 
      INTEGER (KIND=IK) :: LastEvent  = 0                   ! # of last event for that TS
      INTEGER (KIND=IK) :: Hour       = 0                   ! Hour for that TS
      REAL    (KIND=RK) :: METS       = 0.0                 ! Mean METS value for that TS
      REAL    (KIND=RK) :: Exposure   = 0.0                 ! Average exposure for that TS
      REAL    (KIND=RK) :: Dose       = 0.0                 ! Average dose for that TS
      REAL    (KIND=RK) :: EF         = 0.0                 ! Average exp factor for that TS
      REAL    (KIND=RK) :: Amb        = 0.0                 ! Average ambient conc for that TS
      REAL    (KIND=RK) :: VE         = 0.0                 ! Average ventilation rate for that TS
      REAL    (KIND=RK) :: VA         = 0.0                 ! Average alv vent rate for that TS  
      REAL    (KIND=RK) :: EE         = 0.0                 ! Average energy expediture for that TS  
      REAL    (KIND=RK) :: EVR        = 0.0                 ! Average EVR (Ve/BSA) for that TS  
      REAL    (KIND=RK) :: FDose      = 0.0                 ! End-of-TS dose for that TS
      REAL    (KIND=RK) :: PAI        = 0.0                 ! Average TS physical activity index            
      REAL    (KIND=RK) :: DepDose    = 0.0                 ! Deposited mass dose for TS (PM)
      REAL    (KIND=RK) :: IntakeDose = 0.0                 ! Intake dose rate for TS (PM)  
      REAL    (KIND=RK) :: dFEV1max   = 0.0                 ! TImestep maximum of the event level dFEV1                  
   END TYPE TIMESTEPSEQUENCE                                ! Used to declare TSSeq 

   TYPE :: HOURMETEOR                                       ! Stores hourly meteorolog. data
      CHARACTER (LEN=LEN1) :: Precip     = ""               ! Hourly precip
      INTEGER (KIND=IK)    :: Temp       = 0                ! Hourly temperature
      INTEGER (KIND=IK)    :: Humid      = 0                ! Hourly humidity
      INTEGER (KIND=IK)    :: Wind       = 0                ! Hourly windspeed 
      INTEGER (KIND=IK)    :: Dir        = 0                ! Hourly wind dir
      INTEGER (KIND=IK)    :: TempCat                       ! Hourly categories for temperature
      INTEGER (KIND=IK)    :: HumidCat                      ! Hourly categories for humidity
      INTEGER (KIND=IK)    :: PrecipCat                     ! Hourly categories for precip
      INTEGER (KIND=IK)    :: WindCat                       ! Hourly categories for windspeed
      INTEGER (KIND=IK)    :: DirCat                        ! Hourly categories for wind dir
   END TYPE HOURMETEOR                                      ! Used to declare HourMtr

   TYPE :: AQCONDS                                          ! Stores TS AQ data
      INTEGER (KIND=IK)    :: AQCond0 = 0                   ! TS categories for AQ Conditional 0   
      INTEGER (KIND=IK)    :: AQCond1 = 0                   ! TS categories for AQ Conditional 1
      INTEGER (KIND=IK)    :: AQCond2 = 0                   ! TS categories for AQ Conditional 2
      INTEGER (KIND=IK)    :: AQCond3 = 0                   ! TS categories for AQ Conditional 3
      INTEGER (KIND=IK)    :: AQCond4 = 0                   ! TS categories for AQ Conditional 4
      INTEGER (KIND=IK)    :: AQCond5 = 0                   ! TS categories for AQ Conditional 5
   END TYPE AQCONDS                                         ! Used to declare AQCONDS
   
   TYPE :: EXPDAY                                           ! Stores daily exposure and dose data
      REAL    (KIND=RK) :: HomeConc   = 0.0                 ! Daily conc in home location
      REAL    (KIND=RK) :: Max1Exp    = 0.0                 ! Daily maximum 1-hour average exposure
      REAL    (KIND=RK) :: Max8Exp    = 0.0                 ! Daily maximum 8-hour average exposure
      REAL    (KIND=RK) :: MaxTSExp   = 0.0                 ! Daily maximum timestep avg exposure
      REAL    (KIND=RK) :: AvgExp     = 0.0                 ! Daily average exposure
      REAL    (KIND=RK) :: Max8EC     = 0.0                 ! Conc average corresponding to 8-hr max      
      REAL    (KIND=RK) :: dFEV1max   = 0.0                 ! Daily maximum of the event-level %dFEV1      
   END TYPE EXPDAY                                          ! Used to declare DailyExp 
   
   TYPE :: DOSEDAY                                          ! Stores daily exposure and dose data
      REAL    (KIND=RK) :: Max1Dose   = 0.0                 ! Daily maximum 1-hour average dose
      REAL    (KIND=RK) :: Max8Dose   = 0.0                 ! Daily maximum 8-hour average dose
      REAL    (KIND=RK) :: Max1FDose  = 0.0                 ! Daily maximum end-of-hour dose
      REAL    (KIND=RK) :: MaxTSDose  = 0.0                 ! Daily maximum timestep avg dose
      REAL    (KIND=RK) :: AvgDose    = 0.0                 ! Daily average dose
      REAL    (KIND=RK) :: DepDose    = 0.0                 ! Deposited mass dose for day (PM)
      REAL    (KIND=RK) :: IntakeDose = 0.0                 ! Intake dose rate for day (PM)      
   END TYPE DOSEDAY                                         ! Used to declare DailyDose 

   TYPE :: SITE                                             ! Stores geographical data for sites
      INTEGER  (KIND=IK)  :: Index    = 0                   ! Sequential index number for site
      CHARACTER(LEN=LEN2) :: Name     = ""                  ! Site name (Max of LEN2 characters)
      REAL     (KIND=RK)  :: Lat      = 99999.              ! Latitude of site (decimal degrees) 
      REAL     (KIND=RK)  :: Lon      = 99999.              ! Longitude (decimal, West=negative)
      REAL     (KIND=RK)  :: DistC    = -1.                 ! Distance in km to study area center
      REAL     (KIND=RK)  :: DistD    = -1.                 ! Distance in km to district center
      REAL     (KIND=RK)  :: RoadD    = -1.                 ! Distance in km to roadway district center      
      REAL     (KIND=RK)  :: DistZ    = -1.                 ! Distance in km to zone center
      INTEGER  (KIND=IK)  :: NumS     = 0                   ! Sector count for mapping
      INTEGER  (KIND=IK)  :: NumD     = 0                   ! District count for mapping
      INTEGER  (KIND=IK)  :: NumRD    = 0                   ! Road district count or mapping
      INTEGER  (KIND=IK)  :: NumZ     = 0                   ! Zone count or mapping 
      INTEGER  (KIND=IK)  :: NumR     = 0                   ! Region mapping for CVs
      INTEGER  (KIND=IK)  :: CV       = NO                  ! Flag for regional CV
      REAL     (KIND=RK)  :: Radius   = 99999               ! District radius 
   END TYPE SITE                                            ! Used to declare SList, DList, ZList 

   TYPE :: CONDVAR                                          ! Stores the conditional variable data
      INTEGER  (KIND=IK)  :: Index    = 0                   ! Position on CV list
      CHARACTER(LEN=LEN2) :: Name     = ""                  ! Name of profile var (max LEN2 chars)
      INTEGER  (KIND=IK)  :: Levels   = 1                   ! # of responses
      INTEGER  (KIND=IK)  :: Rate     = 1                   ! Rate of change (1=hourly, 2=daily)
      INTEGER  (KIND=IK)  :: Required = NO                  ! Flag for whether CV must be defined
      INTEGER  (KIND=IK)  :: Supplied = NO                  ! Flag for user-supplied definition 
   END TYPE CONDVAR                                         ! Used to declare CVList 
   
   TYPE :: POPTYPE                                          ! Stores info on population categories
      INTEGER  (KIND=IK)  :: Index    = 0                   ! Position on PopCat list
      CHARACTER(LEN=6)    :: Gender   = ""                  ! Gender of this PopCat
      CHARACTER(LEN=5)    :: Race     = ""                  ! Race of this PopCat
      CHARACTER(LEN=LEN3) :: RaceDesc = ""                  ! Longer description for race 
      CHARACTER(LEN=LEN3) :: FName    = ""                  ! Population file for this PopCat
      INTEGER  (KIND=IK)  :: TotPop   = 0                   ! Total population for this PopCat
      REAL     (KIND=RK)  :: Frac     = 0.                  ! Population fraction of all PopCats
   END TYPE POPTYPE                                         ! Used to declare PopCat   
   ! GLOBAL variable list
    
   ! SIMULATION variables  -  From PARAMS file or derived
   INTEGER  (KIND=IK)  :: AgeMin           = 0              ! Minimum age in simulation
   INTEGER  (KIND=IK)  :: AgeMax           = 0              ! Maximum age in simulation
   INTEGER  (KIND=IK)  :: AlertFlag        = 0              ! Flag indicating high exposures
   INTEGER  (KIND=IK)  :: AllocErr         = 1              ! Variable for array allocation status 
   INTEGER  (KIND=IK)  :: ChildMin         = 0              ! Minimum age of children for tables  
   INTEGER  (KIND=IK)  :: ChildMax         = 0              ! Maximum age of children for tables
   INTEGER  (KIND=IK)  :: Clock(2)         = 0              ! Job timing variables
   INTEGER  (KIND=IK)  :: CommuteFlag      = NO             ! Flag for implementing commuting 
   INTEGER  (KIND=IK)  :: RoadWayFlag      = NO             ! Flag for using roadway concentrations    
   INTEGER  (KIND=IK)  :: RoadLastFlag     = NO             ! Flag for using roadwaylast option
   INTEGER  (KIND=IK)  :: HourlyFEVE1Flag  = NO             ! Flag for using hourly FEV E1 option
   INTEGER  (KIND=IK)  :: HourlyFEVE2Flag  = NO             ! Flag for using hourly FEV E2 option   
   INTEGER  (KIND=IK)  :: CountyList       = NO             ! Flag for list of study area counties 
   INTEGER  (KIND=IK)  :: TractList        = NO             ! Flag for list of study area tracts 
   INTEGER  (KIND=IK)  :: CustomWork       = NO             ! Flag for custom work activities
   INTEGER  (KIND=IK)  :: DailyOut         = NO             ! Flag for daily output file
   INTEGER  (KIND=IK)  :: DebugLevel       = 0              ! Level for printing messages to log
   INTEGER  (KIND=IK)  :: DoCustProfs      = NO             ! Flag for doing physiology and dose   
   INTEGER  (KIND=IK)  :: DoHours          = YES            ! Flag for calculating hour sequence   
   INTEGER  (KIND=IK)  :: DoTimeSteps      = NO             ! Flag for doing ts tables and file
   INTEGER  (KIND=IK)  :: DSTAdjust        = YES            ! Flag for Daylight Savings Time 
   INTEGER  (KIND=IK)  :: EndDate          = 0              ! Last day of simulation (YYYYMMDD)
   INTEGER  (KIND=IK)  :: EventSample      = 10             ! Number of event profiles to output
   INTEGER  (KIND=IK)  :: EventsOut        = NO             ! Flag for event-level output
   INTEGER  (KIND=IK)  :: EventsPerson     = NO             ! Flag for event data for current person
   INTEGER  (KIND=IK)  :: EventStart       = 1              ! First event on curent diary to output 
   INTEGER  (KIND=IK)  :: EventStop        = 0              ! Last event on current diary to output
   INTEGER  (KIND=IK)  :: HourlyOut        = NO             ! Flag for hourly output file
   INTEGER  (KIND=IK)  :: Ios(MAXU)        = 0              ! Status of I/O actions
   INTEGER  (KIND=IK)  :: KeepLeavers      = NO             ! Status for commuters leaving study area
   INTEGER  (KIND=IK)  :: LogIsOpen        = NO             ! Status of log file
   INTEGER  (KIND=IK)  :: LogDistrict      = YES            ! Print district info to log file
   INTEGER  (KIND=IK)  :: LogPopulate      = YES            ! Print population info to log file
   INTEGER  (KIND=IK)  :: LogProfiles      = YES            ! Print profile info to log file
   INTEGER  (KIND=IK)  :: LogSectors       = NO             ! Print sector info to log file
   INTEGER  (KIND=IK)  :: LogTables        = NO             ! Print output tables to log file
   INTEGER  (KIND=IK)  :: LogZones         = YES            ! Print zone info to log file
   INTEGER  (KIND=IK)  :: LongitDiary      = NO             ! Use longitudinal diary assembly
   INTEGER  (KIND=IK)  :: ClustDiaryB      = NO             ! Use Clustering diary assembly B
   INTEGER  (KIND=IK)  :: ClustDiaryA      = NO             ! Use Clustering diary assembly A
   INTEGER  (KIND=IK)  :: RerunClus        = NO             ! Use Clustering diary assembly A     
   INTEGER  (KIND=IK)  :: MaxLevels        = 0              ! Maximum number of table levels used 
   INTEGER  (KIND=IK)  :: FEVFlag          = 0              ! Calculate dFEV%?   
   INTEGER  (KIND=IK)  :: MaxCorrNum       = 0              ! Maximum number of correlation subsets
   INTEGER  (KIND=IK)  :: MSumOut          = YES            ! Flag for micro sum file 
   INTEGER  (KIND=IK)  :: MResOut          = NO             ! Flag for micro results file 
   INTEGER  (KIND=IK)  :: ModelAQVar       = NO             ! Flag for AQ variability    
   INTEGER  (KIND=IK)  :: NumAC_Home       = 0              ! Number of AC categories for home
   INTEGER  (KIND=IK)  :: NumAC_Car        = 0              ! Number of AC categories for car
   INTEGER  (KIND=IK)  :: NumAvgTemp       = 0              ! Number of avg temperature categories
   INTEGER  (KIND=IK)  :: NumPCond1        = 0              ! Number of profile cond1 categories
   INTEGER  (KIND=IK)  :: NumPCond2        = 0              ! Number of profile cond2 categories
   INTEGER  (KIND=IK)  :: NumPCond3        = 0              ! Number of profile cond3 categories
   INTEGER  (KIND=IK)  :: NumPCond4        = 0              ! Number of profile cond4 categories
   INTEGER  (KIND=IK)  :: NumPCond5        = 0              ! Number of profile cond5 categories
   INTEGER  (KIND=IK)  :: NumRCond1        = 0              ! Number of regional cond1 categories
   INTEGER  (KIND=IK)  :: NumRCond2        = 0              ! Number of regional cond2 categories
   INTEGER  (KIND=IK)  :: NumRCond3        = 0              ! Number of regional cond3 categories
   INTEGER  (KIND=IK)  :: NumRCond4        = 0              ! Number of regional cond4 categories
   INTEGER  (KIND=IK)  :: NumRCond5        = 0              ! Number of regional cond5 categories
   INTEGER  (KIND=IK)  :: NumAQCond1       = 0              ! Number of AQ1 categories
   INTEGER  (KIND=IK)  :: NumAQCond2       = 0              ! Number of AQ2 categories
   INTEGER  (KIND=IK)  :: NumAQCond3       = 0              ! Number of AQ3 categories
   INTEGER  (KIND=IK)  :: NumAQCond4       = 0              ! Number of AQ4 categories
   INTEGER  (KIND=IK)  :: NumAQCond5       = 0              ! Number of AQ5 categories            
   INTEGER  (KIND=IK)  :: NumDCond1        = 0              ! Number of daily cond1 categories
   INTEGER  (KIND=IK)  :: NumDCond2        = 0              ! Number of daily cond2 categories
   INTEGER  (KIND=IK)  :: NumDCond3        = 0              ! Number of daily cond3 categories
   INTEGER  (KIND=IK)  :: NumCounties      = 0              ! Number of counties on input list
   INTEGER  (KIND=IK)  :: NumTracts        = 0              ! Number of tracts on input list
   INTEGER  (KIND=IK)  :: NumCustProfs     = 0              ! Number of custom profiles
   INTEGER  (KIND=IK)  :: NumWork          = 0              ! Number of custom work activities
   INTEGER  (KIND=IK)  :: NumCV            = 0              ! Number of profile variables used as CV
   INTEGER  (KIND=IK)  :: NumCVFixed       = 0              ! Number of CV not changing daily
   INTEGER  (KIND=IK)  :: NumDays          = 0              ! Number of days in simulation
   INTEGER  (KIND=IK)  :: NumDir           = 0              ! Number of wind direction categories
   INTEGER  (KIND=IK)  :: NumDistricts     = 0              ! # air quality districts
   INTEGER  (KIND=IK)  :: NumEvents        = 0              ! Number of events on composite diary
   INTEGER  (KIND=IK)  :: NumHours         = 0              ! Number of hours in simulation
   INTEGER  (KIND=IK)  :: NumHumid         = 0              ! Number of humidity categories
   INTEGER  (KIND=IK)  :: NumMaxTemp       = 0              ! Number of max temperature categories
   INTEGER  (KIND=IK)  :: NumMicros        = 0              ! Number of microenvironments
   INTEGER  (KIND=IK)  :: NumLocs          = 3              ! Number of locations used
   INTEGER  (KIND=IK)  :: NumMicrosPr      = 0              ! Number of microenvironments to print
   INTEGER  (KIND=IK)  :: NumMP            = 0              ! Number of Micro-Param combinations
   INTEGER  (KIND=IK)  :: NumMPDistrib     = 0              ! Number of MP distributions
   INTEGER  (KIND=IK)  :: NumODistricts    = 0              ! Number of "other" districts
   INTEGER  (KIND=IK)  :: NumPollutants    = 1              ! Number of pollutants
   INTEGER  (KIND=IK)  :: NumPools         = 0              ! Number of diary pools
   INTEGER  (KIND=IK)  :: NumClusters      = 3              ! Number of clusters    
   INTEGER  (KIND=IK)  :: NumPopTypes      = 0              ! Number of gender-race combinations
   INTEGER  (KIND=IK)  :: NumProfiles      = 0              ! # personal profiles to run in model
   INTEGER  (KIND=IK)  :: NumPrecip        = 0              ! Number of precipitation categories
   INTEGER  (KIND=IK)  :: NumRaces         = 0              ! Number of race categories
   INTEGER  (KIND=IK)  :: NumSectors       = 0              ! Number of population sectors (tracts)
   INTEGER  (KIND=IK)  :: NumSpeeds        = 0              ! Number of speed categories
   INTEGER  (KIND=IK)  :: NumTemp          = 0              ! Number of temperature categories
   INTEGER  (KIND=IK)  :: NumTS            = 0              ! Number of timesteps in simulation
   INTEGER  (KIND=IK)  :: NumWinCar        = 0              ! Number of window positions for car
   INTEGER  (KIND=IK)  :: NumWind          = 0              ! Number of windspeed categories
   INTEGER  (KIND=IK)  :: NumWinRes        = 0              ! Number of window positions for home
   INTEGER  (KIND=IK)  :: NumYears         = 0              ! Number of years (including part years)
   INTEGER  (KIND=IK)  :: NumZones         = 0              ! Number of temperature zones
   INTEGER  (KIND=IK)  :: OnePercent       = NO
   INTEGER  (KIND=IK)  :: ProcLevel        = 0              ! Depth of procedure nesting
   INTEGER  (KIND=IK)  :: Rollback         = NO             ! Flag for rolling back ambient values
   INTEGER  (KIND=IK)  :: SampleLocs       = NO             ! Flag for sampling other locations
   INTEGER  (KIND=IK)  :: StartDate        = 0              ! First day of simulation (YYYYMMDD)
   INTEGER  (KIND=IK)  :: FirstProfile     = 1              ! First profile to simulate
   INTEGER  (KIND=IK)  :: TimeStepOut      = NO             ! Flag for writing the timestep file
   INTEGER  (KIND=IK)  :: TimeStepsPerDay  = 24             ! Number of timesteps in a 24-hour day
   INTEGER  (KIND=IK)  :: TSMinutes        = 60             ! Number of minutes in a timestep
   INTEGER  (KIND=IK)  :: TotPop           = 0              ! Population universe
   INTEGER  (KIND=IK)  :: MINDIARIES       = 0              ! Diary minimum in clustering algorithm   
   INTEGER  (KIND=IK)  :: UseDef(MAXINPUTS)= YES            ! Vector of flags denoting default use
   INTEGER  (KIND=IK)  :: PSumFlags(NUMPSUMVARS+4)          ! Flags for Psum file variables
   INTEGER  (KIND=IK)  :: DayFlags(NUMDAYVARS+10)           ! Flags for daily file variables
   INTEGER  (KIND=IK)  :: TSFlags(NUMTSVARS)                ! Flags for ts file variables
   INTEGER  (KIND=IK)  :: MList(MAXMIC)                     ! Array of micro numbers for output
   INTEGER  (KIND=IK)  :: MResFlags(NUMMRESVARS)            ! Flags for MRes file variables
   INTEGER  (KIND=IK)  :: EventFlags(NUMEVENTVARS)          ! Flags for Event file variables
   INTEGER  (KIND=IK)  :: TableFlags(NUMTABLEVARS)          ! Flags for Table file variables
   INTEGER  (KIND=IK)  :: VAoutput         = NO             ! Flag for writing VA to output files 
   INTEGER  (KIND=IK)  :: OccFactor        = NO             ! Flag for tying factor to occupation
   INTEGER  (KIND=IK)  :: MResHome         = NO             ! Flag for writing mres info for home only
   INTEGER  (KIND=IK)  :: ResampleNear     = NO             ! Flag for reasmpling nearby home/work tracts
   INTEGER  (KIND=IK)  :: UseProfileFactors = NO            ! Flag for using profile factors
   INTEGER  (KIND=IK)  :: NumLevels        = 0              ! Number of profile factor levels
   INTEGER  (KIND=IK)  :: UseOccGroups     = NO             ! Flag for matching via employment group
   INTEGER  (KIND=IK)  :: UseCHADOccFile   = NO             ! Flag for using external CHAD empl file
   INTEGER  (KIND=IK)  :: NumFEV           = 0              ! Number of FEV1RISK probability vectors
   INTEGER  (KIND=IK)  :: PrintCounter     = 0              ! Counter for reporting progress on log
   INTEGER  (KIND=IK), ALLOCATABLE :: HourFlags(:)          ! Flags for hour file variables
   INTEGER  (KIND=IK), ALLOCATABLE :: CustProfs(:)          ! Vector for custom profile samples
   INTEGER  (KIND=IK), ALLOCATABLE :: WorkActs(:)           ! Vector for custom work activities
   REAL     (KIND=RK)  :: ActivePAI        = 0.0            ! Median active PAI
   REAL     (KIND=RK)  :: AirRadius        = 99999.         ! Radius of air monitor districts in km
   REAL     (KIND=RK)  :: NearbyRadius     = 99999.         ! Radius of nearby sample in km
   REAL     (KIND=RK)  :: AgeCutPct        = 0.0            ! Percent variation allowed in age match
   REAL     (KIND=RK)  :: Age2Probab       = 0.0            ! Prob. for selecting shoulder age match
   REAL     (KIND=RK)  :: ComProbab1       = 0.0            ! Prob. for selecting shoulder com1 match
   REAL     (KIND=RK)  :: ComProbab2       = 0.0            ! Prob. for selecting shoulder com2 match
   REAL     (KIND=RK)  :: ComWindow1       = 0.0            ! Percent variation allowed in com1 match
   REAL     (KIND=RK)  :: ComWindow2       = 0.0            ! Percent variation allowed in com2 match          
   REAL     (KIND=RK)  :: Altitude         = 0.0            ! Altitude in feet (for lung physiology)
   REAL     (KIND=RK)  :: AutoC            = 0.0            ! Longitudinal diary autocorr    
   REAL     (KIND=RK)  :: CityRadius       = 99999.         ! Radius of study area in km
   REAL     (KIND=RK)  :: COHbFact         = 0.0            ! COHb convergence safety factor
   REAL     (KIND=RK)  :: DiaryD           = 0.0            ! Longitudinal diary diversity   
   REAL     (KIND=RK)  :: HeavyEVR1        = 0.0            ! Limit for "heavy: 1-Hour EVR 
   REAL     (KIND=RK)  :: HeavyEVR8        = 0.0            ! Limit for "heavy: 8-Hour EVR  
   REAL     (KIND=RK)  :: HeavyEVRTS       = 0.0            ! Limit for "heavy: timestep EVR 
   REAL     (KIND=RK)  :: HomeProb         = 0.0            ! Prob of selecting home dist for "other"
   REAL     (KIND=RK)  :: LeaverMult       = 0.0            ! Factor for those who commute out
   REAL     (KIND=RK)  :: LeaverAdd        = 0.0            ! Additive term for those who commute out 
   REAL     (KIND=RK)  :: MissGender       = 0.0            ! Prob. of selecting missing gender 
   REAL     (KIND=RK)  :: MissEmpl         = 0.0            ! Prob. of selecting missing employment 
   REAL     (KIND=RK)  :: MissOcc          = 0.0            ! Prob. of selecting missing occupation
   REAL     (KIND=RK)  :: MissAge          = 0.0            ! Prob. of selecting missing age diaries
   REAL     (KIND=RK)  :: ModEVR1          = 0.0            ! Limit for "moderate: 1-Hour EVR 
   REAL     (KIND=RK)  :: ModEVR8          = 0.0            ! Limit for "moderate: 8-Hour EVR 
   REAL     (KIND=RK)  :: ModEVRTS         = 0.0            ! Limit for "moderate: timestep EVR 
   REAL     (KIND=RK)  :: RBTarget         = 0.0            ! Rollback target
   REAL     (KIND=RK)  :: RBMax            = 0.0            ! Rollback max
   REAL     (KIND=RK)  :: RBFactor         = 0.0            ! Rollback factor
   REAL     (KIND=RK)  :: RBBkgnd          = 0.0            ! Rollback background level
   REAL     (KIND=IK)  :: TimestepsPerHour = 1.0            ! Number of timesteps in an hour
   REAL     (KIND=RK)  :: ZoneRadius       = 99999.         ! Radius of temperature zone in km
   CHARACTER(LEN=5)    :: CList(MAXCOUNTY) = "00000"        ! Array of county FIPS codes
   CHARACTER(LEN=11)   :: TList(MAXTRACT)  = "00000000000"  ! Array of census tract numbers 
   CHARACTER(LEN=20)   :: FEVDescList(MAXFEV) = ""          ! Array of FEV Risk descriptions
   CHARACTER(LEN=10)   :: DateTime(3)      = ""             ! Date and time at start of job
   CHARACTER(LEN=7)    :: TSText           = ""             ! Text for labeling timestep tables
   CHARACTER(LEN=LEN3) :: FName(10:MAXU)   = ""             ! Array of file names for OPEN statements
   CHARACTER(LEN=LEN3) :: Location         = ""             ! Study area location
   CHARACTER(LEN=LEN1) :: Disease          = ""             ! Name of disease to model 
   CHARACTER(LEN=LEN2) :: ProcName(10)     = ""             ! Procedure names for error routines
   CHARACTER(LEN=LEN3) :: Scenario         = ""             ! Name of scenario
   CHARACTER(LEN=LEN3) :: ParamsHeader     = ""             ! Header line on Parameters input file
   CHARACTER(LEN=200 ) :: PolList          = ""             ! List of pollutants
   CHARACTER(LEN=LEN3) :: VarNames(MAXINPUTS)=             &
      (/"ModEVRTS          ","HeavyEVRTS        ",         &
        "Pollutant         ","Location          ",         &
        "Scenario          ","InputUnits        ",         &
        "OutputUnits       ","Rollback          ",         &
        "DSTAdjust         ","HourlyOut         ",         &
        "HourlyList        ","MResOut           ",         & 
        "MSumOut           ","EventsOut         ",         & 
        "DoDose            ","Commuting         ",         &
        "KeepLeavers       ","LogDistrict       ",         & 
        "LogPopulation     ","LogProfiles       ",         & 
        "LogSectors        ","LogTables         ",         & 
        "LogZones          ","VAOutput          ",         & 
        "CountyList        ","CustomSample      ",         & 
        "Start_Date        ","End_Date          ",         & 
        "#Profiles         ","#Micros           ",         & 
        "#Sources          ","AgeMin            ",         & 
        "AgeMax            ","RandomSeed        ",         & 
        "DebugLevel        ","Latitude          ",         & 
        "Longitude         ","CityRadius        ",         & 
        "AirRadius         ","ZoneRadius        ",         & 
        "PPMFactor         ","RBTarget          ",         & 
        "RBBackGnd         ","RBMax             ",         & 
        "MissGender        ","MissEmpl          ",         & 
        "MissAge           ","AgeCutPct         ",         & 
        "Age2Probab        ","Altitude          ",         & 
        "ComCut1           ","ComCut2           ",         & 
        "ComProbab1        ","ComProbab2        ",         &         
        "COHbFactor        ","AlertThresh       ",         & 
        "LeaverMult        ","LeaverAdd         ",         & 
        "EventSample       ","ActivePAI         ",         & 
        "ChildMin          ","ChildMax          ",         & 
        "HeavyEVR1         ","HeavyEVR8         ",         & 
        "ModEVR1           ","ModEVR8           ",         & 
        "DiaryD            ","DiaryAutoC        ",         & 
        "LongitDiary       ","Disease           ",         & 
        "FirstProfile      ","#Pollutants       ",         & 
        "PSumList          ","CustomWork        ",         & 
        "HomeProbability   ","SampleOtherLocs   ",         & 
        "#OtherDistricts   ","TractList         ",         & 
        "MResList          ","MResHome          ",         & 
        "MResMicros        ","DailyOut          ",         & 
        "DailyList         ","ModelAQVar        ",         &
        "TimeStepsPerDay   ","DMTSExp           ",         & 
        "TSExp             ","DMTSDose          ",         & 
        "TSDose            ","TimeStepList      ",         & 
        "Size              ","Density           ",         & 
        "TimeStepOut       ","TSMultiCutpoints  ",         &
        "EventsList        ","TablesList        ",         &
        "ClustDiaryA       ","ClustDiaryB       ",         &
        "RerunClus         ","Roadway           ",         &
        "NearbyRadius      ","ResampleN         ",         &
        "RoadLast          ","HourlyFEVE1       ",         &
        "HourlyFEVE2       ","OccFactor         ",         &
        "UseOccGroups      ","MissOccup         ",         &
        "OnePercent"/)                                      ! Input variable names 
   CHARACTER(LEN=LEN3), ALLOCATABLE  :: AirHeader(:)        ! Header line on Air Quality input file
   TYPE     (SITE)     :: Center                            ! Coordinates of study area center
   TYPE     (POPTYPE),   ALLOCATABLE :: PopCat(:)           ! Array of population categories
   TYPE     (POLLUTANT), ALLOCATABLE :: Pol(:)              ! Array of population categories
       
   ! Random Number seeds - Set in ReadParams or SetSeeds
   INTEGER (KIND=IK) :: ClockSeed       = YES               ! Flag, 1=seed from clock, 2=no 
   INTEGER (KIND=IK) :: KSeed           = 0                 ! # words needed to store a seed (<=MAXK)
   INTEGER (KIND=IK) :: PSeedInc        = 101               ! Offset for main profile seeds
   INTEGER (KIND=IK) :: SeedInc         = 157               ! Offset for individual seeds
   INTEGER (KIND=IK) :: ISdAct2(MAXK)   = 0					! Random seed for activity diary selection in clustering
   INTEGER (KIND=IK) :: ISeed(MAXK)     = 0                 ! Random seed for profile vars
   INTEGER (KIND=IK) :: ISdAct(MAXK)    = 0                 ! Random seed for activity diary selection 
   INTEGER (KIND=IK) :: ISdMETS(MAXK)   = 0                 ! Random seed for METS distributions
   INTEGER (KIND=IK) :: ISdDay(MAXK)    = 0                 ! Random seed for daily varying values
   INTEGER (KIND=IK) :: ISdVent(MAXK)   = 0                 ! Random seed for ventilation routine
   INTEGER (KIND=IK) :: ISdMic(MAXK)    = 0                 ! Random seed for micro variables
   INTEGER (KIND=IK) :: ISdPhys(MAXK)    = 0                ! Random seed for physiology variables
   INTEGER (KIND=IK), ALLOCATABLE :: MPSeeds(:)             ! Random seeds for microparameters
      
   ! Variables set in SpaceTimeModule
   TYPE (CALENDAR),  TARGET,ALLOCATABLE :: Calen(:)         ! Daily-varying profile-indep. variables
   TYPE (SITE),      TARGET,ALLOCATABLE :: SList(:)         ! List of population sector descriptions
   TYPE (SITE),      TARGET,ALLOCATABLE :: DList(:)         ! List of air district descriptions
   TYPE (SITE),      TARGET,ALLOCATABLE :: ZList(:)         ! List of temperature zone descriptions
   TYPE (TEMPERDAY), TARGET,ALLOCATABLE :: TemData(:,:)     ! Temperature data by day and zone
   TYPE (HOURMETEOR),TARGET,ALLOCATABLE :: HourMtr(:,:)     ! Meteor. data by hour and zone
   REAL (KIND=RK),   TARGET,ALLOCATABLE :: AQData(:,:,:)    ! Air quality data by hour and district
   TYPE (AQCONDS),   TARGET,ALLOCATABLE :: AQCond(:,:,:)    ! Air quality conditionals by hour, district
   REAL (KIND=RK),   TARGET,ALLOCATABLE :: NearbyS(:,:)     ! Nearby tracts, for random sample loc
 
   ! Variables set in ProfileModule
   TYPE (CONDVAR)                          :: CVList(MAXCV) ! List of potential conditional variables
   TYPE (PROFILE),      TARGET,ALLOCATABLE :: Person(:)     ! Vector of generated personal profiles
   TYPE (PHYSIOLOGY),   TARGET,ALLOCATABLE :: Phys(:)       ! Physiology data for all profiles
   TYPE (PROFILEDAY),   TARGET,ALLOCATABLE :: PersonDay(:)  ! Daily-varying data for current profile 
   TYPE (EVENTSEQUENCE),TARGET,ALLOCATABLE :: EventSeq(:)   ! Event sequence for current profile
   TYPE (HOURSEQUENCE), TARGET,ALLOCATABLE :: HourSeq(:)    ! Hourly sequence for current profile
   TYPE (TIMESTEPSEQUENCE), TARGET,ALLOCATABLE :: TSSeq(:)  ! Timestep sequence for current profile
   TYPE (EXPDAY),       TARGET,ALLOCATABLE :: DailyExp(:)   ! Daily exposures
   TYPE (DOSEDAY),      TARGET,ALLOCATABLE :: DailyDose(:)  ! Daily doses 
   CHARACTER(LEN=LEN1), ALLOCATABLE        :: GroupNames(:) ! Profile factor group names 

   ! Variables set in MicroEnvModule
   REAL (KIND=RK),     ALLOCATABLE :: MicroConc(:,:,:)      ! Concentrations (hour,micro,district#)
   CHARACTER(LEN=LEN2),ALLOCATABLE :: MicroName(:)          ! User supplied name for each micro
   INTEGER  (KIND=IK)  :: LineNumM = 0                      ! Counter for Micros line number
   INTEGER  (KIND=IK)  :: MEFlag = 0                        ! Flag indicates whether or not to count lines
   INTEGER  (KIND=IK)  :: UseNHLocs = NO                    ! Flag indicating if near home locs used
   INTEGER  (KIND=IK)  :: UseNWLocs = NO                    ! Flag indicating if near work locs used
   INTEGER (KIND=IK),  ALLOCATABLE :: NWLocs(:)             ! Random near work locs
   INTEGER (KIND=IK),  ALLOCATABLE :: NHLocs(:)             ! Random near home locs   
      
   ! Levels for Output Tables - Set in ReadParameters
   REAL (KIND=RK), ALLOCATABLE :: PctlGrp(:)                ! Percentiles for population groups
   REAL (KIND=RK), ALLOCATABLE :: Pctl(:,:)                 ! Percentiles for use on output tables
   REAL (KIND=RK), ALLOCATABLE :: DM1HExpLev(:,:)           ! Levels for daily max 1-hour exposure
   REAL (KIND=RK), ALLOCATABLE :: DM8HExpLev(:,:)           ! Levels for daily max 8-hour exposure
   REAL (KIND=RK), ALLOCATABLE :: FEV1RiskLev(:,:)          ! Levels for FEV1Risk probabilities
   REAL (KIND=RK), ALLOCATABLE :: DAvgExpLev(:,:)           ! Levels for daily average exposure
   REAL (KIND=RK), ALLOCATABLE :: SAvgExpLev(:,:)           ! Levels for annual average exposure
   REAL (KIND=RK), ALLOCATABLE :: TimeExpLev(:,:)           ! Levels for exposure duration table
   REAL (KIND=RK), ALLOCATABLE :: DMEHDoseLev(:,:)          ! Levels for daily max end-of-hour dose
   REAL (KIND=RK), ALLOCATABLE :: DM1HDoseLev(:,:)          ! Levels for daily max 1-hour dose
   REAL (KIND=RK), ALLOCATABLE :: DM8HDoseLev(:,:)          ! Levels for daily max 8-hour dose
   REAL (KIND=RK), ALLOCATABLE :: DAvgDoseLev(:,:)          ! Levels for daily average dose
   REAL (KIND=RK), ALLOCATABLE :: SAvgDoseLev(:,:)          ! Levels for annual average dose
   REAL (KIND=RK), ALLOCATABLE :: TimeDoseLev(:,:)          ! Levels for dose duration table
   REAL (KIND=RK), ALLOCATABLE :: H_EHDoseLev(:,:)          ! Levels for end-of-hour dose
   REAL (KIND=RK), ALLOCATABLE :: DMTSExpLev(:,:)           ! Levels for daily max timestep exp
   REAL (KIND=RK), ALLOCATABLE :: DMTSDoseLev(:,:)          ! Levels for daily max timestep dose
   REAL (KIND=RK), ALLOCATABLE :: TSExpLev(:,:)             ! Levels for timestep exp
   REAL (KIND=RK), ALLOCATABLE :: TSDoseLev(:,:)            ! Levels for timestep dose
   REAL (KIND=RK), ALLOCATABLE :: FEVProbList(:,:)          ! Probabilities for FEV1Risk

   ! Group Populations - Set in Exposure
   INTEGER (KIND=IK) :: NChild       = 0                    ! Total # children
   INTEGER (KIND=IK) :: NActive      = 0                    ! Total # actives
   INTEGER (KIND=IK) :: NActiveChild = 0                    ! Total # active children
   INTEGER (KIND=IK) :: NIll         = 0                    ! Total # ill 
   INTEGER (KIND=IK) :: NIllChild    = 0                    ! Total # ill children
   INTEGER (KIND=IK) :: NEmployed    = 0                    ! Total # employed people 

   ! Micro, Exposure, and Dose Table Info - Set in ExposureDose
   TYPE :: MICROSUMMARY                                     ! Stores profile sums by micro 
      INTEGER (KIND=IK)   :: Minutes   = 0                  ! # minutes spent in each micro 
      REAL    (KIND=LONG) :: MeanConc  = 0.0                ! Mean exposure in each micro
      REAL    (KIND=RK)   :: MaxConc   = 0.0                ! Max. exposure in each micro
   END TYPE MICROSUMMARY                                    ! Used to declare MSum

   TYPE(MICROSUMMARY),ALLOCATABLE :: MSum(:)                ! Summary by micro
   INTEGER (KIND=IK), ALLOCATABLE :: DAvgExp(:,:,:)         ! Daily average exposure table
   INTEGER (KIND=IK), ALLOCATABLE :: DM1HExp(:,:,:,:)       ! Daily max 1-hour exposure tables
   INTEGER (KIND=IK), ALLOCATABLE :: DM8HExp(:,:,:,:)       ! Daily max 8-hour exposure tables
   INTEGER (KIND=IK), ALLOCATABLE :: DMTSExp(:,:,:,:)       ! Daily max timestep exposure tables
   INTEGER (KIND=IK), ALLOCATABLE :: ALLTSExp(:,:,:,:)      ! All-sim timestep exposure table
   INTEGER (KIND=IK), ALLOCATABLE :: SAvgExp(:,:,:)         ! Simulation avg exposure table
   INTEGER (KIND=IK), ALLOCATABLE :: DAvgDose(:,:,:)        ! Daily average dose table
   INTEGER (KIND=IK), ALLOCATABLE :: DM1HDose(:,:,:,:)      ! Daily max 1-hour dose table
   INTEGER (KIND=IK), ALLOCATABLE :: DM8HDose(:,:,:,:)      ! Daily max 8-hour dose table
   INTEGER (KIND=IK), ALLOCATABLE :: DMTSDose(:,:,:,:)      ! Daily max timestep dose table
   INTEGER (KIND=IK), ALLOCATABLE :: ALLTSDose(:,:,:,:)     ! All-sim timestep dose table
   INTEGER (KIND=IK), ALLOCATABLE :: DMEHDose(:,:,:,:)      ! Daily max end-of-hour dose table
   INTEGER (KIND=IK), ALLOCATABLE :: H_EHDose(:,:,:)        ! Hourly end-of-hour dose table
   INTEGER (KIND=IK), ALLOCATABLE :: SAvgDose(:,:,:)        ! Simulation average dose table
   INTEGER (KIND=IK), ALLOCATABLE :: HourTime(:,:)          ! Hourly time spent in micros 
   INTEGER (KIND=RK), ALLOCATABLE :: TSMultiLev(:,:)        ! Levels for multiple ts exp/doses
   REAL  (KIND=RK2),  ALLOCATABLE :: TimeDose(:,:,:)        ! Minutes per dose level table
   REAL  (KIND=RK),   ALLOCATABLE :: TimeExp(:,:,:)         ! Minutes per exposure level table
   REAL  (KIND=RK2),  ALLOCATABLE :: MaxExp(:,:)            ! Personal max exposure by pollutant
   REAL  (KIND=RK2),  ALLOCATABLE :: AvgExp(:,:)            ! Personal avg exposure by pollutant
   REAL  (KIND=RK2),  ALLOCATABLE :: MaxDose(:,:)           ! Personal max dose by pollutant
   REAL  (KIND=RK2),  ALLOCATABLE :: AvgDose(:,:)           ! Personal avg dose by pollutant 
   REAL  (KIND=RK2),  ALLOCATABLE :: EventExp(:,:)          ! Event exposure by pollutant
   REAL  (KIND=RK2),  ALLOCATABLE :: EventAmb(:,:)          ! Event ambient by pollutant
   REAL  (KIND=RK2),  ALLOCATABLE :: EventAmbHome(:,:)      ! Event ambient in home loc by pollutant   
   REAL  (KIND=RK2),  ALLOCATABLE :: EventDose(:,:)         ! Event dose by pollutant
   REAL  (KIND=RK2),  ALLOCATABLE :: EventInDose(:,:)       ! Event intake dose by pollutant
   REAL  (KIND=RK2),  ALLOCATABLE :: EventDepDose(:,:)      ! Event deposited dose by pollutant
   REAL  (KIND=RK2),  ALLOCATABLE :: EventFDose(:,:)        ! Event final dose by pollutant 
   REAL  (KIND=RK2),  ALLOCATABLE :: HourExp(:,:)           ! HourExp by pollutant 
   REAL  (KIND=RK2),  ALLOCATABLE :: HourDose(:,:)          ! Hourly dose by pollutant 
   REAL  (KIND=RK2),  ALLOCATABLE :: HourDep(:,:)           ! Hourly intake dose by pollutant 
   REAL  (KIND=RK2),  ALLOCATABLE :: HourIntake(:,:)        ! Hour deposited dose by pollutant 
   REAL  (KIND=RK2),  ALLOCATABLE :: HourEF(:,:)            ! Hour exposure factor by pollutant 
   REAL  (KIND=RK2),  ALLOCATABLE :: HourHomeEF(:,:)        ! Hour exposure factor by pollutant    
   REAL  (KIND=RK2),  ALLOCATABLE :: HourAmb(:,:)           ! Hour ambient conc by pollutant 
   REAL  (KIND=RK2),  ALLOCATABLE :: HourHomeAmb(:,:)       ! Hour home ambient conc by pollutant    
   REAL  (KIND=RK2),  ALLOCATABLE :: HourConc(:,:,:)        ! Hour micro conc by pollutant 
   REAL  (KIND=RK2),  ALLOCATABLE :: DayHomeConc(:,:)       ! Daily home concentration
   REAL  (KIND=RK2),  ALLOCATABLE :: DayAvgExp(:,:)         ! Day avg exp by pollutant 
   REAL  (KIND=RK2),  ALLOCATABLE :: DayMax1Exp(:,:)        ! Day max 1 hour exp by pollutant 
   REAL  (KIND=RK2),  ALLOCATABLE :: DayMax8Exp(:,:)        ! Day max 8 hour exp by pollutant 
   REAL  (KIND=RK2),  ALLOCATABLE :: DayAvgDose(:,:)        ! Day avg dose by pollutant 
   REAL  (KIND=RK2),  ALLOCATABLE :: DayDep(:,:)            ! Day total deposited dose by pollutant 
   REAL  (KIND=RK2),  ALLOCATABLE :: DayIntake(:,:)         ! Day avg intake dose by pollutant 
   REAL  (KIND=RK2),  ALLOCATABLE :: DayMax1Dose(:,:)       ! Day max 1 hour dose by pollutant 
   REAL  (KIND=RK2),  ALLOCATABLE :: DayMax8Dose(:,:)       ! Day max 8 hour dose by pollutant 
   REAL  (KIND=RK2),  ALLOCATABLE :: DayMax8EC(:,:)         ! Day max 8-hour E/C ratio   
   REAL  (KIND=RK2),  ALLOCATABLE :: DayMax1FDose(:,:)      ! Day max end of hour dose by pollutant 
   REAL  (KIND=RK2),  ALLOCATABLE :: DayMaxTSExp(:,:)       ! Day max TS exposure by pollutant 
   REAL  (KIND=RK2),  ALLOCATABLE :: DayMaxTSDose(:,:)      ! Day max TS dose by pollutant 
   REAL  (KIND=RK2),  ALLOCATABLE :: TSExp(:,:)             ! TS exp by pollutant 
   REAL  (KIND=RK2),  ALLOCATABLE :: TSDose(:,:)            ! TS dose by pollutant 
   REAL  (KIND=RK2),  ALLOCATABLE :: TSInDose(:,:)          ! TS intake dose by pollutant 
   REAL  (KIND=RK2),  ALLOCATABLE :: TSDepDose(:,:)         ! TS deposited dose by pollutant     
   REAL  (KIND=RK2),  ALLOCATABLE :: TSEF(:,:)              ! TS exposure factor by pollutant 
   REAL  (KIND=RK2),  ALLOCATABLE :: TSAmb(:,:)             ! TS ambient AQ by pollutant 
END MODULE GlobalModule                                     ! End of GlobalModule source code 
 

  
